#include "Columns.h"
#include "Utils.h"

ColumnInfo ColInfo[NUM_COLUMNS] = {
{" ", "Image Name"},
{" ", "PID"},
{" ", "CPU"},
{" ", "CPU Time"},
{"   ", "Creation Time"},
{" ", "Mem Usage"},
{" ", "VM Size"},
{" ", "Priority"},
{" ", "Threads"},
{"  USER", "USER Objects"},
{"  GDI", "GDI Objects"},
{" ", "Handles"},
{"  ", "I/O Read Bytes"},
{"  ", "I/O Write Bytes"},
{" ", "User Name"},
{"  ", "Image Path"}};

bool IsNumberCol[NUM_COLUMNS] = {0,1,0,0,0,1,1,0,1,1,1,1,1,1,0,0};

void FormatProcessData(TMyProcess *p, int Column, char *buf)
{
    lstrcpy(buf, "");
    switch (Column)
    {
        case 0:
            lstrcpy(buf, p->ProcessName);
            break;
        case 1:
			wsprintf(buf, "%4lu (0x%04X)", p->pid, p->pid);
            break;
        case 2:
            wsprintf(buf, "%02d", p->CPUUse / 100);
            break;
        case 3:
        {
            int t = ((p->KernelTime.QuadPart + p->UserTime.QuadPart) / 10000000) % 86400;
            wsprintf(buf, "%02d:%02d:%02d", (t / 3600), (t % 3600) / 60, t % 60);
            break;
        }
        case 4:
        {
            if (p->CreateTime.wYear < 1900)
                lstrcpy(buf, "N/A");
            else
                wsprintf(buf, "%02d:%02d:%02d %02d.%02d.%d",
                    p->CreateTime.wHour,
                    p->CreateTime.wMinute,
                    p->CreateTime.wSecond,
                    p->CreateTime.wDay,
                    p->CreateTime.wMonth,
                    p->CreateTime.wYear);
            break;
        }
        case 5:
            FormatSize(buf, p->VmCounters.WorkingSetSize);
            break;
        case 6:
            FormatSize(buf, p->VmCounters.PagefileUsage);
            break;
        case 7:
            switch (p->BasePriority)
            {
                case ABOVE_NORMAL_PRIORITY_CLASS: lstrcpy(buf, "Above Normal"); break;
                case BELOW_NORMAL_PRIORITY_CLASS: lstrcpy(buf, "Below Normal"); break;
                case HIGH_PRIORITY_CLASS: lstrcpy(buf, "High"); break;
                case IDLE_PRIORITY_CLASS: lstrcpy(buf, "Low"); break;
                case NORMAL_PRIORITY_CLASS: lstrcpy(buf, "Normal"); break;
                case REALTIME_PRIORITY_CLASS: lstrcpy(buf, "Realtime"); break;
                default: lstrcpy(buf, "N/A");
            }
            break;
        case 8:
            wsprintf(buf, "%lu", p->ThreadCount);
            break;
        case 9:
            wsprintf(buf, "%lu", p->USERObjects);
            break;
        case 10:
            wsprintf(buf, "%lu", p->GDIObjects);
            break;
        case 11:
            wsprintf(buf, "%lu", p->HandleCount);
            break;
        case 12:
            FormatSize2(buf, p->IoCounters.ReadTransferCount);
            break;
        case 13:
            FormatSize2(buf, p->IoCounters.WriteTransferCount);
            break;
        case 14:
            lstrcpy(buf, p->UserName);
            break;
        case 15:
            lstrcpy(buf, p->ImagePath);
            break;
    }
}

void FormatThreadData(PSYSTEM_THREADS thread, int Column, char *buf)
{
    lstrcpy(buf, "");
    switch (Column)
    {
        case 0:
			wsprintf(
				buf,
				"%4lu (0x%04X)",
				thread->ClientId.UniqueThread,
				thread->ClientId.UniqueThread);
            break;
        case 1:
        {
            int t = (thread->KernelTime.QuadPart / 10000000) % 86400;
            wsprintf(buf, "%02d:%02d:%02d", (t / 3600), (t % 3600) / 60, t % 60);
            break;
        }
        case 2:
        {
            int t = (thread->UserTime.QuadPart / 10000000) % 86400;
            wsprintf(buf, "%02d:%02d:%02d", (t / 3600), (t % 3600) / 60, t % 60);
            break;
        }
        case 3:
        {
            FILETIME lt;
            SYSTEMTIME st;
            FileTimeToLocalFileTime((FILETIME *)&thread->CreateTime, &lt);
            FileTimeToSystemTime(&lt, &st);
            if (st.wYear < 1900)
                lstrcpy(buf, "N/A");
            else
                wsprintf(buf, "%02d:%02d:%02d %02d.%02d.%d",
                    st.wHour, st.wMinute, st.wSecond,
                    st.wDay, st.wMonth, st.wYear);
            break;
        }
        case 4:
        {
            int t = (thread->WaitTime / 1000) % 86400;
            wsprintf(buf, "%02d:%02d:%02d", (t / 3600), (t % 3600) / 60, t % 60);
            break;
        }
        case 5:
            wsprintf(buf, "0x%08X", thread->StartAddress);
            break;
        case 6:
            wsprintf(buf, "%lu", thread->Priority);
            break;
        case 7:
            wsprintf(buf, "%lu", thread->BasePriority);
            break;
        case 8:
            wsprintf(buf, "%lu", thread->ContextSwitchCount);
            break;
        case 9:
            wsprintf(buf, "%lu", thread->State);
            break;
/*            switch (thread->State)
            {
                case : lstrcpy(buf, ""); break;
                default: lstrcpy(buf, "N/A");
            }
            break;*/
        case 10:
            wsprintf(buf, "%lu", thread->WaitReason);
            break;
    }
}

