#ifndef __DEBUG__
#define __DEBUG__

//includes
#include <Dbghelp.h>
#include <Tlhelp32.h>

//pragmas
#pragma warning (push) 
#pragma warning (disable : 4290) 

//defines
#define MEMDUMP_ADDRESS	1
#define MEMDUMP_TEXT	2

#ifndef max
#define max(x,y) ((x > y) ? x : y)
#endif

//typedefs
typedef	const IMAGE_SECTION_HEADER*		PCIMAGE_SECTION_HEADER;

namespace DebugSys
{

class	CSymbolInfo;
class	CLineInfo;


PVOID __cdecl	GetEIP();
PVOID __cdecl	GetESP();
PVOID __cdecl	GetEBP();

PCSTR			ExceptionCodeToString(DWORD dwExceptionCode);
PSTR			DumpBasicRegisters(PSTR pBuffer, PCONTEXT lpEC);

MODULEENTRY32	GetModuleInfoByAddress(PVOID pvAddress, DWORD dwPID);
BOOL			SymLoadModuleByAddr(DWORD dwAddress,  DWORD dwPID, HANDLE hProcess);
PSTR			GetModuleList(PSTR pStr, DWORD dwPID);
PSTR			AddressInfo(PSTR pBuff,PVOID pvAddr, DWORD dwPID, HANDLE hProcess);


template <class T>
BOOL	IsBadReadPointer(T* p);

template <class T>
BOOL	IsBadWritePointer(T* p);

template <class T>
PSTR	WriteMemoryDump(PSTR pStr,PVOID pDump, int iSize, DWORD dwFlags = MEMDUMP_ADDRESS | MEMDUMP_TEXT);

template <class T>
PSTR	DumpStruct(PSTR pStr,PVOID pDump,int iSize);

template <class T>
PSTR	WriteHexDumpRow(PSTR pStr,PVOID pDump, int iSize);

template <class T>
PSTR	WriteTextDumpRow(PSTR pStr,PVOID pDump, int iSize);

void	MailBug(PCSTR pEMail,PCSTR pSubj,PCSTR pText);


//===========================================================================
//======================CSymbolInfo declaration==============================

class	CSymbolInfo
{
private:
	DWORD	dwAddr;
	char	sName[255];
	DWORD	dwSize;
	DWORD	dwOffset;
	BOOL	bOk;

protected:
	BOOL	Init(DWORD dwAddress,DWORD dwPID, HANDLE hProcess);

public:
	inline	CSymbolInfo(DWORD dwAddress,DWORD dwPID, HANDLE hProcess);
	inline	CSymbolInfo(PVOID pvAddress,DWORD dwPID, HANDLE hProcess);
	inline	~CSymbolInfo(){};

	inline	DWORD	GetAddress()	const	{ return dwAddr;	}
	inline	PCSTR	GetName()		const	{ return sName;		}
	inline	DWORD	GetOffset()		const	{ return dwOffset;	}
	inline	DWORD	GetSize()		const	{ return dwSize;	}

	inline	BOOL	operator!()		const	{ return !bOk;		}
	inline	operator BOOL()			const	{ return bOk;		}

	__declspec(property(get=GetAddress))	DWORD	Address;
	__declspec(property(get=GetName))		PCSTR	Name;
	__declspec(property(get=GetOffset))		DWORD	Offset;
	__declspec(property(get=GetSize))		DWORD	Size;
};
//=====================end CSymbolInfo declaration===========================
//===========================================================================



//===========================================================================
//=======================CLineInfo declaration===============================
class	CLineInfo
{
private:
	DWORD	dwAddr;
	char	sFile[255];
	DWORD	dwLine;
	DWORD	dwOffset;
	BOOL	bOk;

protected:
	BOOL	Init(DWORD dwAddress, DWORD dwPID, HANDLE hProcess);

public:
	inline	CLineInfo(DWORD dwAddress, DWORD dwPID, HANDLE hProcess);
	inline	CLineInfo(PVOID pvAddress, DWORD dwPID, HANDLE hProcess);
	inline	~CLineInfo(){};

	inline	DWORD	GetAddress()	const	{ return dwAddr;	}
	inline	PCSTR	GetFile()		const	{ return sFile;		}
	inline	DWORD	GetLineNumber()	const	{ return dwLine;	}
	inline	DWORD	GetOffset()		const	{ return dwOffset;	}

	inline	BOOL	operator!()		const	{ return !bOk;		}
	inline	operator BOOL()			const	{ return bOk;		}

	__declspec(property(get=GetAddress))	DWORD	Address;
	__declspec(property(get=GetFile))		PCSTR	File;
	__declspec(property(get=GetLineNumber))	DWORD	LineNumber;
	__declspec(property(get=GetOffset))		DWORD	Offset;
};

//=====================end CLineInfo declaration=============================
//===========================================================================

//===========================================================================
//========================CModuleInfo declaration============================
class	CModuleInfo
{
private:
	DWORD	m_dwAddr;
	DWORD	m_dwImageBase;
	DWORD	m_dwImageSize;
	char	m_sModuleName[255];
	char	m_sImageName[255];
	char	m_sFileName[255];
	BOOL	m_bOk;

protected:
	BOOL	Init(DWORD dwAddress, DWORD dwPID, HANDLE hProcess);

public:
	inline	CModuleInfo(DWORD dwAddress, DWORD dwPID, HANDLE hProcess);
	inline	CModuleInfo(PVOID pvAddress, DWORD dwPID, HANDLE hProcess);
	inline	~CModuleInfo(){}

	inline	BOOL	operator!()		const	{ return !m_bOk;	}
	inline	operator BOOL()			const	{ return m_bOk;		}

	inline	DWORD	GetAddress()	const	{ return m_dwAddr;				}
	inline	PCSTR	GetModuleName()	const	{ return m_sModuleName;			}
	inline	PVOID	GetImageBase()	const	{ return (PVOID)m_dwImageBase;	}
	inline	PCSTR	GetImageName()	const	{ return m_sImageName;			}
	inline	DWORD	GetImageSize()	const	{ return m_dwImageSize;			}
	inline	PCSTR	GetFileName()	const	{ return m_sFileName;			}

	__declspec(property(get=GetAddress))	DWORD	Address;
	__declspec(property(get=GetModuleName))	PCSTR	ModuleName;
	__declspec(property(get=GetImageBase))	PVOID	ImageBase;
	__declspec(property(get=GetImageSize))	DWORD	ImageSize;
	__declspec(property(get=GetImageName))	PCSTR	ImageName;
	__declspec(property(get=GetFileName))	PCSTR	FileName;
	
	//sections
	inline  PCIMAGE_SECTION_HEADER	GetSection(int iSection) const;
	inline  int						GetSectionCount()		 const;

	__declspec(property(get=GetSection))		PCIMAGE_SECTION_HEADER	Section[];
	__declspec(property(get=GetSectionCount))	int						SectionCount;

	//section by address
	inline  PCIMAGE_SECTION_HEADER	GetSectionByAddress(PVOID pvAddr) const;
};
//========================end CModuleInfo declaration========================
//===========================================================================



//===========================================================================
//======================CSymbolInfo implementation===========================

//CSymbolInfo(DWORD dwAddress, DWORD dwPID, HANDLE hProcess)
inline	CSymbolInfo::CSymbolInfo(DWORD dwAddress, DWORD dwPID, HANDLE hProcess)
	:dwAddr(0), dwSize(0), dwOffset(0),bOk(FALSE)
{
	sName[0] = 0;
	bOk = Init(dwAddress, dwPID, hProcess);
}

//CSymbolInfo(PVOID pvAddress, DWORD dwPID, HANDLE hProcess)
inline	CSymbolInfo::CSymbolInfo(PVOID pvAddress, DWORD dwPID, HANDLE hProcess)
	:dwAddr(0), dwSize(0), dwOffset(0),bOk(FALSE)
{
	sName[0] = 0;
	bOk = Init((DWORD)pvAddress, dwPID, hProcess);
}

//=====================end CSymbolInfo implementation========================
//===========================================================================





//===========================================================================
//========================CLineInfo implemetation============================

//CLineInfo(DWORD dwAddress, DWORD dwPID, HANDLE hProcess)
inline	CLineInfo::CLineInfo(DWORD dwAddress, DWORD dwPID, HANDLE hProcess)
	:dwAddr(0),dwLine(0),dwOffset(0),bOk(FALSE)
{
	sFile[0] = 0;
	bOk = Init(dwAddress, dwPID, hProcess);
}

//CLineInfo(PVOID pvAddress, DWORD dwPID, HANDLE hProcess)
inline	CLineInfo::CLineInfo(PVOID pvAddress, DWORD dwPID, HANDLE hProcess)
	:dwAddr(0),dwLine(0),dwOffset(0),bOk(FALSE)
{
	sFile[0] = 0;
	bOk = Init((DWORD)pvAddress, dwPID, hProcess);
}

//======================end CLineInfo implemetation==========================
//===========================================================================

//===========================================================================
//========================CModuleInfo implemetation==========================

inline	CModuleInfo::CModuleInfo(DWORD dwAddress, DWORD dwPID, HANDLE hProcess)
	:m_dwAddr(0), m_dwImageBase(0), m_dwImageSize(0), m_bOk(FALSE)
{
	m_sModuleName[0]= 0;
	m_sImageName[0]	= 0;
	m_sFileName[0]	= 0;

	m_bOk = Init(dwAddress, dwPID, hProcess);
}

inline	CModuleInfo::CModuleInfo(PVOID pvAddress, DWORD dwPID, HANDLE hProcess)
	:m_dwAddr(0), m_dwImageBase(0), m_dwImageSize(0), m_bOk(FALSE)
{
	m_sModuleName[0]= 0;
	m_sImageName[0]	= 0;
	m_sFileName[0]	= 0;

	m_bOk = Init((DWORD)pvAddress, dwPID, hProcess);
}

inline  PCIMAGE_SECTION_HEADER	CModuleInfo::GetSection(int iSection) const
{
	if (!m_bOk)
		return NULL;

	PIMAGE_DOS_HEADER	pDosHdr	= (PIMAGE_DOS_HEADER)m_dwImageBase;
	PIMAGE_NT_HEADERS	pNtHdr	= (PIMAGE_NT_HEADERS)(m_dwImageBase + pDosHdr->e_lfanew);
	
	return (pNtHdr->FileHeader.NumberOfSections <= iSection) ?  NULL
			:IMAGE_FIRST_SECTION( pNtHdr ) + iSection;
}

inline  int	CModuleInfo::GetSectionCount() const
{
	if (!m_bOk)
		return NULL;

	PIMAGE_DOS_HEADER	pDosHdr	= (PIMAGE_DOS_HEADER)m_dwImageBase;
	PIMAGE_NT_HEADERS	pNtHdr	= (PIMAGE_NT_HEADERS)(m_dwImageBase + pDosHdr->e_lfanew);
	return pNtHdr->FileHeader.NumberOfSections;
}

inline  PCIMAGE_SECTION_HEADER	CModuleInfo::GetSectionByAddress(PVOID pvAddr) const
{
	if (!m_bOk)
		return NULL;
	
	if ((m_dwImageBase < (DWORD)pvAddr) || (m_dwImageBase + m_dwImageSize >= (DWORD)pvAddr))
		return NULL;

	DWORD	dwRVA = (DWORD)pvAddr - m_dwImageBase;
	for(int i=0 ; i<SectionCount; i++)
	{
		PCIMAGE_SECTION_HEADER	pSection		= Section[i];
		DWORD					dwSectionStart	= Section[i]->VirtualAddress;
		DWORD					dwSectionEnd	= dwSectionStart
												+ max(pSection->SizeOfRawData, pSection->Misc.VirtualSize);

		// Is the address in this section???
		if ( (dwRVA >= dwSectionStart) && (dwRVA <= dwSectionEnd) )
			return pSection;
	}
	return NULL;
}
//======================end CModuleInfo implemetation========================
//===========================================================================

template <class T>
class CRemotePtr
{
private:
	const	T*			m_ptr;
	mutable	T			m_Data;
	mutable	HANDLE		m_hProcess;

protected:
	BOOL		ReadMemory() const
	{
		DWORD	dwRead = 0;
		ZeroMemory(&m_Data,sizeof(T));
		return ::ReadProcessMemory(m_hProcess, m_ptr, &m_Data, sizeof(T), &dwRead)
			&& (dwRead == sizeof(T));
	}


public:
	CRemotePtr(HANDLE hProcess,PVOID ptr)
		:m_hProcess(hProcess),
		m_ptr(reinterpret_cast<T*>(ptr)),
		m_Data()
	{
	}

	//copy constructor
	CRemotePtr(const CRemotePtr& other)
		:m_hProcess(other.m_hProcess),
		m_ptr(other.m_ptr),
		m_Data()
	{
	}

	//template constructor
	template <class U>
		CRemotePtr(const CRemotePtr<U>& other)
		:m_hProcess(other.m_hProcess),
		m_ptr(other.m_ptr),
		m_Data()
	{
	}

	//template operator=
	template <class U>
		CRemotePtr&	operator=(const CRemotePtr<U>& other)
	{
		m_hProcess	= other.m_hProcess;
		m_ptr		= other.m_ptr;
	}

	CRemotePtr&	operator=(const T*	pPtr)
	{
		m_ptr = pPtr;
		return *this;
	}

	bool		operator!() const
	{
		return !ReadMemory();
	}

	const T*	operator->() const				{ ReadMemory(); return &m_Data; }

	CRemotePtr&	operator+=	(int i)				{ m_ptr += i;   return *this;	}
	CRemotePtr&	operator-=	(int i)				{ m_ptr -= i;   return *this;	}
	CRemotePtr	operator+	(int i) const		{ CRemotePtr p(*this); p+=i; return p;	}
	CRemotePtr	operator-	(int i) const		{ CRemotePtr p(*this); p-=i; return p;	}
	CRemotePtr  operator++	()					{ CRemotePtr p(*this); ++(*this); return p;	}
	CRemotePtr& operator++	(int)				{ *this += 1; return *this;					}
	CRemotePtr  operator--	()					{ CRemotePtr p(*this); --(*this); return p;	}
	CRemotePtr& operator--	(int)				{ *this -= 1; return *this;					}

	operator T*() const							{ ReadMemory(); return &m_Data;	}
};


template <class T>
BOOL IsBadReadPointer(T* p)
{
	return IsBadReadPtr(p,sizeof(T));
}
template <class T>
BOOL IsBadWritePointer(T* p)
{
	return IsBadWritePtr(p,sizeof(T));
}

inline
BOOL IsBadRemoteStringPtrA(HANDLE hProcess,PCSTR pStr,int chMax)
{
	char	c;
	DWORD	dwRead;
	for(int i=0; i<chMax; i++)
	{
		if (!::ReadProcessMemory(hProcess,pStr+i,&c,sizeof(c),&dwRead) || (dwRead!=sizeof(c)))
			break;
		else if (c==0)
			return FALSE;
	}
	return TRUE;
}

inline
BOOL IsBadRemoteStringPtrW(HANDLE hProcess,PCWSTR pStr,int chMax)
{
	WCHAR	c;
	DWORD	dwRead;
	for(int i=0; i<chMax; i++)
	{
		if (!::ReadProcessMemory(hProcess,pStr+i,&c,sizeof(c),&dwRead) || (dwRead!=sizeof(c)))
			break;
		else if (c==0)
			return FALSE;
	}
	return TRUE;
}

inline
BOOL IsBadRemoteReadPtr(HANDLE hProcess,PVOID pV,int iBytes)
{
	char	buff[64];
	DWORD	dwRead;
	int		iBytesLeft = iBytes;
	int		iToRead;
	BYTE*	pCurrPtr = (BYTE*)pV;

	while (iBytesLeft)
	{
		if (iBytesLeft/sizeof(buff))
		{
			iToRead		= sizeof(buff);
			iBytesLeft -= sizeof(buff);
		}
		else
		{
			iToRead		= iBytesLeft;
			iBytesLeft	= 0;
		}
		
		if (!::ReadProcessMemory(hProcess,pCurrPtr,buff,iToRead,&dwRead) || (dwRead!=iToRead))
			return TRUE;
		
		pCurrPtr+=iToRead;
	}
	return FALSE;
}



};

#define CLASS_EXCEPTION_HANDLER( _comment_ )						\
			CExceptionHandler::ExceptionHandler(					\
				GetExceptionInformation(),							\
				GetCurrentThreadId(),								\
				__FILE__,											\
				__LINE__,											\
				__FUNCTION__,										\
				(ConversionTo<CExceptionHandler*>::From(this))		\
					? reinterpret_cast<CExceptionHandler*>(this)	\
					: NULL,											\
				CExceptionHandler::IsBadThisPtr(this)				\
					? "Bad this"_comment_							\
					: _comment_										\
												)					

#define EXTERN_EXCEPTION_HANDLER( _comment_ )						\
			CExceptionHandler::ExceptionHandler(					\
				GetExceptionInformation(),							\
				GetCurrentThreadId(),								\
				__FILE__,											\
				__LINE__,											\
				__FUNCTION__,										\
				NULL,												\
				_comment_											\
												)


#pragma warning (pop)
#endif