#include "EditSecurity.h"

#include "Procs.h"

AccessRight PrcAccessRights[NUM_PROCESS_RIGHTS] = {
{L"All Access", PROCESS_ALL_ACCESS},
{L"Standard Rights", STANDARD_RIGHTS_REQUIRED},
{L"Terminate", PROCESS_TERMINATE},
{L"Create Thread", PROCESS_CREATE_THREAD},
{L"Set Session ID", PROCESS_SET_SESSIONID},
{L"VM Operation", PROCESS_VM_OPERATION},
{L"VM Read", PROCESS_VM_READ},
{L"VM Write", PROCESS_VM_WRITE},
{L"Duplicate Handle", PROCESS_DUP_HANDLE},
{L"Create Process", PROCESS_CREATE_PROCESS},
{L"Set Quota", PROCESS_SET_QUOTA},
{L"Set Information", PROCESS_SET_INFORMATION},
{L"Query Information", PROCESS_QUERY_INFORMATION},
{L"Suspend & Resume", PROCESS_SUSPEND_RESUME},
{L"Delete", DELETE},
{L"Read Control", READ_CONTROL},
{L"Write DAC", WRITE_DAC},
{L"Write Owner", WRITE_OWNER},
{L"Synchronize", SYNCHRONIZE}};

AccessRight ThrdAccessRights[NUM_THREAD_RIGHTS] = {
{L"All Access", THREAD_ALL_ACCESS},
{L"Standard Rights", STANDARD_RIGHTS_REQUIRED},
{L"Direct Impersonation", THREAD_DIRECT_IMPERSONATION},
{L"Get Context", THREAD_GET_CONTEXT},
{L"Impersonate", THREAD_IMPERSONATE},
{L"Query Information", THREAD_QUERY_INFORMATION},
{L"Set Context", THREAD_SET_CONTEXT},
{L"Set Information", THREAD_SET_INFORMATION},
{L"Set Token", THREAD_SET_THREAD_TOKEN},
{L"Suspend & Resume", THREAD_SUSPEND_RESUME},
{L"Terminate", THREAD_TERMINATE},
{L"Delete", DELETE},
{L"Read Control", READ_CONTROL},
{L"Write DAC", WRITE_DAC},
{L"Write Owner", WRITE_OWNER},
{L"Synchronize", SYNCHRONIZE}};

TObjectSecurityInfo::TObjectSecurityInfo(HANDLE hObject, LPTSTR ObjectName,
        AccessRight *ar, int NumAccessRights)
{
    this->hObject = hObject;
    this->NumAccessRights = NumAccessRights;
    AccessRights = ar;
    m_cRef = 0;
    psd = NULL;
#ifdef UNICODE
    lstrcpy(this->ObjectName, ObjectName);
#else
    MultiByteToWideChar(CP_ACP, 0, ObjectName, -1, this->ObjectName,
        MAX_PATH);
#endif
    psi = (PSI_ACCESS)LocalAlloc(LMEM_FIXED | LMEM_ZEROINIT,
        sizeof(SI_ACCESS) * NumAccessRights);
}

TObjectSecurityInfo::~TObjectSecurityInfo()
{
    LocalFree(psi);
}

STDMETHODIMP TObjectSecurityInfo::QueryInterface(
    REFIID riid, LPVOID FAR *ppv)
{
    *ppv = NULL;

    if (IsEqualIID(riid, IID_ISecurityInformation) ||
        IsEqualIID(riid, IID_IUnknown))
    	*ppv = (ISecurityInformation *)this;

    if (*ppv)
    {
        AddRef();
        return NOERROR;
    }

	return E_NOINTERFACE;
}

STDMETHODIMP_(ULONG) TObjectSecurityInfo::AddRef()
{
    return ++m_cRef;
}

STDMETHODIMP_(ULONG) TObjectSecurityInfo::Release()
{
    if (--m_cRef) return m_cRef;
    delete this;
    return S_OK;
}

STDMETHODIMP TObjectSecurityInfo::GetObjectInformation(
    PSI_OBJECT_INFO pObjectInfo)
{
    pObjectInfo->dwFlags =
        SI_EDIT_ALL |
        SI_ADVANCED |
        SI_NO_ACL_PROTECT |
        SI_PAGE_TITLE;
    pObjectInfo->hInstance = GetModuleHandle(NULL);
    pObjectInfo->pszServerName = NULL;
    pObjectInfo->pszObjectName = ObjectName;
    pObjectInfo->pszPageTitle = L"  ";
    return S_OK;
}

STDMETHODIMP TObjectSecurityInfo::GetSecurity(
    SECURITY_INFORMATION RequestedInformation,
    PSECURITY_DESCRIPTOR *ppSecurityDescriptor,
    BOOL fDefault)
{
    DWORD nLengthNeeded;
    _GetKernelObjectSecurity(hObject, RequestedInformation,
        psd, 0, &nLengthNeeded);
    if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
    {
        psd = (PSECURITY_DESCRIPTOR)LocalAlloc(
            LMEM_FIXED | LMEM_ZEROINIT, nLengthNeeded);
        if (!_GetKernelObjectSecurity(hObject, RequestedInformation,
            psd, nLengthNeeded, &nLengthNeeded))
            return E_FAIL;
    }
    else return E_FAIL;
    *ppSecurityDescriptor = psd;
    return S_OK;
}

STDMETHODIMP TObjectSecurityInfo::SetSecurity(
    SECURITY_INFORMATION SecurityInformation,
    PSECURITY_DESCRIPTOR pSecurityDescriptor)
{
    if (!_SetKernelObjectSecurity(hObject,
        SecurityInformation, pSecurityDescriptor)) return E_FAIL;
    return S_OK;
}

STDMETHODIMP TObjectSecurityInfo::GetAccessRights(
    const GUID *pguidObjectType,
    DWORD dwFlags,
    PSI_ACCESS *ppAccess,
    ULONG *pcAccesses,
    ULONG *piDefaultAccess)
{
    for (int i = 0; i < NumAccessRights; i++)
    {
        psi[i].pguid = pguidObjectType;
        psi[i].mask = AccessRights[i].AccessMask;
        psi[i].pszName = AccessRights[i].Name;
        psi[i].dwFlags = SI_ACCESS_GENERAL | SI_ACCESS_SPECIFIC;
    }
    *ppAccess = psi;
    *pcAccesses = NumAccessRights;
    *piDefaultAccess = 0;
    return S_OK;
}

STDMETHODIMP TObjectSecurityInfo::MapGeneric(
    const GUID *pguidObjectType,
    UCHAR *pAceFlags,
    ACCESS_MASK *pMask)
{
    return S_OK;
}

STDMETHODIMP TObjectSecurityInfo::GetInheritTypes(
    PSI_INHERIT_TYPE *ppInheritTypes,
    ULONG *pcInheritTypes)
{
    return S_OK;
}

STDMETHODIMP TObjectSecurityInfo::PropertySheetPageCallback(
    HWND hwnd,
    UINT uMsg,
    SI_PAGE_TYPE uPage)
{
    return S_OK;
}

BOOL EditProcessSecurity(HWND hWnd, DWORD ProcessId, LPTSTR ProcessName)
{
    HANDLE hProcess = OpenProcess(PROCESS_ALL_ACCESS, FALSE, ProcessId);
    if (!hProcess) return FALSE;
    TObjectSecurityInfo *si = new TObjectSecurityInfo(hProcess, ProcessName,
        PrcAccessRights, NUM_PROCESS_RIGHTS);
    BOOL Res = _EditSecurity(hWnd, si);
    CloseHandle(hProcess);
    return Res;
}

BOOL EditThreadSecurity(HWND hWnd, DWORD ThreadId)
{
    HANDLE hThread = OpenThread(THREAD_ALL_ACCESS, FALSE, ThreadId);
    if (!hThread) return FALSE;
    char buf[32];
    wsprintf(buf, "Thread #%lu", ThreadId);
    TObjectSecurityInfo *si = new TObjectSecurityInfo(hThread, buf,
        ThrdAccessRights, NUM_THREAD_RIGHTS);
    BOOL Res = _EditSecurity(hWnd, si);
    CloseHandle(hThread);
    return Res;
}

