//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdio.h>
#pragma hdrstop

#include "Main.h"
#include "TColumnsDlg.h"
#include "TPrivilegeDlg.h"
#include "Columns.h"
#include "MyProcess.h"
#include "TModsForm.h"
#include "TMemForm.h"
#include "TThreadsForm.h"
#include "TSaveTableDlg.h"
#include "TVersionDlg.h"
#include "TStartupInfoDlg.h"
#include "TViewMemForm.h"
#include "TChangeWorkingSetDlg.h"
#include "TModSearchDlg.h"
#include "EditSecurity.h"
#include "Utils.h"
#include "Procs.h"
#include "Ver.h"
#include "cpuid.h"

#include "CallStack.h"
#include "Debug.h"
#include "SnapshotFrm.h"
#include "FAbout.h"
#include "FViewSrc.h"
#include <winperf.h>
#include "PerfInfo.h"
#include "FHandles.h"

#define WM_TRAYICON			WM_APP + 100
#define MAX_GRAPH_WIDTH		2048
#define GRID_SPACE_HOR		12
#define GRID_SPACE_VERT		12

const char AppName[] = " ";
int LastCol = -1, OrderBy = 1;
bool Modified = false;
double* cpu_graph;
double* mem_graph;
int nGraphWidth;
int nGridOffs = 0;
MEMORYSTATUS ms;
int g_NumProcesses = 0;
int g_NumThreads = 0;
int g_NumHandles = 0;
double g_CPUUsage = 0;
double g_CPUUsage2 = 0;
__int64 g_MemUsage = 0;
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "Placemnt"
#pragma link "CustomProgressBar"
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
    OSVERSIONINFO os;
    os.dwOSVersionInfoSize = sizeof(os);
    GetVersionEx(&os);
	if (!(os.dwMajorVersion >= 5 && os.dwPlatformId == VER_PLATFORM_WIN32_NT))
    {
    	Application->MessageBox(
        	",       WinNT 5.0  ",
            Application->Title.c_str(),
            MB_OK | MB_ICONSTOP);
        TerminateProcess(GetCurrentProcess(), -1);
    }
    Reg = new TRegIniFile("Software\\Forte-IT\\TaskManager");
    PerfInit();
    if (IsWinNT())
        SetProcessPrivilege(GetCurrentProcessId(), SE_DEBUG_NAME, true);
    WindowProc = SubClassWndProc;
    NOTIFYICONDATA data = {0};
    data.cbSize = sizeof(data);
    data.hWnd = Handle;
    data.uID = 1;
    data.uFlags = NIF_ICON | NIF_MESSAGE;
    data.uCallbackMessage = WM_TRAYICON;
    data.hIcon = Application->Icon->Handle;
    Shell_NotifyIcon(NIM_ADD, &data);
    cpu_graph = new double[MAX_GRAPH_WIDTH];
    mem_graph = new double[MAX_GRAPH_WIDTH];
    ZeroMemory(cpu_graph, MAX_GRAPH_WIDTH * sizeof(double));
    ZeroMemory(mem_graph, MAX_GRAPH_WIDTH * sizeof(double));

    AddAdvPerfItem("IdleProcessTime");
    AddAdvPerfItem("IoReadTransferCount");
    AddAdvPerfItem("IoWriteTransferCount");
    AddAdvPerfItem("IoOtherTransferCount");
    AddAdvPerfItem("IoReadOperationCount");
    AddAdvPerfItem("IoWriteOperationCount");
    AddAdvPerfItem("IoOtherOperationCount");
    AddAdvPerfItem("AvailablePages");
    AddAdvPerfItem("CommittedPages");
    AddAdvPerfItem("CommitLimit");
    AddAdvPerfItem("PeakCommitment");
    AddAdvPerfItem("PageFaultCount");
    AddAdvPerfItem("CopyOnWriteCount");
    AddAdvPerfItem("TransitionCount");
    AddAdvPerfItem("CacheTransitionCount");
    AddAdvPerfItem("DemandZeroCount");
    AddAdvPerfItem("PageReadCount");
    AddAdvPerfItem("PageReadIoCount");
    AddAdvPerfItem("CacheReadCount");
    AddAdvPerfItem("CacheIoCount");
    AddAdvPerfItem("DirtyPagesWriteCount");
    AddAdvPerfItem("DirtyWriteIoCount");
    AddAdvPerfItem("MappedPagesWriteCount");
    AddAdvPerfItem("MappedWriteIoCount");
    AddAdvPerfItem("PagedPoolPages");
    AddAdvPerfItem("NonPagedPoolPages");
    AddAdvPerfItem("PagedPoolAllocs");
    AddAdvPerfItem("PagedPoolFrees");
    AddAdvPerfItem("NonPagedPoolAllocs");
    AddAdvPerfItem("NonPagedPoolFrees");
    AddAdvPerfItem("FreeSystemPtes");
    AddAdvPerfItem("ResidentSystemCodePage");
    AddAdvPerfItem("TotalSystemDriverPages");
    AddAdvPerfItem("TotalSystemCodePages");
    AddAdvPerfItem("Spare0Count");
    AddAdvPerfItem("Spare1Count");
    AddAdvPerfItem("Spare3Count");
    AddAdvPerfItem("ResidentSystemCachePage");
    AddAdvPerfItem("ResidentPagedPoolPage");
    AddAdvPerfItem("ResidentSystemDriverPage");
    AddAdvPerfItem("CcFastReadNoWait");
    AddAdvPerfItem("CcFastReadWait");
    AddAdvPerfItem("CcFastReadResourceMiss");
    AddAdvPerfItem("CcFastReadNotPossible");
    AddAdvPerfItem("CcFastMdlReadNoWait");
    AddAdvPerfItem("CcFastMdlReadWait");
    AddAdvPerfItem("CcFastMdlReadResourceMiss");
    AddAdvPerfItem("CcFastMdlReadNotPossible");
    AddAdvPerfItem("CcMapDataNoWait");
    AddAdvPerfItem("CcMapDataWait");
    AddAdvPerfItem("CcMapDataNoWaitMiss");
    AddAdvPerfItem("CcMapDataWaitMiss");
    AddAdvPerfItem("CcPinMappedDataCount");
    AddAdvPerfItem("CcPinReadNoWait");
    AddAdvPerfItem("CcPinReadWait");
    AddAdvPerfItem("CcPinReadNoWaitMiss");
    AddAdvPerfItem("CcPinReadWaitMiss");
    AddAdvPerfItem("CcCopyReadNoWait");
    AddAdvPerfItem("CcCopyReadWait");
    AddAdvPerfItem("CcCopyReadNoWaitMiss");
    AddAdvPerfItem("CcCopyReadWaitMiss");
    AddAdvPerfItem("CcMdlReadNoWait");
    AddAdvPerfItem("CcMdlReadWait");
    AddAdvPerfItem("CcMdlReadNoWaitMiss");
    AddAdvPerfItem("CcMdlReadWaitMiss");
    AddAdvPerfItem("CcReadAheadIos");
    AddAdvPerfItem("CcLazyWriteIos");
    AddAdvPerfItem("CcLazyWritePages");
    AddAdvPerfItem("CcDataFlushes");
    AddAdvPerfItem("CcDataPages");
    AddAdvPerfItem("ContextSwitches");
    AddAdvPerfItem("FirstLevelTbFills");
    AddAdvPerfItem("SecondLevelTbFills");
    AddAdvPerfItem("SystemCalls");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddAdvPerfItem(AnsiString name)
{
	TListItem* li = LVAdvPerf->Items->Add();
    li->Caption = name;
    li->SubItems->Add("");
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
    NOTIFYICONDATA data = {0};
    data.cbSize = sizeof(data);
    data.hWnd = Handle;
    data.uID = 1;
    data.uFlags = 0;
    Shell_NotifyIcon(NIM_DELETE, &data);
    delete cpu_graph;
    delete mem_graph;
    delete Reg;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
    LoadSettings();
    GetSysInfo();
    RefreshProcesses();
    ExpandTree(NULL);
    SetStayOnTop(Handle, iStayOnTop->Checked);
    if (IsWinNT())
    {
        SHFILEINFO sfi;
        ProcessList->SmallImages = ImageList;
        ImageList->Handle = (DWORD)SHGetFileInfo("C:\\", 0, &sfi, sizeof(SHFILEINFO),
	    	SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
    }
    else
    {
        ProcessPopup->Items->Delete(ProcessPopup->Items->IndexOf(iSetPrivileges));
        ProcessPopup->Items->Delete(ProcessPopup->Items->IndexOf(iSecurity));
        ProcessPopup->Items->Delete(ProcessPopup->Items->IndexOf(iResetWorkingSetSize));
        ProcessPopup->Items->Delete(ProcessPopup->Items->IndexOf(iChangeWorkingSet));
    }
	UpdateProcessorInfo();
   	Timer->Enabled = true;
}
//---------------------------------------------------------------------------
VOID WINAPI ProcessEnumProc(PSYSTEM_PROCESSES pProc, LPVOID lpParam)
{
    ProcEnum *p = (ProcEnum *)lpParam;
    TListItems *lis = MainForm->ProcessList->Items;
    TListItem *li;
    int i, k = p->k;

    for (i = 0; i < k; i++)
    {
        li = lis->Item[i];
        if (!p->Used[i] && ((TMyProcess *)li->Data)->pid == pProc->ProcessId)
        {
            p->Used[i] = true;
            MainForm->AddProcessToList(li, pProc);
            break;
        }
    }

    if (i == k)
    {
        li = lis->Add();
        li->Data = NULL;
        MainForm->AddProcessToList(li, pProc);
    }

    while (!MainForm->AddProcessToTree(NULL, (TMyProcess *)li->Data))
        ((TMyProcess *)li->Data)->ParentPid = 0;

    if (!MainForm->CBSelProcesses->Checked || li->Selected)
    {
		g_MemUsage += pProc->VmCounters.PagefileUsage;
    }

    p->NumProcess[0]++;

    g_NumProcesses++;
    g_NumThreads += pProc->ThreadCount;
    g_NumHandles += pProc->HandleCount;
}
//---------------------------------------------------------------------------
int __stdcall CustomSortProc(long Item1, long Item2, long ParamSort)
{
    int res, n = (ParamSort & 0xFF), OrderBy = ParamSort >> 16;
    int k = (ParamSort & 0xFF00) >> 8;
    TListItem *li1 = (TListItem *)Item1, *li2 = (TListItem *)Item2;

    if (!n)
        res = MyCompareStr(li1->Caption.c_str(),
            li2->Caption.c_str(), IsNumberCol[k]);
    else
        res = MyCompareStr(li1->SubItems->Strings[n - 1].c_str(),
            li2->SubItems->Strings[n - 1].c_str(), IsNumberCol[k]);

    return res * OrderBy;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RefreshProcesses()
{
    ProcEnum p;
    int i, j, k = ProcessList->Items->Count;
    bool *Used = (bool *)LocalAlloc(LMEM_FIXED | LMEM_ZEROINIT,
        k * sizeof(bool));
    p.Used = Used;
    p.k = k;
    p.NumProcess = &NumProcess;
    NumProcess = 0;

    g_CPUUsage = 0;
    g_MemUsage = 0;
    Modified = false;
    g_NumProcesses = 0;
    g_NumThreads = 0;
    g_NumHandles = 0;

    if (IsWinNT())
        EnumProcessesNt(ProcessEnumProc, &p);
    else
        EnumProcesses9x(ProcessEnumProc, &p);

    DeleteUnusedNodes(NULL);
    for (i = 0, j = 0; i < k; i++)
        if (!Used[i])
        {
            ProcessList->Items->Delete(j);
            Modified = true;
        }
        else j++;

    LocalFree(Used);

    if (LastCol >= 0 && Modified)
    {
        ProcessList->CustomSort(CustomSortProc, LastCol |
            (ProcessList->Columns->Items[LastCol]->Tag << 8) | (OrderBy << 16));
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RefreshColumns()
{
    DWORD t = Columns;
    TListColumns *lcs = ProcessList->Columns;
    TListColumn *lc;
    int k = 0, m = lcs->Count;

    for (int i = 0; i < NUM_COLUMNS; i++, t >>= 1)
    {
        if (!(t & 1)) continue;
        if (k < m)
            lc = lcs->Items[k];
        else
        {
            lc = lcs->Add();
            m++;
        }
        lc->Caption = ColInfo[i].DisplayName;
        lc->Width = 100;
        lc->Tag = i;
        k++;
    }

    while (k < m)
    {
        lcs->Delete(k);
        m--;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iExitClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iRefreshClick(TObject *Sender)
{
    RefreshProcesses();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iSelColumnsClick(TObject *Sender)
{
    ColumnsDlg->Columns = Columns;
    if (ColumnsDlg->ShowModal() != mrOk) return;
    if (Columns == ColumnsDlg->Columns) return;
    Columns = ColumnsDlg->Columns;
    RefreshColumns();
    RefreshProcesses();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimerTimer(TObject *Sender)
{
    char buf[64];
    RefreshProcesses();
    if (ThreadsForm->Visible) ThreadsForm->RefreshList();
    GlobalMemoryStatus(&ms);

    g_CPUUsage2 = 0;
    GetCPUUsage(g_CPUUsage2);
    
    if (!CBSelProcesses->Checked)
    {
    	g_CPUUsage = g_CPUUsage2;
    }

    StatusBar->Panels->Items[0]->Text = AnsiString(": ") + NumProcess;
    AnsiString s;
    s.printf(" : %.2f%%", g_CPUUsage2);
    if (IsWinNT())
    {
        StatusBar->Panels->Items[1]->Text = s;
    }
    NOTIFYICONDATA data = {0};
    data.cbSize = sizeof(data);
    data.hWnd = Handle;
    data.uID = 1;
    data.uFlags = NIF_TIP;
    strcpy(data.szTip, StatusBar->Panels->Items[1]->Text.c_str());
    Shell_NotifyIcon(NIM_MODIFY, &data);
    wsprintf(buf, ": %dK/%dK",
        (ms.dwTotalPageFile - ms.dwAvailPageFile) >> 10, ms.dwTotalPageFile >> 10);
    StatusBar->Panels->Items[2]->Text = buf;
    UpdatePerfTab();
    UpdateAdvancedTab();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessListColumnClick(TObject *Sender,
      TListColumn *Column)
{
    int index = Column->Index;
    OrderBy = index == LastCol ? OrderBy * -1 : 1;
    ProcessList->CustomSort(CustomSortProc, index |
        (Column->Tag << 8) | (OrderBy << 16));
    LastCol = index;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PriorityClick(TObject *Sender)
{
    ((TMenuItem *)Sender)->Checked = true;
    int p;
    if (iRealTime->Checked) p = REALTIME_PRIORITY_CLASS;
    else if (iHigh->Checked) p = HIGH_PRIORITY_CLASS;
    else if (iAboveNormal->Checked) p = ABOVE_NORMAL_PRIORITY_CLASS;
    else if (iNormal->Checked) p = NORMAL_PRIORITY_CLASS;
    else if (iBelowNormal->Checked) p = BELOW_NORMAL_PRIORITY_CLASS;
    else if (iLow->Checked) p = IDLE_PRIORITY_CLASS;
    if (!GetProcess()->SetPriority(p))
        ShowGUIError(Handle, "   ");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TerminateBtnClick(TObject *Sender)
{
	if (PageControl->ActivePage == ProcessesTab &&
    	ProcessList->SelCount > 1)
    {
        if (Application->MessageBox(
        	"  ?",
            "", MB_YESNO | MB_ICONWARNING) != IDYES) return;
    	int i = 0, k = ProcessList->Items->Count;
        for (; i < k; i++)
        {
        	if (ProcessList->Items->Item[i]->Selected)
            {
                if (!((TMyProcess*)ProcessList->Items->Item[i]->Data)->Kill())
                {
                    ShowGUIError(
                    	Handle,
                        ("   " + ProcessList->Items->Item[i]->Caption).c_str());
                    k = ProcessList->Items->Count;
					if (i > k - 1)
                    {
                    	i = k - 1;
                    }
                }
            }
        }
    }
    else
    {
        if (Application->MessageBox(("  \"" +
            GetCaption() + "\"?").c_str(),
            "", MB_YESNO | MB_ICONWARNING) != IDYES) return;
        if (!GetProcess()->Kill())
            ShowGUIError(Handle, "  ");
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iKillProcessTreeClick(TObject *Sender)
{
    if (Application->MessageBox(("  , \n\"" +
        GetCaption() + "\"?").c_str(),
        "", MB_YESNO | MB_ICONWARNING) != IDYES) return;
    if (!GetProcess()->Kill())
        ShowGUIError(Handle, "    ");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iSetPrivilegesClick(TObject *Sender)
{
    DWORD pid = GetProcess()->pid;
    DWORD i, Count, t;
    LPTSTR PrivilegeNames = NULL, p;
    BOOL *Enabled = NULL;

    HANDLE hHeap = GetProcessHeap();

    try
    {
        if (!GetProcessPrivileges(pid, &PrivilegeNames, &Enabled, &Count))
            throw 0;

        PrivilegeDlg->PrivList->Clear();
        p = PrivilegeNames;
        for (i = 0; i < Count; i++)
        {
            PrivilegeDlg->PrivList->Items->Add(p);
            PrivilegeDlg->PrivList->Checked[i] = Enabled[i];
            while (*p++);
        }
    }
    catch (int)
    {
        if (Enabled) HeapFree(hHeap, 0, Enabled);
        if (PrivilegeNames) HeapFree(hHeap, 0, PrivilegeNames);
        ShowGUIError(Handle, "   ");
        return;
    }

    if (PrivilegeDlg->ShowModal() != mrOk) return;

    for (i = 0, t = 0; i < Count; i++)
    {
        if (Enabled[i] == PrivilegeDlg->PrivList->Checked[i]) t++;
        Enabled[i] = PrivilegeDlg->PrivList->Checked[i];
    }

    if (t < Count)
        if (!SetProcessPrivileges(pid, PrivilegeNames, Enabled, Count))
            ShowGUIError(Handle, "   ");

    if (Enabled) HeapFree(hHeap, 0, Enabled);
    if (PrivilegeNames) HeapFree(hHeap, 0, PrivilegeNames);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddProcessToList(TListItem *li, PSYSTEM_PROCESSES p)
{
    char buf[MAX_PATH];
    TMyProcess *proc;
    TStrings *ts = li->SubItems;
    DWORD t = Columns >> 1;
    int k = 0, m = ts->Count;
    bool NeedIcon = false;
    SHFILEINFO sfi;

    if (!li->Data)
    {
        proc = new TMyProcess(p);
        li->Data = proc;
        NeedIcon = true;
    }
    else
    {
        proc = (TMyProcess *)li->Data;
        if (proc->pid != p->ProcessId)
        {
            delete proc;
            proc = new TMyProcess(p);
            NeedIcon = true;
        }
    }
    proc->UpdateInfo(p);

    if (CBSelProcesses->Checked && li->Selected && p->ProcessId)
    {
   		g_CPUUsage += proc->CPUUse / 100;
    }

    FormatProcessData(proc, 0, buf);
    if (li->Caption != buf)
    {
        li->Caption = buf;
        Modified = LastCol == 0;
    }

    if (NeedIcon)
    {
        li->ImageIndex = -1;
        if (proc->ImagePath[0] &&
            SHGetFileInfo(proc->ImagePath, 0, &sfi, sizeof(sfi), SHGFI_ICON))
            li->ImageIndex = sfi.iIcon;
    }

    for (int i = 1; i < NUM_COLUMNS; i++, t >>= 1)
    {
        if (!(t & 1)) continue;
        FormatProcessData(proc, i, buf);
        if (k < m)
        {
            if (lstrcmpi(ts->Strings[k].c_str(), buf))
            {
                ts->Strings[k] = buf;
                Modified = i == LastCol;
            }
        } else
        {
            Modified = true;
            ts->Add(buf);
            m++;
        }
        k++;
    }

    while (k < m)
    {
        Modified = true;
        li->SubItems->Delete(k);
        m--;
    }
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::AddProcessToTree(TTreeNode *Root, TMyProcess *p)
{
    TTreeNode *Child;
    DWORD ParentPid = 0;
    if (Root)
    {
        Child = Root->getFirstChild();
        ParentPid = ((TMyProcess *)Root->Data)->pid;
    }
    else Child = ProcessTree->Items->GetFirstNode();

    while (Child)
    {
        if (p->ParentPid == ParentPid)
        {
            if (p->pid == ((TMyProcess *)Child->Data)->pid)
            {
                if (Child->Text != p->ProcessName)
                    Child->Text = p->ProcessName;
                ((TMyProcess *)Child->Data)->Flag = 1;
                return true;
            }
        }
        else if (AddProcessToTree(Child, p)) return true;
        Child = Child->getNextSibling();
    }

    if (p->ParentPid == ParentPid)
    {
        Child = ProcessTree->Items->AddChild(Root, p->ProcessName);
        Child->Data = p;
        p->Flag = 1;
        return true;
    }

    return false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DeleteUnusedNodes(TTreeNode *Root)
{
    TTreeNode *Child, *t;
    if (Root)
        Child = Root->getFirstChild();
    else
        Child = ProcessTree->Items->GetFirstNode();
    while (Child)
    {
        if (!((TMyProcess *)Child->Data)->Flag)
        {
            t = Child;
            Child = Child->getNextSibling();
            ProcessTree->Items->Delete(t);
        }
        else
        {
            ((TMyProcess *)Child->Data)->Flag = 0;
            DeleteUnusedNodes(Child);
            Child = Child->getNextSibling();
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExpandTree(TTreeNode *Root)
{
    TTreeNode *Child;
    if (Root)
        Child = Root->getFirstChild();
    else
        Child = ProcessTree->Items->GetFirstNode();
    while (Child)
    {
        if (Child->HasChildren)
            Child->Expand(true);
        ExpandTree(Child);
        Child = Child->getNextSibling();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetSysInfo()
{
    char buf[MAX_PATH + 1] = {0};
    DWORD Size;
    SYSTEM_INFO si;
/*    OSVERSIONINFOEX os;

    os.dwOSVersionInfoSize = sizeof(os);
    GetVersionEx(&os);

    AddSysInfoItem(" ",
        AnsiString(os.dwMajorVersion) + '.' + os.dwMinorVersion);

    switch (os.dwMajorVersion)
    {
    	case 3:
            switch (os.dwPlatformId)
            {
                case VER_PLATFORM_WIN32s:
                    sprintf(buf, "Win32s on Windows 3.%d", os.dwMinorVersion);
                    break;
                case VER_PLATFORM_WIN32_NT:
                    sprintf(buf, "Windows NT 3.%d", os.dwMinorVersion);
                    break;
                default:
                	goto ver_hz;
            }
            break;

    	case 4:
            switch (os.dwPlatformId)
            {
                case VER_PLATFORM_WIN32_NT:
                    sprintf(buf, "Windows NT 4.%d", os.dwMinorVersion);
                    break;

                case VER_PLATFORM_WIN32_WINDOWS:
                    if ((os.dwMinorVersion < 10) && (os.dwBuildNumber == 950))
                        strcpy(buf, "Windows 95");
                    else if ((os.dwMinorVersion < 10) && ((os.dwBuildNumber > 950) && (os.dwBuildNumber <= 1080)))
                        strcpy(buf, "Windows 95 SP1");
                    else if ((os.dwMinorVersion < 10) && (os.dwBuildNumber > 1080))
                        strcpy(buf, "Windows 95 OSR2");
                    else if ((os.dwMinorVersion == 10) && (os.dwBuildNumber == 1998))
                        strcpy(buf, "Windows 98");
                    else if ((os.dwMinorVersion == 10) && ((os.dwBuildNumber > 1998) && (os.dwBuildNumber < 2183)))
                        strcpy(buf, "Windows 98 SP1");
                    else if ((os.dwMinorVersion == 10) && (os.dwBuildNumber >= 2183))
                        strcpy(buf, "Windows 98 SE");
                    else if (os.dwMinorVersion == 90)
                        strcpy(buf, "Windows ME");
                    else
                    	goto ver_hz;
                    break;
            }
            break;

        case 5:
            if (os.dwPlatformId == VER_PLATFORM_WIN32_NT)
            {
            	switch (os.dwMinorVersion)
                {
                	case 0:
                    	strcpy(buf, "Windows 2000");
                        break;
                	case 1:
                    	strcpy(buf, "Windows XP");
                        break;
                	case 2:
                    	strcpy(buf, "Windows Server 2003");
                        break;
                }
            }
            else
            {
               	goto ver_hz;
            }
            break;

ver_hz:
        default:
            sprintf(buf, "Windows %d.%d", os.dwMajorVersion, os.dwMinorVersion);
            break;
    }            */

/*    switch (os.dwPlatformId)
    {
        case VER_PLATFORM_WIN32s:
            lstrcpy(buf, "Win32s on Windows 3.1");
            break;
        case VER_PLATFORM_WIN32_WINDOWS:
            lstrcpy(buf, "Windows 9x/Me");
            break;
        case VER_PLATFORM_WIN32_NT:
            lstrcpy(buf, "Windows NT");
    }*/


    OSVERSIONINFOEX osvi;
    BOOL bOsVersionInfoEx;

    // Try calling GetVersionEx using the OSVERSIONINFOEX structure.
    // If that fails, try using the OSVERSIONINFO structure.

    ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

    if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
    {
       osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
       GetVersionEx ( (OSVERSIONINFO *) &osvi);
    }

    switch (osvi.dwPlatformId)
    {
       // Test for the Windows NT product family.
       case VER_PLATFORM_WIN32_NT:

          // Test for the specific product family.
          if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 )
             strcpy (buf, "Microsoft Windows Server 2003 family, ");

          if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )
             strcpy (buf, "Microsoft Windows XP ");

          if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
             strcpy (buf, "Microsoft Windows 2000 ");

          if ( osvi.dwMajorVersion <= 4 )
             strcpy (buf, "Microsoft Windows NT ");

          // Test for specific product on Windows NT 4.0 SP6 and later.
          if( bOsVersionInfoEx )
          {
             // Test for the workstation type.
             if ( osvi.wProductType == VER_NT_WORKSTATION )
             {
                if( osvi.dwMajorVersion == 4 )
                   strcat (buf,  "Workstation 4.0 " );
                else if( osvi.wSuiteMask & VER_SUITE_PERSONAL )
                   strcat (buf,  "Home Edition " );
                else
                   strcat (buf,  "Professional " );
             }

             // Test for the server type.
             else if ( osvi.wProductType == VER_NT_SERVER )
             {
                if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 )
                {
                   if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                      strcat (buf,  "Datacenter Edition " );
                   else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                      strcat (buf,  "Enterprise Edition " );
                   else if ( osvi.wSuiteMask == VER_SUITE_BLADE )
                      strcat (buf,  "Web Edition " );
                   else
                      strcat (buf,  "Standard Edition " );
                }

                else if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
                {
                   if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                      strcat (buf,  "Datacenter Server " );
                   else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                      strcat (buf,  "Advanced Server " );
                   else
                      strcat (buf,  "Server " );
                }

                else  // Windows NT 4.0 
                {
                   if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                      strcat (buf, "Server 4.0, Enterprise Edition " );
                   else
                      strcat (buf,  "Server 4.0 " );
                }
             }
          }
          else  // Test for specific product on Windows NT 4.0 SP5 and earlier
          {
          #define BUFSIZE 256
             HKEY hKey;
             char szProductType[BUFSIZE];
             DWORD dwBufLen=BUFSIZE;
             LONG lRet;

             lRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                "SYSTEM\\CurrentControlSet\\Control\\ProductOptions",
                0, KEY_QUERY_VALUE, &hKey );
//             if( lRet != ERROR_SUCCESS )
//                return FALSE;

             lRet = RegQueryValueEx( hKey, "ProductType", NULL, NULL,
                (LPBYTE) szProductType, &dwBufLen);
//             if( (lRet != ERROR_SUCCESS) || (dwBufLen > BUFSIZE) )
//                return FALSE;

             RegCloseKey( hKey );

             if ( lstrcmpi( "WINNT", szProductType) == 0 )
                strcat(buf, "Workstation " );
             if ( lstrcmpi( "LANMANNT", szProductType) == 0 )
                strcat(buf, "Server " );
             if ( lstrcmpi( "SERVERNT", szProductType) == 0 )
                strcat(buf, "Advanced Server " );
          }

          break;

       // Test for the Windows 95 product family.
       case VER_PLATFORM_WIN32_WINDOWS:

          if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0)
          {
              strcpy (buf, "Microsoft Windows 95");
              if ( osvi.szCSDVersion[1] == 'C' || osvi.szCSDVersion[1] == 'B' )
                 strcat(buf,"OSR2 " );
          }

          if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 10)
          {
              strcpy (buf, "Microsoft Windows 98 ");
              if ( osvi.szCSDVersion[1] == 'A' )
                 strcat(buf,"SE " );
          }

          if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 90)
          {
              strcpy (buf, "Microsoft Windows Millennium Edition");
          }
          break;

       case VER_PLATFORM_WIN32s:

          strcpy (buf, "Microsoft Win32s");
          break;
    }

    AddSysInfoItem(" ",
        AnsiString(osvi.dwMajorVersion) + '.' + osvi.dwMinorVersion);
    AddSysInfoItem(" ", buf);
    AddSysInfoItem("", osvi.dwBuildNumber & 0xFFFF);
    AddSysInfoItem("", osvi.szCSDVersion);

    Size = sizeof(buf);
    GetUserName(buf, &Size);
    AddSysInfoItem(" ", buf);
    Size = sizeof(buf);
    GetComputerName(buf, &Size);
    AddSysInfoItem(" ", buf);

    Size = sizeof(buf);
    GetSystemDirectory(buf, Size);
    AddSysInfoItem(" ", buf);
    GetWindowsDirectory(buf, Size);
    AddSysInfoItem(" Windows", buf);

    GetSystemInfo(&si);

    switch (si.wProcessorArchitecture)
    {
        case PROCESSOR_ARCHITECTURE_INTEL:
            switch (si.wProcessorLevel)
            {
                case 3: lstrcpy(buf, "Intel 80386"); break;
                case 4: lstrcpy(buf, "Intel 80486"); break;
                case 5: lstrcpy(buf, "Intel Pentium"); break;
                case 6: lstrcpy(buf, "Intel Pentium Pro or Pentium II"); break;
                default: lstrcpy(buf, "");
            }
            break;

        case PROCESSOR_ARCHITECTURE_MIPS:
            if (si.wProcessorLevel == 4)
                lstrcpy(buf, "MIPS R4000");
            else
                lstrcpy(buf, "");
                break;

        case PROCESSOR_ARCHITECTURE_ALPHA:
            switch (si.wProcessorLevel)
            {
                case 21064: lstrcpy(buf, "Alpha 21064"); break;
                case 21066: lstrcpy(buf, "Alpha 21066"); break;
                case 21164: lstrcpy(buf, "Alpha 21164"); break;
                default: lstrcpy(buf, "");
            }
            break;

        case PROCESSOR_ARCHITECTURE_PPC:
            switch (si.wProcessorLevel)
            {
                case 1: lstrcpy(buf, "PPC 601"); break;
                case 3: lstrcpy(buf, "PPC 603"); break;
                case 4: lstrcpy(buf, "PPC 604"); break;
                case 6: lstrcpy(buf, "PPC 603+"); break;
                case 9: lstrcpy(buf, "PPC 604+"); break;
                case 20: lstrcpy(buf, "PPC 620"); break;
                default: lstrcpy(buf, "");
            }
            break;

        case PROCESSOR_ARCHITECTURE_IA64:
            lstrcpy(buf, "IA64");
            break;

        case PROCESSOR_ARCHITECTURE_IA32_ON_WIN64:
            lstrcpy(buf, "IA32 on Win64");
            break;

        case PROCESSOR_ARCHITECTURE_AMD64:
            lstrcpy(buf, "AMD64");
            break;

        case PROCESSOR_ARCHITECTURE_UNKNOWN:
        default:
            lstrcpy(buf, "");
    }

    AddSysInfoItem(" ", buf);
    AddSysInfoItem(" ", si.dwNumberOfProcessors);
    AddSysInfoItem(" ", si.dwPageSize);
    wsprintf(buf, "%08X - %08X",
        si.lpMinimumApplicationAddress,
        si.lpMaximumApplicationAddress);
    AddSysInfoItem("  ", buf);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddSysInfoItem(AnsiString param, AnsiString val)
{
    TListItem *li;
    li = SysInfoList->Items->Add();
    li->Caption = param + ':';
    li->SubItems->Add(val);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iRunClick(TObject *Sender)
{
    if (IsWinNT())
    {
        WCHAR CurDir[MAX_PATH];
        GetCurrentDirectoryW(MAX_PATH, CurDir);
        SHRunProgramDlgW(Handle, 0, CurDir,
            L" ",
            L"  , ,    , \
 Windows    :", 0x14);
    }
    else
    {
        CHAR CurDir[MAX_PATH];
        GetCurrentDirectoryA(MAX_PATH, CurDir);
        SHRunProgramDlgA(Handle, 0, CurDir,
            " ",
            "  , ,    , \
 Windows    :", 0x14);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iStayOnTopClick(TObject *Sender)
{
    iStayOnTop->Checked ^= true;
    iStayOnTop2->Checked = iStayOnTop->Checked;
    SetStayOnTop(Handle, iStayOnTop->Checked);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iRefreshSpeedClick(TObject *Sender)
{
    if (Sender == iHighSpeed) Timer->Interval = 500;
    else if (Sender == iNormalSpeed) Timer->Interval = 1000;
    else if (Sender == iLowSpeed) Timer->Interval = 2000;
    else if (Sender == iPaused)
    {
        Timer->Enabled = false;
        return;
    }
    Timer->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadFormSettings(TForm *frm, AnsiString Name)
{
/*    int x = Reg->ReadInteger("Settings", Name + 'X', 0x7FFFFFFF);
    int y = Reg->ReadInteger("Settings", Name + 'Y', 0x7FFFFFFF);
    int w = Reg->ReadInteger("Settings", Name + 'W', -1);
    int h = Reg->ReadInteger("Settings", Name + 'H', -1);
    if (w > 0 && h > 0)
    {
        if (w >= frm->Constraints->MinWidth) frm->Width = w;
        if (h >= frm->Constraints->MinHeight) frm->Height = h;
    }
    if (x != 0x7FFFFFFF && y != 0x7FFFFFFF)
    {
        frm->Left = x;
        frm->Top = y;
    } */
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadSettings()
{
//    LoadFormSettings(this, Name);
    if (Reg->ReadInteger("Settings", "MainFormStayOnTop", 0))
        iStayOnTopClick(NULL);
    LoadFormSettings(ModsForm, ModsForm->Name);
    LoadFormSettings(MemForm, MemForm->Name);
    LoadFormSettings(ThreadsForm, ThreadsForm->Name);
    int s = Reg->ReadInteger("Settings", "RefreshSpeed", 2);
    switch (s)
    {
        case 3: iHighSpeed->Checked = true; iRefreshSpeedClick(iHighSpeed); break;
        case 1: iLowSpeed->Checked = true; iRefreshSpeedClick(iLowSpeed); break;
        case 0: iPaused->Checked = true; iRefreshSpeedClick(iPaused); break;
        default: iNormalSpeed->Checked = true; iRefreshSpeedClick(iNormalSpeed);
    }
    Columns = (IsWinNT() ? WINNT_MASK : WIN9X_MASK) &
        Reg->ReadInteger("Settings", "Columns", 0x000F);
    RefreshColumns();
    for (int i = 0; i < ProcessList->Columns->Count; i++)
    {
	    ProcessList->Columns->Items[i]->Width = Reg->ReadInteger(
        	"Settings",
            "ColWidth" + AnsiString(i + 1),
            100);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveFormSettings(TForm *frm, AnsiString Name)
{
//    Reg->WriteInteger("Settings", Name + 'X', frm->Left);
//    Reg->WriteInteger("Settings", Name + 'Y', frm->Top);
//    Reg->WriteInteger("Settings", Name + 'W', frm->Width);
//    Reg->WriteInteger("Settings", Name + 'H', frm->Height);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveSettings()
{
    SaveFormSettings(this, Name);
    Reg->WriteInteger("Settings", "MainStayOnTop", iStayOnTop->Checked);
//    SaveFormSettings(ModsForm, ModsForm->Name);
//    SaveFormSettings(MemForm, MemForm->Name);
//    SaveFormSettings(ThreadsForm, ThreadsForm->Name);
    int s = 0;
    if (iHighSpeed->Checked) s = 3;
    else if (iNormalSpeed->Checked) s = 2;
    else if (iLowSpeed->Checked) s = 1;
    Reg->WriteInteger("Settings", "RefreshSpeed", s);
    Reg->WriteInteger("Settings", "Columns", Columns &
        (IsWinNT() ? WINNT_MASK : WIN9X_MASK));
    for (int i = 0; i < ProcessList->Columns->Count; i++)
    {
	    Reg->WriteInteger(
        	"Settings",
            "ColWidth" + AnsiString(i + 1),
            ProcessList->Columns->Items[i]->Width);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    ImageList->Handle = 0;
    SaveSettings();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iModsListClick(TObject *Sender)
{
    iModsList->Checked ^= true;
    if (iModsList->Checked)
    {
		ModsForm->SetProcess(GetProcess());
        ModsForm->Show();
    }
    else
        ModsForm->Hide();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessListSelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
	TerminateBtn->Enabled = Selected && ((TMyProcess *)Item->Data)->pid;
    if (!Selected) return;
    if (ModsForm->Visible) ModsForm->SetProcess((TMyProcess *)Item->Data);
    if (MemForm->Visible) MemForm->SetProcess((TMyProcess *)Item->Data);
    if (ThreadsForm->Visible) ThreadsForm->SetProcess((TMyProcess *)Item->Data);
    if (FrmHandles->Visible) FrmHandles->SetProcess((TMyProcess *)Item->Data);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iSaveInfoClick(TObject *Sender)
{
    SaveTableDlg->Save(ProcessList);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iVerInfoClick(TObject *Sender)
{
    VersionDlg->ShowVersionInfo(GetProcess()->ImagePath);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iStartupInfoClick(TObject *Sender)
{
    StartupInfoDlg->ShowStartupInfo(GetProcess()->pid);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TerminateProcessTreeBtnClick(TObject *Sender)
{
    if (Application->MessageBox(("  , \n\"" +
        ProcessTree->Selected->Text + "\"?").c_str(),
        "", MB_YESNO | MB_ICONWARNING) != IDYES) return;
    if (!KillProcessTree(((TMyProcess *)ProcessTree->Selected->Data)->pid))
        ShowGUIError(Handle, "    ");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessTreeChange(TObject *Sender,
      TTreeNode *Node)
{
    TerminateBtn2->Enabled = ((TMyProcess *)Node->Data)->pid;
    TerminateProcessTreeBtn->Enabled = TerminateBtn2->Enabled;
    if (ModsForm->Visible) ModsForm->SetProcess((TMyProcess *)Node->Data);
    if (MemForm->Visible) MemForm->SetProcess((TMyProcess *)Node->Data);
    if (ThreadsForm->Visible) ThreadsForm->SetProcess((TMyProcess *)Node->Data);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessTreeKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if (Key == VK_DELETE && TerminateBtn2->Enabled)
        TerminateBtnClick(NULL);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMainForm::GetCaption()
{
    if (PageControl->ActivePage == ProcessesTab)
        return ProcessList->Selected->Caption;
    else if (PageControl->ActivePage == ProcessTreeTab)
        return ProcessTree->Selected->Text;
    return "";
}
//---------------------------------------------------------------------------
TMyProcess* __fastcall TMainForm::GetProcess()
{
	if (PageControl->ActivePage == ProcessesTab && ProcessList->Selected)
		return (TMyProcess *)ProcessList->Selected->Data;
	else if (PageControl->ActivePage == ProcessTreeTab && ProcessTree->Selected)
		return (TMyProcess *)ProcessTree->Selected->Data;
	return NULL;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessTreeMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if (!ProcessTree->Selected && ModsForm->Visible)
        ModsForm->SetProcess(NULL);
    if (Button != mbRight) return;
	TTreeNode* pNode = ProcessTree->GetNodeAt(X, Y);
    if (pNode)
    {
    	ProcessTree->Selected = pNode;
    }
    TTreeNode *li = ProcessTree->Selected;
    if (!li || !((TMyProcess *)li->Data)->pid) return;
/*    switch (((TMyProcess *)li->Data)->BasePriority)
    {
        case ABOVE_NORMAL_PRIORITY_CLASS: iAboveNormal->Checked = true; break;
        case BELOW_NORMAL_PRIORITY_CLASS: iBelowNormal->Checked = true; break;
        case HIGH_PRIORITY_CLASS: iHigh->Checked = true; break;
        case IDLE_PRIORITY_CLASS: iLow->Checked = true; break;
        case NORMAL_PRIORITY_CLASS: iNormal->Checked = true; break;
        case REALTIME_PRIORITY_CLASS: iRealTime->Checked = true;
    }*/
    TPoint p = ProcessTree->ClientToScreen(TPoint(X, Y));
    ProcessPopup->Popup(p.x, p.y);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iMemInfoClick(TObject *Sender)
{
	iMemInfo->Checked ^= true;
    if (iMemInfo->Checked)
    {
		MemForm->SetProcess(GetProcess());
		MemForm->Show();
	}
	else
		MemForm->Hide();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iViewMemClick(TObject *Sender)
{
    try
    {
        TViewMemForm *frm = new TViewMemForm(this, GetProcess());
        frm->Show();
    }
    catch (int)
    {
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessListDeletion(TObject *Sender,
      TListItem *Item)
{
    delete (TMyProcess *)Item->Data;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iSecurityClick(TObject *Sender)
{
    TMyProcess *prc = GetProcess();
    if (!EditProcessSecurity(Handle, prc->pid, prc->ProcessName))
        ShowGUIError(Handle,
        " /   ");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iThreadsListClick(TObject *Sender)
{
	iThreadsList->Checked ^= true;
    if (iThreadsList->Checked)
    {
		ThreadsForm->SetProcess(GetProcess());
		ThreadsForm->Show();
    }
    else
        ThreadsForm->Hide();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iResetWorkingSetSizeClick(TObject *Sender)
{
    TMyProcess *prc = GetProcess();
    HANDLE hProcess = OpenProcess(PROCESS_SET_QUOTA, FALSE, prc->pid);
    if (!hProcess || !_SetProcessWorkingSetSize(hProcess, -1, -1))
        Application->MessageBox("  Working Set Size ",
            "", MB_OK | MB_ICONEXCLAMATION);
    else CloseHandle(hProcess);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iChangeWorkingSetClick(TObject *Sender)
{
    TMyProcess *prc = GetProcess();
    HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION |
        PROCESS_SET_QUOTA, FALSE, prc->pid);
    if (!hProcess)
    {
        ShowGUIError(Handle, "  ");
        return;
    }
    ChangeWorkingSetDlg->MinSize = 0;
    ChangeWorkingSetDlg->MaxSize = 0;
    _GetProcessWorkingSetSize(hProcess,
        &ChangeWorkingSetDlg->MinSize,
        &ChangeWorkingSetDlg->MaxSize);
    if (ChangeWorkingSetDlg->ShowModal() != mrOk)
    {
        CloseHandle(hProcess);
        return;
    }
    if (!_SetProcessWorkingSetSize(hProcess,
        ChangeWorkingSetDlg->MinSize, ChangeWorkingSetDlg->MaxSize))
        ShowGUIError(Handle, "  Working Set Size ");
    CloseHandle(hProcess);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iPropertiesClick(TObject *Sender)
{
    TMyProcess *prc = GetProcess();
	ShowFileProperties(prc->ImagePath);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iModsList2Click(TObject *Sender)
{
	ModsForm->SetProcess(GetProcess());
	ModsForm->Show();
	iModsList->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iThreadsList2Click(TObject *Sender)
{
	ThreadsForm->SetProcess(GetProcess());
	ThreadsForm->Show();
	iThreadsList->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iMemInfo2Click(TObject *Sender)
{
	MemForm->SetProcess(GetProcess());
	MemForm->Show();
	iMemInfo->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iKillSamePrcClick(TObject *Sender)
{
	AnsiString name;
	if (PageControl->ActivePage == ProcessesTab)
		name = ProcessList->Selected->Caption;
	else if (PageControl->ActivePage == ProcessTreeTab)
		name = ProcessTree->Selected->Text;
	if (Application->MessageBox(("     \"" +
		name + "\"?").c_str(),
		"", MB_YESNO | MB_ICONWARNING) != IDYES) return;
	int i = 0, k = ProcessList->Items->Count;
	for (; i < k; i++)
	{
		if (!strcmpi(ProcessList->Items->Item[i]->Caption.c_str(), name.c_str()))
		{
			((TMyProcess*)ProcessList->Items->Item[i]->Data)->Kill();
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iModSearchClick(TObject *Sender)
{
    ModSearchDlg->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessPopupPopup(TObject *Sender)
{
	bool bSelected = GetProcess() != NULL;
    TerminateBtn->Enabled = bSelected && GetProcess()->pid;
    if (!ProcessList->Selected)
    {
        if (ModsForm->Visible) ModsForm->SetProcess(NULL);
        if (MemForm->Visible) MemForm->SetProcess(NULL);
        if (ThreadsForm->Visible) ThreadsForm->SetProcess(NULL);
    }
    for (int i = 0; i < ProcessPopup->Items->Count; i++)
    {
    	ProcessPopup->Items->Items[i]->Enabled = bSelected;
    }
	if (bSelected)
    {
        switch (GetProcess()->BasePriority)
        {
            case ABOVE_NORMAL_PRIORITY_CLASS: iAboveNormal->Checked = true; break;
            case BELOW_NORMAL_PRIORITY_CLASS: iBelowNormal->Checked = true; break;
            case HIGH_PRIORITY_CLASS: iHigh->Checked = true; break;
            case IDLE_PRIORITY_CLASS: iLow->Checked = true; break;
            case NORMAL_PRIORITY_CLASS: iNormal->Checked = true; break;
            case REALTIME_PRIORITY_CLASS: iRealTime->Checked = true;
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iSnapshotClick(TObject *Sender)
{
    HMODULE hMod = LoadLibrary("dbghelp.dll");
    if (!hMod)
    {
    	Application->MessageBox(
        	"   DBGHELP.DLL",
            Application->Title.c_str(),
            MB_OK | MB_ICONSTOP);
    	return;
    }
	char szFileName[MAX_PATH] = {0};
    GetModuleFileName(hMod, szFileName, sizeof(szFileName));
	CModuleVersion ver(szFileName);
    if (!ver || (ver.FileVersion >> 32) < 0x00050002)
    {
    	Application->MessageBox(
        	"   DBGHELP.DLL",
            Application->Title.c_str(),
            MB_OK | MB_ICONSTOP);
        FreeLibrary(hMod);
    	return;
    }

    TMyProcess *prc = GetProcess();
    AnsiString dir = ExtractFilePath(prc->ImagePath);

   	DWORD				dwPID = prc->pid;
	HANDLE				hProcess = OpenProcess(PROCESS_ALL_ACCESS,FALSE,dwPID);
	char*				snapshot_str = new char[65536];//[40960] = {0};
    ZeroMemory(snapshot_str, 65536);
	PSTR				pStr = snapshot_str;

/*    HMODULE hMod = LoadLibrary("imagehlp.dll");
    if (!hMod)
    {
    	Application->MessageBox(
        	"   IMAGEHLP.DLL",
            Application->Title.c_str(),
            MB_OK | MB_ICONSTOP);
    	return;
    }
    LPCVOID proc = GetProcAddress(hMod, "SymEnumSymbols");
    if (!proc)
    {
    	Application->MessageBox(
        	"   IMAGEHLP.DLL\n .",
            Application->Title.c_str(),
            MB_OK | MB_ICONSTOP);
    	return;
    }*/

	SymSetOptions(SymGetOptions() | SYMOPT_DEFERRED_LOADS | SYMOPT_LOAD_LINES);
	SymInitialize(hProcess, dir.c_str(), TRUE);



	//CallStack  
	{
		THREADENTRY32	te		= { sizeof(THREADENTRY32) };
		HANDLE			hSnapShot = CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD,dwPID);
		if (hSnapShot)
		{
			__try
			{
				if (Thread32First(hSnapShot,&te))
				{
					do{
						if (te.th32OwnerProcessID!=dwPID)
						{
							continue;
						}

						HANDLE		hThread = OpenThread(THREAD_ALL_ACCESS,FALSE,te.th32ThreadID);
						CONTEXT		thContext = {0};

						if (!hThread)
							continue;

						if (te.th32ThreadID != GetCurrentThreadId())
							if (SuspendThread(hThread) == (DWORD)-1)
							{
								CloseHandle(hThread);
								continue;
							}

							if (te.th32ThreadID!=GetCurrentThreadId())
							{
								thContext.ContextFlags = CONTEXT_FULL;
								if (!GetThreadContext(hThread,&thContext))
                                	continue;
							}
							else
                            	continue;
//  								thContext = *pEC;

							pStr += sprintf(pStr,"==================================="
								"Thread ID=0x%p(%i)"
								"==================================="
								"\r\n\r\n",
								te.th32ThreadID,
								te.th32ThreadID
								);

							//call stack for thread
							pStr = WriteMiniCallStack(pStr,thContext.Ebp,dwPID,hProcess);
							pStr += sprintf(pStr,"\r\n");

							pStr = WriteCallStackDetails(pStr, dwPID, hProcess, hThread,&thContext,true);
							pStr += sprintf(pStr,"\r\n");

							if (te.th32ThreadID != GetCurrentThreadId())
								ResumeThread(hThread);

							pStr += sprintf(pStr,"==================================="
								"=========================="
								"==================================="
								"\r\n\r\n"
								);

							CloseHandle(hThread);
					}while(Thread32Next(hSnapShot,&te));
				}
			}
			__finally
			{
				CloseHandle(hSnapShot);
			}
		}
	}

    SymCleanup(hProcess);
    CloseHandle(hProcess);

	SnapshotForm->Memo->Lines->Text = snapshot_str;
    SnapshotForm->Caption = AnsiString("  ") + prc->ProcessName;
    SnapshotForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iAboutClick(TObject *Sender)
{
    AboutDlg->Position = poOwnerFormCenter;
	AboutDlg->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iViewSrcClick(TObject *Sender)
{
	FrmViewSrc->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessListMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if (!ProcessList->Selected && ModsForm->Visible)
        ModsForm->SetProcess(NULL);
    if (Button != mbRight) return;
    TListItem *li = ProcessList->Selected;
    if (!li || !((TMyProcess *)li->Data)->pid) return;
    TPoint p = ProcessList->ClientToScreen(TPoint(X, Y));
    ProcessPopup->Popup(p.x, p.y);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SubClassWndProc(Messages::TMessage &Message)
{
    if (Message.Msg == WM_TRAYICON && Message.WParam == 1)
    {
    	switch (Message.LParam)
        {
        	case WM_RBUTTONDOWN:
            	SetForegroundWindow(Handle);
				PopupMenu1->Popup(Mouse->CursorPos.x, Mouse->CursorPos.y);
            	break;
        	case WM_LBUTTONDBLCLK:
		        WndProc(Message);
            	iRestoreClick(NULL);
                break;
        }
    }
    else
    {
        WndProc(Message);
        if (Message.Msg == WM_SYSCOMMAND &&
            Message.WParam == SC_MINIMIZE &&
            iHide->Checked)
        {
restore:
        	iRestore->Visible = true;
            ShowWindow(Application->Handle, SW_HIDE);
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iRestoreClick(TObject *Sender)
{
   	iRestore->Visible = false;
   	SetForegroundWindow(Handle);
	ShowWindow(Application->Handle, SW_SHOWNORMAL);
//	WindowState = wsNormal;    
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdatePerfTab()
{
	memmove(cpu_graph + 1, cpu_graph, (MAX_GRAPH_WIDTH - 1) * sizeof(double));
	cpu_graph[0] = g_CPUUsage / 100;
	memmove(mem_graph + 1, mem_graph, (MAX_GRAPH_WIDTH - 1) * sizeof(double));
    if (!CBSelProcesses->Checked)
    {
    	g_MemUsage = ms.dwTotalPageFile - ms.dwAvailPageFile;
    }
	mem_graph[0] = double(g_MemUsage) / ms.dwTotalPageFile;
    if (PageControl->ActivePage == PerfTab)
    {
		RepaintGraph(PBCPUUse, cpu_graph, RGB(0, 255, 0));
		RepaintGraph(PBMemUse, mem_graph, RGB(255, 255, 0));
    }
    nGridOffs++;
    nGridOffs %= GRID_SPACE_HOR;
    CustomProgressBar1->cur_val = g_CPUUsage;
	CustomProgressBar2->max_val = ms.dwTotalPageFile / 1024;
	CustomProgressBar2->cur_val = g_MemUsage / 1024;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateAdvancedTab()
{
	if (PageControl->ActivePage == AdvancedTab)
    {
        LDescriptors->Caption = g_NumHandles;
        LThreads->Caption = g_NumThreads;
        LProcesses->Caption = g_NumProcesses;

        LTotalPM->Caption = ms.dwTotalPhys / 1024;
        LAvailPM->Caption = ms.dwAvailPhys / 1024;
        LCachePM->Caption = SysCacheInfo.TransitionSharedPages * 4;

        LCommitTotal->Caption = SysPerfInfo.CommittedPages * 4;
        LCommitLimit->Caption = SysPerfInfo.CommitLimit * 4;
        LCommitPeak->Caption = SysPerfInfo.PeakCommitment * 4;

        LKernelTotal->Caption = (SysPerfInfo.PagedPoolPages + SysPerfInfo.NonPagedPoolPages) * 4;
        LKernelPaged->Caption = SysPerfInfo.PagedPoolPages * 4;
        LKernelNP->Caption = SysPerfInfo.NonPagedPoolPages * 4;

        SYSTEMTIME st;
        FileTimeToSystemTime((FILETIME*)&SysPerfInfo.IdleProcessTime, &st);

        UpdateAdvPerfInfo(0, "%d:%02d:%02d.%03d",
        	(int)(*(__int64*)&SysPerfInfo.IdleProcessTime / 36000000000),
            (int)((*(__int64*)&SysPerfInfo.IdleProcessTime % 36000000000) / 600000000),
            (int)(((*(__int64*)&SysPerfInfo.IdleProcessTime % 36000000000) % 600000000) / 10000000),
            (int)((((*(__int64*)&SysPerfInfo.IdleProcessTime % 36000000000) % 600000000) % 10000000) / 10000));
        UpdateAdvPerfInfo(1, "%Lu", SysPerfInfo.IoReadTransferCount);
        UpdateAdvPerfInfo(2, "%Lu", SysPerfInfo.IoWriteTransferCount);
        UpdateAdvPerfInfo(3, "%Lu", SysPerfInfo.IoOtherTransferCount);
        UpdateAdvPerfInfo(4, "%u", SysPerfInfo.IoReadOperationCount);
        UpdateAdvPerfInfo(5, "%u", SysPerfInfo.IoWriteOperationCount);
        UpdateAdvPerfInfo(6, "%u", SysPerfInfo.IoOtherOperationCount);
        UpdateAdvPerfInfo(7, "%u", SysPerfInfo.AvailablePages);
        UpdateAdvPerfInfo(8, "%u", SysPerfInfo.CommittedPages);
        UpdateAdvPerfInfo(9, "%u", SysPerfInfo.CommitLimit);
        UpdateAdvPerfInfo(10, "%u", SysPerfInfo.PeakCommitment);
        UpdateAdvPerfInfo(11, "%u", SysPerfInfo.PageFaultCount);
        UpdateAdvPerfInfo(12, "%u", SysPerfInfo.CopyOnWriteCount);
        UpdateAdvPerfInfo(13, "%u", SysPerfInfo.TransitionCount);
        UpdateAdvPerfInfo(14, "%u", SysPerfInfo.CacheTransitionCount);
        UpdateAdvPerfInfo(15, "%u", SysPerfInfo.DemandZeroCount);
        UpdateAdvPerfInfo(16, "%u", SysPerfInfo.PageReadCount);
        UpdateAdvPerfInfo(17, "%u", SysPerfInfo.PageReadIoCount);
        UpdateAdvPerfInfo(18, "%u", SysPerfInfo.CacheReadCount);
        UpdateAdvPerfInfo(19, "%u", SysPerfInfo.CacheIoCount);
        UpdateAdvPerfInfo(20, "%u", SysPerfInfo.DirtyPagesWriteCount);
        UpdateAdvPerfInfo(21, "%u", SysPerfInfo.DirtyWriteIoCount);
        UpdateAdvPerfInfo(22, "%u", SysPerfInfo.MappedPagesWriteCount);
        UpdateAdvPerfInfo(23, "%u", SysPerfInfo.MappedWriteIoCount);
        UpdateAdvPerfInfo(24, "%u", SysPerfInfo.PagedPoolPages);
        UpdateAdvPerfInfo(25, "%u", SysPerfInfo.NonPagedPoolPages);
        UpdateAdvPerfInfo(26, "%u", SysPerfInfo.PagedPoolAllocs);
        UpdateAdvPerfInfo(27, "%u", SysPerfInfo.PagedPoolFrees);
        UpdateAdvPerfInfo(28, "%u", SysPerfInfo.NonPagedPoolAllocs);
        UpdateAdvPerfInfo(29, "%u", SysPerfInfo.NonPagedPoolFrees);
        UpdateAdvPerfInfo(30, "%u", SysPerfInfo.FreeSystemPtes);
        UpdateAdvPerfInfo(31, "%u", SysPerfInfo.ResidentSystemCodePage);
        UpdateAdvPerfInfo(32, "%u", SysPerfInfo.TotalSystemDriverPages);
        UpdateAdvPerfInfo(33, "%u", SysPerfInfo.TotalSystemCodePages);
        UpdateAdvPerfInfo(34, "%u", SysPerfInfo.Spare0Count);
        UpdateAdvPerfInfo(35, "%u", SysPerfInfo.Spare1Count);
        UpdateAdvPerfInfo(36, "%u", SysPerfInfo.Spare3Count);
        UpdateAdvPerfInfo(37, "%u", SysPerfInfo.ResidentSystemCachePage);
        UpdateAdvPerfInfo(38, "%u", SysPerfInfo.ResidentPagedPoolPage);
        UpdateAdvPerfInfo(39, "%u", SysPerfInfo.ResidentSystemDriverPage);
        UpdateAdvPerfInfo(40, "%u", SysPerfInfo.CcFastReadNoWait);
        UpdateAdvPerfInfo(41, "%u", SysPerfInfo.CcFastReadWait);
        UpdateAdvPerfInfo(42, "%u", SysPerfInfo.CcFastReadResourceMiss);
        UpdateAdvPerfInfo(43, "%u", SysPerfInfo.CcFastReadNotPossible);
        UpdateAdvPerfInfo(44, "%u", SysPerfInfo.CcFastMdlReadNoWait);
        UpdateAdvPerfInfo(45, "%u", SysPerfInfo.CcFastMdlReadWait);
        UpdateAdvPerfInfo(46, "%u", SysPerfInfo.CcFastMdlReadResourceMiss);
        UpdateAdvPerfInfo(47, "%u", SysPerfInfo.CcFastMdlReadNotPossible);
        UpdateAdvPerfInfo(48, "%u", SysPerfInfo.CcMapDataNoWait);
        UpdateAdvPerfInfo(49, "%u", SysPerfInfo.CcMapDataWait);
        UpdateAdvPerfInfo(50, "%u", SysPerfInfo.CcMapDataNoWaitMiss);
        UpdateAdvPerfInfo(51, "%u", SysPerfInfo.CcMapDataWaitMiss);
        UpdateAdvPerfInfo(52, "%u", SysPerfInfo.CcPinMappedDataCount);
        UpdateAdvPerfInfo(53, "%u", SysPerfInfo.CcPinReadNoWait);
        UpdateAdvPerfInfo(54, "%u", SysPerfInfo.CcPinReadWait);
        UpdateAdvPerfInfo(55, "%u", SysPerfInfo.CcPinReadNoWaitMiss);
        UpdateAdvPerfInfo(56, "%u", SysPerfInfo.CcPinReadWaitMiss);
        UpdateAdvPerfInfo(57, "%u", SysPerfInfo.CcCopyReadNoWait);
        UpdateAdvPerfInfo(58, "%u", SysPerfInfo.CcCopyReadWait);
        UpdateAdvPerfInfo(59, "%u", SysPerfInfo.CcCopyReadNoWaitMiss);
        UpdateAdvPerfInfo(60, "%u", SysPerfInfo.CcCopyReadWaitMiss);
        UpdateAdvPerfInfo(61, "%u", SysPerfInfo.CcMdlReadNoWait);
        UpdateAdvPerfInfo(62, "%u", SysPerfInfo.CcMdlReadWait);
        UpdateAdvPerfInfo(63, "%u", SysPerfInfo.CcMdlReadNoWaitMiss);
        UpdateAdvPerfInfo(64, "%u", SysPerfInfo.CcMdlReadWaitMiss);
        UpdateAdvPerfInfo(65, "%u", SysPerfInfo.CcReadAheadIos);
        UpdateAdvPerfInfo(66, "%u", SysPerfInfo.CcLazyWriteIos);
        UpdateAdvPerfInfo(67, "%u", SysPerfInfo.CcLazyWritePages);
        UpdateAdvPerfInfo(68, "%u", SysPerfInfo.CcDataFlushes);
        UpdateAdvPerfInfo(69, "%u", SysPerfInfo.CcDataPages);
        UpdateAdvPerfInfo(70, "%u", SysPerfInfo.ContextSwitches);
        UpdateAdvPerfInfo(71, "%u", SysPerfInfo.FirstLevelTbFills);
        UpdateAdvPerfInfo(72, "%u", SysPerfInfo.SecondLevelTbFills);
        UpdateAdvPerfInfo(73, "%u", SysPerfInfo.SystemCalls);
    }
}
//---------------------------------------------------------------------------
void __cdecl TMainForm::UpdateAdvPerfInfo(int nCol, const char* pszFmt, ...)
{
	char szBuf[256];
    va_list	argList;
    va_start(argList, pszFmt);
	_vsnprintf(szBuf, sizeof(szBuf), pszFmt, argList);
	va_end(argList);
    TStrings* ps = LVAdvPerf->Items->Item[nCol]->SubItems;
    if (strcmp(szBuf, ps->Strings[0].c_str()))
    {
    	ps->Strings[0] = szBuf;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PBCPUUsePaint(TObject *Sender)
{
	RepaintGraph(PBCPUUse, cpu_graph, RGB(0, 255, 0));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PBMemUsePaint(TObject *Sender)
{
	RepaintGraph(PBMemUse, mem_graph, RGB(255, 255, 0));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RepaintGraph(TPaintBox* pBox, double* pData, COLORREF color)
{
	HDC hDC = pBox->Canvas->Handle;
	HDC hMemDC = CreateCompatibleDC(hDC);
    HBITMAP hBM = CreateCompatibleBitmap(hDC, pBox->Width, pBox->Height);
    HPEN hPen = CreatePen(PS_SOLID, 1, RGB(0, 128, 64));
    HPEN hPen2 = CreatePen(PS_SOLID, 1, color);
    RECT r;

    r.left = 0;
    r.top = 0;
    r.right = pBox->Width;
    r.bottom = pBox->Height;

    SelectObject(hMemDC, hBM);
    FillRect(hMemDC, &r, (HBRUSH)GetStockObject(BLACK_BRUSH));

    SelectObject(hMemDC, hPen);

    for (int x = -nGridOffs; x < pBox->Width; x += GRID_SPACE_HOR)
    {
    	MoveToEx(hMemDC, x, 0, NULL);
        LineTo(hMemDC, x, pBox->Height);
    }

    for (int y = GRID_SPACE_VERT; y < pBox->Height; y += GRID_SPACE_VERT)
    {
    	MoveToEx(hMemDC, 0, y, NULL);
        LineTo(hMemDC, pBox->Width, y);
    }

    SelectObject(hMemDC, hPen2);

    int k = (pBox->Width < MAX_GRAPH_WIDTH) ? pBox->Width : MAX_GRAPH_WIDTH;
    for (int x = 0; x < k; x++)
    {
    	int y = r.bottom - pData[k - x - 1] * r.bottom;
    	if (!x)
        {
        	MoveToEx(hMemDC, x, y, NULL);
        }
        else
        {
       		LineTo(hMemDC, x, y);
        }
    }

    BitBlt(pBox->Canvas->Handle, 0, 0, r.right, r.bottom, hMemDC, 0, 0, SRCCOPY);

    DeleteDC(hMemDC);
    DeleteObject(hBM);
    DeleteObject(hPen);
    DeleteObject(hPen2);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormResize(TObject *Sender)
{
    int nHeight = CBSelProcesses->Top - GBCPUUse->Top - 5 - 7;
    GBMemUse->Width = PerfTab->Width - 111;
    GBCPUUse->Width = PerfTab->Width - 111;
	GBMemUse->Top = GBCPUUse->Top + nHeight / 2 + 5;
    CustomProgressBar2->Top = GBCPUUse->Top + nHeight / 2 + 5;
    GBCPUUse->Height = nHeight / 2;
	GBMemUse->Height = nHeight / 2;
    CustomProgressBar1->Height = nHeight / 2;
    CustomProgressBar2->Height = nHeight / 2;
}
//---------------------------------------------------------------------------
int __stdcall EnumProc(char *lpszStr, void *lpParam)
{
	SIZE size;
	HWND hWnd = (HWND)lpParam;
	HDC hDC = GetDC(hWnd);
	GetTextExtentPoint(hDC, lpszStr, strlen(lpszStr), &size);
	ReleaseDC(hWnd, hDC);
	if (SendMessage(hWnd, LB_GETHORIZONTALEXTENT, 0, 0) < size.cx)
	{
		SendMessage(hWnd, LB_SETHORIZONTALEXTENT, size.cx, 0);
	}
	SendMessage(hWnd, LB_ADDSTRING, 0, (LPARAM)lpszStr);
	return 1;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateProcessorInfo()
{
    char szBuf[256];
    PROCESSOR_SN sn;
    PROCESSOR_SIGNATURE sig;

    GetProcessorType(szBuf);
    CPUID_GetProcessorSignature(&sig);
    AnsiString str(szBuf);
    str.Trim();
    LProcType->Caption = str;
    sprintf(szBuf, "%d", sig.FamilyCode);
    LFamily->Caption = szBuf;
    sprintf(szBuf, "%d", sig.ModelNumber);
    LModel->Caption = szBuf;
    sprintf(szBuf, "%d", sig.SteppingID);
    LStepping->Caption = szBuf;
    CPUID_GetVendorID(szBuf);
    LVendor->Caption = szBuf;
    if (IsIntel())
    {
        CPUID_GetSerialNumber(&sn);
        sprintf(szBuf, "%04X-%04X-%04X-%04X-%04X-%04X",
            sn.Part6, sn.Part5, sn.Part4, sn.Part3, sn.Part2, sn.Part1);
    }
    else
    {
        strcpy(szBuf, "");
    }
    LSerialNo->Caption = szBuf;

	EnumProcessorFeaturesAll(EnumProc, LBFeatures->Handle);
	EnumProcessorCacheInfo(EnumProc, LBCache->Handle);

    if (IsAMD())
    {
        EnumAMDPowerFeatures(EnumProc, LBPowerFeatures->Handle);
        if (CPUID_GetMaxFuncNumberEx() >= 0x80000008)
        {
            char szBuf[32];
            AMD_ADDRESS_RANGE_INFO addr;
            CPUID_GetAMDAddressRangeInfo(&addr);
            sprintf(szBuf, "%d ", (int)addr.MaxPhysAddrWidth);
			LMaxPARange->Caption = szBuf;
            sprintf(szBuf, "%d ", (int)addr.MaxVirtAddrWidth);
			LMaxVARange->Caption = szBuf;
        }
    }
    else
    {
        LBPowerFeatures->Enabled = false;
        LMaxPARange->Caption = "";
        LMaxVARange->Caption = "";
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iHandleListClick(TObject *Sender)
{
	FrmHandles->SetProcess(GetProcess());
	FrmHandles->Show();
	iHandles->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::iHandlesClick(TObject *Sender)
{
	iHandles->Checked ^= true;
    if (iHandles->Checked)
    {
		FrmHandles->SetProcess(GetProcess());
		FrmHandles->Show();
	}
	else
		FrmHandles->Hide();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateActions()
{
	TMyProcess* prc = GetProcess();
	bool bEnabled = prc && prc->pid;
    iModsList->Enabled = bEnabled;
    iThreadsList->Enabled = bEnabled;
    iMemInfo->Enabled = bEnabled;
    iHandles->Enabled = prc;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PerfTabShow(TObject *Sender)
{
	FormResize(NULL);
}
//---------------------------------------------------------------------------

