#include "MyProcess.h"

TMyProcess::TMyProcess(PSYSTEM_PROCESSES p)
{
    BasePriority = 0;
    USERObjects = 0;
    GDIObjects = 0;
    CPUUse = 0;
    OldTime = GetTickCount();
    OldCPUTime = UserTime.QuadPart + KernelTime.QuadPart;
    ZeroMemory(&CreateTime, sizeof(CreateTime));
    ZeroMemory(UserName, sizeof(UserName));
    ZeroMemory(ImagePath, sizeof(ImagePath));
    ZeroMemory(ProcessName, sizeof(ProcessName));
    Threads = NULL;

    pid = p->ProcessId;
    if (IsWinNT())
    {
        GetProcessUserName(pid, UserName);
        GetProcessModulePath(pid, ImagePath);
    }
#ifndef UNICODE
    WideCharToMultiByte(CP_ACP, 0, p->ProcessName.Buffer,
        -1, ProcessName, MAX_PATH, NULL, NULL);
#else
    lstrcpy(ProcessName, p->ProcessName.Buffer);
#endif
    if (!IsWinNT())
    {
        lstrcpy(ImagePath, ProcessName);
        TCHAR *p = ProcessName + lstrlen(ProcessName);
        while (p > ProcessName && *p != '\\') p--;
        if (*p == '\\') p++;
        memmove(ProcessName, p, lstrlen(p) + 1);
    }
    if (!ProcessName[0])
        lstrcpy(ProcessName, _T("System Idle Process"));
}

TMyProcess::~TMyProcess()
{
    if (Threads) LocalFree(Threads);
}

void TMyProcess::UpdateInfo(PSYSTEM_PROCESSES p)
{
    DWORD (WINAPI *_GetGuiResources)(HANDLE, DWORD);
    DWORD t, d, CPUTime;
    FILETIME lt;
    HANDLE hProcess, hUser;

    ThreadCount = p->ThreadCount;
    UserTime = p->UserTime;
    KernelTime = p->KernelTime;
    BasePriority = p->BasePriority;
    ParentPid = p->InheritedFromProcessId;
    HandleCount = p->HandleCount;
    VmCounters = p->VmCounters;
    IoCounters = p->IoCounters;

    t = GetTickCount();
    d = t - OldTime;
    CPUTime = UserTime.QuadPart + KernelTime.QuadPart;
    CPUUse = d ? (CPUTime - OldCPUTime) / d : 0;
    if (CPUUse > 9999) CPUUse = 9999;
    OldCPUTime = CPUTime;
    OldTime = t;

    FileTimeToLocalFileTime((LPFILETIME)&p->CreateTime, &lt);
    FileTimeToSystemTime(&lt, &CreateTime);

    if (Threads)
    {
        LocalFree(Threads);
        Threads = NULL;
    }
    if (ThreadCount)
    {
        DWORD dwSize = ThreadCount * sizeof(SYSTEM_THREADS);
        Threads = (PSYSTEM_THREADS)LocalAlloc(
            LMEM_FIXED | LMEM_ZEROINIT, dwSize);
        memcpy(Threads, p->Threads, dwSize);
    }

    if ((hProcess = OpenProcess(PROCESS_QUERY_INFORMATION,
        FALSE, pid)) == NULL) return;

    BasePriority = GetPriorityClass(hProcess);

    hUser = GetModuleHandle(_T("User32.dll"));
    if (hUser && (*(FARPROC *)&_GetGuiResources =
        GetProcAddress(hUser, "GetGuiResources")) != NULL)
    {
        USERObjects = _GetGuiResources(hProcess, GR_USEROBJECTS);
        GDIObjects = _GetGuiResources(hProcess, GR_GDIOBJECTS);
    }

    CloseHandle(hProcess);
}

BOOL TMyProcess::Kill()
{
    return KillProcess(pid);
}

BOOL TMyProcess::SetPriority(DWORD NewPriority)
{
    HANDLE hProc = OpenProcess(PROCESS_SET_INFORMATION, FALSE, pid);
    if (!hProc) return FALSE;
    if (!SetPriorityClass(hProc, NewPriority))
    {
        CloseHandle(hProc);
        return FALSE;
    }
    CloseHandle(hProc);
    return TRUE;
}

