//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdio.h>
#include <map>
#include <vector>

typedef struct _UNICODE_STRING {
    USHORT Length;
    USHORT MaximumLength;
    PWSTR Buffer;
} UNICODE_STRING;

#include "PerfInfo.h"
#pragma hdrstop
//--------------------- NT
#define SystemBasicInformation 0
#define SystemPerformanceInformation 2
#define SystemTimeInformation 3
#define SystemObjectInformation 17
#define SystemCacheInformation 21
#define NT_SUCCESS(Status)                      ((NTSTATUS)(Status) >= 0)
#define STATUS_INFO_LENGTH_MISMATCH             ((NTSTATUS)0xC0000004L)

#define Li2Double(x) ((double)((x).HighPart) * 4.294967296E9 + (double)((x).LowPart))

typedef struct
{
DWORD dwUnknown1;
ULONG uKeMaximumIncrement;
ULONG uPageSize;
ULONG uMmNumberOfPhysicalPages;
ULONG uMmLowestPhysicalPage;
ULONG uMmHighestPhysicalPage;
ULONG uAllocationGranularity;
PVOID pLowestUserAddress;
PVOID pMmHighestUserAddress;
ULONG uKeActiveProcessors;
BYTE bKeNumberProcessors;
BYTE bUnknown2;
WORD wUnknown3;
} SYSTEM_BASIC_INFORMATION;

typedef struct
{
LARGE_INTEGER liKeBootTime;
LARGE_INTEGER liKeSystemTime;
LARGE_INTEGER liExpTimeZoneBias;
ULONG uCurrentTimeZoneId;
DWORD dwReserved;
} SYSTEM_TIME_INFORMATION;


// ntdll!NtQuerySystemInformation (NT specific!)
//
// The function copies the system information of the
// specified type into a buffer
//
// NTSYSAPI
// NTSTATUS
// NTAPI
// NtQuerySystemInformation(
// IN UINT SystemInformationClass, // information type
// OUT PVOID SystemInformation, // pointer to buffer
// IN ULONG SystemInformationLength, // buffer size in bytes
// OUT PULONG ReturnLength OPTIONAL // pointer to a 32-bit
// // variable that receives
// // the number of bytes
// // written to the buffer
// );
typedef LONG (WINAPI *PROCNTQSI)(UINT,PVOID,ULONG,PULONG);
PROCNTQSI NtQuerySystemInformation;
//--------------------- prod
SYSTEM_PERFORMANCE_INFORMATION SysPerfInfo;
SYSTEM_TIME_INFORMATION SysTimeInfo;
SYSTEM_BASIC_INFORMATION SysBaseInfo;
SYSTEM_CACHE_INFORMATION SysCacheInfo;

typedef enum _OBJECT_INFORMATION_CLASS {
	ObjectBasicInformation,				// 0	Y		N
	ObjectNameInformation,				// 1	Y		N
	ObjectTypeInformation,				// 2	Y		N
	ObjectAllTypesInformation,			// 3	Y		N
	ObjectHandleInformation				// 4	Y		Y
} OBJECT_INFORMATION_CLASS;

typedef
NTSYSAPI
NTSTATUS
(NTAPI *_ZwQueryObject)(
	IN HANDLE ObjectHandle,
    IN OBJECT_INFORMATION_CLASS ObjectInformationClass,
    OUT PVOID ObjectInformation,
    IN ULONG ObjectInformationLength,
    OUT PULONG ReturnLength OPTIONAL);

typedef
NTSYSAPI
NTSTATUS
(NTAPI *_ZwDuplicateObject)(
	IN HANDLE SourceProcessHandle,
	IN HANDLE SourceHandle,
    IN HANDLE TargetProcessHandle,
    OUT PHANDLE TargetHandle OPTIONAL,
    IN ACCESS_MASK DesiredAccess,
    IN ULONG Attributes,
    IN ULONG Options);

#define DUPLICATE_SAME_ATTRIBUTES		3

_ZwQueryObject ZwQueryObject = NULL;
_ZwDuplicateObject ZwDuplicateObject = NULL;

double dbIdleTime;
double dbSystemTime;
LONG status;
LARGE_INTEGER liOldIdleTime = {0,0};
LARGE_INTEGER liOldSystemTime = {0,0};

ULONG GetSystemGlobalFlags()
{
	SYSTEM_GLOBAL_FLAG flags;
    if (!NT_SUCCESS(NtQuerySystemInformation(
    	SystemGlobalFlag,
        &flags,
        sizeof(flags),
        NULL)))
    {
    	return 0;
    }
    return flags.GlobalFlag;
}

bool PerfInit()
{

	HANDLE hNtDll = GetModuleHandle("ntdll");

    NtQuerySystemInformation = (PROCNTQSI)GetProcAddress(
    hNtDll,
    "NtQuerySystemInformation"
    );

    *(FARPROC*)&ZwQueryObject = GetProcAddress(hNtDll, "ZwQueryObject");
    *(FARPROC*)&ZwDuplicateObject = GetProcAddress(hNtDll, "ZwDuplicateObject");

    PVOID p;
    HANDLE hntos = GetModuleHandle("ntoskrnl.exe");
    int err = GetLastError();
	*(FARPROC*)&p = GetProcAddress(hntos, "ObReferenceObjectByPointer");

    if (!NtQuerySystemInformation)
    	return false;

    // get number of processors in the system
    status = NtQuerySystemInformation(SystemBasicInformation,&SysBaseInfo,sizeof(SysBaseInfo),NULL);
    if (status != NO_ERROR)
    	return false;
    return true;
}
//---------------------------------------------------------------------------
bool GetCPUUsage(double& val)
{
    // get new system time
    status = NtQuerySystemInformation(SystemTimeInformation,&SysTimeInfo,sizeof(SysTimeInfo),0);
    if (status!=NO_ERROR)
    	return false;

    // get new CPU's idle time
    status = NtQuerySystemInformation(SystemPerformanceInformation,&SysPerfInfo,sizeof(SysPerfInfo),NULL);
    if (status != NO_ERROR)
    	return false;

    // get system cache info
    status = NtQuerySystemInformation(SystemCacheInformation,&SysCacheInfo,sizeof(SysCacheInfo),NULL);
    if (status != NO_ERROR)
    	return false;

    // if it's a first call - skip it
    if (liOldIdleTime.QuadPart != 0)
    {
        // CurrentValue = NewValue - OldValue
        dbIdleTime = Li2Double(SysPerfInfo.IdleProcessTime) - Li2Double(liOldIdleTime);
        dbSystemTime = Li2Double(SysTimeInfo.liKeSystemTime) - Li2Double(liOldSystemTime);

        // CurrentCpuIdle = IdleTime / SystemTime
        dbIdleTime = dbIdleTime / dbSystemTime;

        // CurrentCpuUsage% = 100 - (CurrentCpuIdle * 100) / NumberOfProcessors
        dbIdleTime = 100.0 - dbIdleTime * 100.0 / (double)SysBaseInfo.bKeNumberProcessors + 0.5;

        val = dbIdleTime;
    }

    // store new CPU's idle and system time
    liOldIdleTime = SysPerfInfo.IdleProcessTime;
    liOldSystemTime = SysTimeInfo.liKeSystemTime;

    return true;
}
//---------------------------------------------------------------------------
struct OBJECTS_AND_TYPES {
	std::map<ULONG, PSYSTEM_OBJECT_TYPE_INFORMATION, std::less<ULONG> > types;
	std::map<PVOID, PSYSTEM_OBJECT_INFORMATION, std::less<PVOID> > objects;
};
//---------------------------------------------------------------------------
std::vector<SYSTEM_HANDLE_INFORMATION> GetHandles()
{
    ULONG n = 0x1000;
    PULONG p = new ULONG[n];

    NTSTATUS hr;
    while ((hr = NtQuerySystemInformation(
    	SystemHandleInformation,
        p,
        n * sizeof(*p),
        0)) == STATUS_INFO_LENGTH_MISMATCH)
    {
    	delete p;
        p = new ULONG[n *= 2];
    }

    if (!NT_SUCCESS(hr))
    {
    	delete p;
    	return std::vector<SYSTEM_HANDLE_INFORMATION>();
    }

	PSYSTEM_HANDLE_INFORMATION h = PSYSTEM_HANDLE_INFORMATION(p + 1);

    return std::vector<SYSTEM_HANDLE_INFORMATION>(h, h + *p);
}
//---------------------------------------------------------------------------
OBJECTS_AND_TYPES GetObjectsAndTypes()
{
    ULONG n = 0x1000, k = 0;
    PCHAR p = new CHAR[n];

    NTSTATUS hr;
    while ((hr = NtQuerySystemInformation(
    	SystemObjectInformation,
        p,
        n,
        0)) == STATUS_INFO_LENGTH_MISMATCH)
    {
    	delete p;
        p = new CHAR[n *= 2];
    }

    if (!NT_SUCCESS(hr))
    {
    	delete p;
    	return OBJECTS_AND_TYPES();
    }

    OBJECTS_AND_TYPES oats;

    for (
    	PSYSTEM_OBJECT_TYPE_INFORMATION t = PSYSTEM_OBJECT_TYPE_INFORMATION(p);
        ;
    	t = PSYSTEM_OBJECT_TYPE_INFORMATION(p + t->NextEntryOffset))
    {
    	oats.types[t->TypeNumber] = t;
        for (
        	PSYSTEM_OBJECT_INFORMATION o = PSYSTEM_OBJECT_INFORMATION(PCHAR(t->Name.Buffer) + t->Name.MaximumLength);
            ;
            o = PSYSTEM_OBJECT_INFORMATION(p + o->NextEntryOffset))
        {
        	oats.objects[o->Object] = o;
            if (o->NextEntryOffset == 0)
            {
            	break;
            }
        }
        if (t->NextEntryOffset == 0)
        {
            break;
        }
    }

    return oats;
}
//---------------------------------------------------------------------------
bool GetHandleInfo_Method1(DWORD dwPID, PMyHandleInfo* ppHandleInfo, LPDWORD pcbHandles)
{
	int k = 0;
    
    OBJECTS_AND_TYPES oats = GetObjectsAndTypes();
    std::vector<SYSTEM_HANDLE_INFORMATION>handles = GetHandles();
    SYSTEM_OBJECT_INFORMATION defobj = {0};

    PMyHandleInfo phi = new MyHandleInfo[handles.size()];

    for (
    	std::vector<SYSTEM_HANDLE_INFORMATION>::iterator h = handles.begin();
        h != handles.end(); h++)
    {
    	if (h->ProcessId == dwPID)
        {
        	PSYSTEM_OBJECT_TYPE_INFORMATION t = oats.types[h->ObjectTypeNumber];
            PSYSTEM_OBJECT_INFORMATION o = oats.objects[h->Object];

            if (o != NULL)
            {
                phi[k].ObjectTypeNumber = h->ObjectTypeNumber;
                phi[k].Flags = h->Flags;
                phi[k].Handle = h->Handle;
                phi[k].Object = h->Object;
                phi[k].GrantedAccess = h->GrantedAccess;
                phi[k].HandleCount = o->HandleCount;
                phi[k].PointerCount = o->PointerCount;
                phi[k].PagedPoolUsage = o->PagedPoolUsage;
                phi[k].NonPagedPoolUsage = o->NonPagedPoolUsage;
	            phi[k].TypeName = SysAllocString(t->Name.Buffer);
            	phi[k].ObjectName = SysAllocString(o->Name.Buffer);
                k++;
            }
        }
    }

    *ppHandleInfo = phi;
    *pcbHandles = k;
    
    return true;
}
//---------------------------------------------------------------------------
bool GetHandleInfo_Method2(DWORD dwPID, PMyHandleInfo* ppHandleInfo, LPDWORD pcbHandles)
{
	HANDLE hProcess = OpenProcess(PROCESS_DUP_HANDLE, FALSE, dwPID);
    if (!hProcess)
    {
    	return false;
    }

    ULONG n = 0x1000;
    PULONG p = new ULONG[n];

    NTSTATUS hr;
    while ((hr = NtQuerySystemInformation(
    	SystemHandleInformation,
        p,
        n * sizeof(*p),
        0)) == STATUS_INFO_LENGTH_MISMATCH)
    {
    	delete p;
        p = new ULONG[n *= 2];
    }

    if (!NT_SUCCESS(hr))
    {
    	delete p;
        CloseHandle(hProcess);
    	return false;
    }

	PSYSTEM_HANDLE_INFORMATION h = PSYSTEM_HANDLE_INFORMATION(p + 1);

    n = 0;
    for (ULONG i = 0; i < *p; i++)
    {
    	if (h[i].ProcessId == dwPID)
        {
        	n++;
        }
    }

	*ppHandleInfo = new MyHandleInfo[n];
    PMyHandleInfo phi = *ppHandleInfo;

    int k = 0;
    int fail_count = 0;
    for (ULONG i = 0; i < *p; i++)
    {
    	if (h[i].ProcessId == dwPID)
        {
        	HANDLE hObject;
            if (!NT_SUCCESS(ZwDuplicateObject(
            	hProcess,
                HANDLE(h[i].Handle),
                GetCurrentProcess(),
				&hObject,
                0,
                0,
                DUPLICATE_SAME_ACCESS)))
            {
            	fail_count++;
            	continue;
            }

            OBJECT_BASIC_INFORMATION obj;
            ZwQueryObject(
            	hObject,
                ObjectBasicInformation,
                &obj,
                sizeof(obj),
                &n);
            n = obj.TypeInformationLength + 2;

            POBJECT_TYPE_INFORMATION oti = POBJECT_TYPE_INFORMATION(new CHAR[n]);

            ZwQueryObject(
            	hObject,
                ObjectTypeInformation,
                oti,
                n,
                &n);

            n = obj.NameInformationLength == 0
            	? MAX_PATH * sizeof(WCHAR) : obj.NameInformationLength;

            POBJECT_NAME_INFORMATION oni = POBJECT_NAME_INFORMATION(new CHAR[n]);

            NTSTATUS rv = ZwQueryObject(
            	hObject,
                ObjectNameInformation,
                oni,
                n,
                &n);

			phi[k].ObjectTypeNumber = h[i].ObjectTypeNumber;
            phi[k].Flags = h[i].Flags;
            phi[k].Handle = h[i].Handle;
            phi[k].Object = h[i].Object;
            phi[k].GrantedAccess = h[i].GrantedAccess;
            phi[k].Attributes = obj.Attributes;
            phi[k].HandleCount = obj.HandleCount;
//            phi[k].PeakHandleCount = oti->PeakHandleCount;
            phi[k].PointerCount = obj.PointerCount;
//            phi[k].ObjectCount = oti->ObjectCount;
//            phi[k].PeakObjectCount = oti->PeakObjectCount;
//            phi[k].PoolType = oti->PoolType;
            phi[k].PagedPoolUsage = obj.PagedPoolUsage;
            phi[k].NonPagedPoolUsage = obj.NonPagedPoolUsage;
            phi[k].CreateTime = obj.CreateTime;
//            phi[k].InvalidAttributes = oti->InvalidAttributes;
//            phi[k].GenericMapping = oti->GenericMapping;
//            phi[k].ValidAccess = oti->ValidAccess;
//            phi[k].MaintainHandleDatabase = oti->MaintainHandleDatabase;
            phi[k].TypeName = SysAllocString(oti->Name.Buffer);

            if (NT_SUCCESS(rv))
            {
            	phi[k].ObjectName = SysAllocString(oni->Name.Buffer);
            }
            else
            {
            	phi[k].ObjectName = NULL;
            }

            delete oti;
            delete oni;
            CloseHandle(hObject);
            
            k++;
        }
    }

    *pcbHandles = k;

    delete p;
    CloseHandle(hProcess);

    return true;
}
//---------------------------------------------------------------------------
bool GetHandleInfo_Method3(DWORD dwPID, PMyHandleInfo* ppHandleInfo, LPDWORD pcbHandles)
{
	return true;
}
//---------------------------------------------------------------------------
bool GetHandleInfo(DWORD dwPID, PMyHandleInfo* ppHandleInfo, LPDWORD pcbHandles)
{
	if (GetSystemGlobalFlags() & FLG_MAINTAIN_OBJECT_TYPELIST)
    {
    	GetHandleInfo_Method1(dwPID, ppHandleInfo, pcbHandles);
    }
    else
    {
    	GetHandleInfo_Method2(dwPID, ppHandleInfo, pcbHandles);
    }
}
//---------------------------------------------------------------------------

