//---------------------------------------------------------------------------

#ifndef PerfInfoH
#define PerfInfoH
//---------------------------------------------------------------------------

typedef LONG NTSTATUS;

typedef LONG POOL_TYPE;

typedef struct _SYSTEM_PERFORMANCE_INFORMATION {
    LARGE_INTEGER IdleProcessTime;
    LARGE_INTEGER IoReadTransferCount;
    LARGE_INTEGER IoWriteTransferCount;
    LARGE_INTEGER IoOtherTransferCount;
    ULONG IoReadOperationCount;
    ULONG IoWriteOperationCount;
    ULONG IoOtherOperationCount;
    ULONG AvailablePages;
    ULONG CommittedPages;
    ULONG CommitLimit;
    ULONG PeakCommitment;
    ULONG PageFaultCount;
    ULONG CopyOnWriteCount;
    ULONG TransitionCount;
    ULONG CacheTransitionCount;
    ULONG DemandZeroCount;
    ULONG PageReadCount;
    ULONG PageReadIoCount;
    ULONG CacheReadCount;
    ULONG CacheIoCount;
    ULONG DirtyPagesWriteCount;
    ULONG DirtyWriteIoCount;
    ULONG MappedPagesWriteCount;
    ULONG MappedWriteIoCount;
    ULONG PagedPoolPages;
    ULONG NonPagedPoolPages;
    ULONG PagedPoolAllocs;
    ULONG PagedPoolFrees;
    ULONG NonPagedPoolAllocs;
    ULONG NonPagedPoolFrees;
    ULONG FreeSystemPtes;
    ULONG ResidentSystemCodePage;
    ULONG TotalSystemDriverPages;
    ULONG TotalSystemCodePages;
    ULONG Spare0Count;
    ULONG Spare1Count;
    ULONG Spare3Count;
    ULONG ResidentSystemCachePage;
    ULONG ResidentPagedPoolPage;
    ULONG ResidentSystemDriverPage;
    ULONG CcFastReadNoWait;
    ULONG CcFastReadWait;
    ULONG CcFastReadResourceMiss;
    ULONG CcFastReadNotPossible;
    ULONG CcFastMdlReadNoWait;
    ULONG CcFastMdlReadWait;
    ULONG CcFastMdlReadResourceMiss;
    ULONG CcFastMdlReadNotPossible;
    ULONG CcMapDataNoWait;
    ULONG CcMapDataWait;
    ULONG CcMapDataNoWaitMiss;
    ULONG CcMapDataWaitMiss;
    ULONG CcPinMappedDataCount;
    ULONG CcPinReadNoWait;
    ULONG CcPinReadWait;
    ULONG CcPinReadNoWaitMiss;
    ULONG CcPinReadWaitMiss;
    ULONG CcCopyReadNoWait;
    ULONG CcCopyReadWait;
    ULONG CcCopyReadNoWaitMiss;
    ULONG CcCopyReadWaitMiss;
    ULONG CcMdlReadNoWait;
    ULONG CcMdlReadWait;
    ULONG CcMdlReadNoWaitMiss;
    ULONG CcMdlReadWaitMiss;
    ULONG CcReadAheadIos;
    ULONG CcLazyWriteIos;
    ULONG CcLazyWritePages;
    ULONG CcDataFlushes;
    ULONG CcDataPages;
    ULONG ContextSwitches;
    ULONG FirstLevelTbFills;
    ULONG SecondLevelTbFills;
    ULONG SystemCalls;
} SYSTEM_PERFORMANCE_INFORMATION, *PSYSTEM_PERFORMANCE_INFORMATION;

typedef struct _SYSTEM_CACHE_INFORMATION {
	ULONG SystemCacheWsSize;
	ULONG SystemCacheWsPeakSize;
	ULONG SystemCacheWsFaults;
	ULONG SystemCacheWsMinimum;
	ULONG SystemCacheWsMaximum;
	ULONG TransitionSharedPages;
	ULONG TransitionSharedPagesPeak;
    ULONG Reserved[2];
} SYSTEM_CACHE_INFORMATION, *PSYSTEM_CACHE_INFORMATION;

#define SystemHandleInformation		16

typedef struct _SYSTEM_HANDLE_INFORMATION {
	ULONG ProcessId;
    UCHAR ObjectTypeNumber;
	UCHAR Flags;
    USHORT Handle;
    PVOID Object;
    ACCESS_MASK GrantedAccess;
} SYSTEM_HANDLE_INFORMATION, *PSYSTEM_HANDLE_INFORMATION;

typedef struct _OBJECT_BASIC_INFORMATION {
	ULONG Attributes;
    ACCESS_MASK GrantedAccess;
    ULONG HandleCount;
    ULONG PointerCount;
    ULONG PagedPoolUsage;
    ULONG NonPagedPoolUsage;
	ULONG Reserved[3];
    ULONG NameInformationLength;
    ULONG TypeInformationLength;
    ULONG SecurityDescriptorLength;
	LARGE_INTEGER CreateTime;
} OBJECT_BASIC_INFORMATION, *POBJECT_BASIC_INFORMATION;

typedef struct _OBJECT_TYPE_INFORMATION {
	UNICODE_STRING Name;
    ULONG ObjectCount;
    ULONG HandleCount;
    ULONG Reserved1[4];
    ULONG PeakObjectCount;
    ULONG PeakHandleCount;
    ULONG Reserved2[4];
	ULONG InvalidAttributes;
	GENERIC_MAPPING GenericMapping;
    ULONG ValidAccess;
    UCHAR Unknown;
    BOOLEAN MaintainHandleDatabase;
    POOL_TYPE PoolType;
    ULONG PagedPoolUsage;
    ULONG NonPagedPoolUsage;
} OBJECT_TYPE_INFORMATION, *POBJECT_TYPE_INFORMATION;

typedef struct _OBJECT_NAME_INFORMATION {
	UNICODE_STRING Name;
} OBJECT_NAME_INFORMATION, *POBJECT_NAME_INFORMATION;


typedef struct _SYSTEM_OBJECT_TYPE_INFORMATION {
	ULONG NextEntryOffset;
    ULONG ObjectCount;
    ULONG HandleCount;
    ULONG TypeNumber;
    ULONG InvalidAttributes;
    GENERIC_MAPPING GenericMapping;
    ACCESS_MASK ValidAccessMask;
    POOL_TYPE PoolType;
    UCHAR Unknown;
    UNICODE_STRING Name;
} SYSTEM_OBJECT_TYPE_INFORMATION, *PSYSTEM_OBJECT_TYPE_INFORMATION;

typedef struct _SYSTEM_OBJECT_INFORMATION {
	ULONG NextEntryOffset;
	PVOID Object;
    ULONG CreatorProcessId;
    USHORT Unknown;
    USHORT Flags;
    ULONG PointerCount;
    ULONG HandleCount;
    ULONG PagedPoolUsage;
    ULONG NonPagedPoolUsage;
    ULONG ExclusiveProcessId;
    PSECURITY_DESCRIPTOR SecurityDescriptor;
    UNICODE_STRING Name;
} SYSTEM_OBJECT_INFORMATION, *PSYSTEM_OBJECT_INFORMATION;

/*typedef struct _SYSTEM_OBJECT_INFORMATION {
	ULONG NextEntryOffset;
    PVOID Object;
    ULONG CreatorProcessId;
    USHORT Unknown;
    USHORT Flags;
    ULONG PointerCount;
    ULONG HandleCount;
    ULONG
} SYSTEM_OBJECT_INFORMATION, *PSYSTEM_OBJECT_INFORMATION;*/

typedef struct tagMyHandleInfo {
    UCHAR ObjectTypeNumber;
	UCHAR Flags;
    USHORT Handle;
    PVOID Object;
    ACCESS_MASK GrantedAccess;
	ULONG Attributes;
    ULONG HandleCount;
    ULONG PeakHandleCount;
    ULONG PointerCount;
    ULONG ObjectCount;
    ULONG PeakObjectCount;
    POOL_TYPE PoolType;
    ULONG PagedPoolUsage;
    ULONG NonPagedPoolUsage;
	LARGE_INTEGER CreateTime;
	ULONG InvalidAttributes;
	GENERIC_MAPPING GenericMapping;
    ULONG ValidAccess;
    BOOLEAN MaintainHandleDatabase;
	BSTR TypeName;
    BSTR ObjectName;
} MyHandleInfo, *PMyHandleInfo;

#define SystemGlobalFlag	9

typedef struct _SYSTEM_GLOBAL_FLAG {
	ULONG GlobalFlag;
} SYSTEM_GLOBAL_FLAG, *PSYSTEM_GLOBAL_FLAG;

#define FLG_STOP_ON_EXCEPTION 				0x00000001
#define FLG_SHOW_LDR_SNAPS 					0x00000002
#define FLG_DEBUG_INITIAL_COMMAND 			0x00000004
#define FLG_STOP_ON_HUNG_GUI 				0x00000008
#define FLG_HEAP_ENABLE_TAIL_CHECK 			0x00000010
#define FLG_HEAP_ENABLE_FREE_CHECK 			0x00000020
#define FLG_HEAP_VALIDATE_PARAMETERS 		0x00000040
#define FLG_HEAP_VALIDATE_ALL 				0x00000080
#define FLG_POOL_ENABLE_TAIL_CHECK 			0x00000100
#define FLG_POOL_ENABLE_FREE_CHECK 			0x00000200
#define FLG_POOL_ENABLE_TAGGING 			0x00000400
#define FLG_HEAP_ENABLE_TAGGING 			0x00000800
#define FLG_USER_STACK_TRACE_DB 			0x00001000
#define FLG_KERNEL_STACK_TRACE_DB 			0x00002000
#define FLG_MAINTAIN_OBJECT_TYPELIST 		0x00004000
#define FLG_HEAP_ENABLE_TAG_BY_DLL 			0x00008000
#define FLG_IGNORE_DEBUG_PRIV 				0x00010000
#define FLG_ENABLE_CSRDEBUG 				0x00020000
#define FLG_ENABLE_KDEBUG_SYMBOL_LOAD 		0x00040000
#define FLG_DISABLE_PAGE_KERNEL_STACKS 		0x00080000
#define FLG_HEAP_ENABLE_CALL_TRACING 		0x00100000
#define FLG_HEAP_DISABLE_COALESCING 		0x00200000
#define FLG_ENABLE_CLOSE_EXCEPTIONS 		0x00400000
#define FLG_ENABLE_EXCEPTION_LOGGING 		0x00800000
#define FLG_ENABLE_DBGPRINT_BUFFERING 		0x08000000


extern SYSTEM_PERFORMANCE_INFORMATION SysPerfInfo;
extern SYSTEM_CACHE_INFORMATION SysCacheInfo;


bool GetCPUUsage(double& val);
bool PerfInit();
bool GetHandleInfo(DWORD dwPID, PMyHandleInfo* ppHandleInfo, LPDWORD pcbHandles);


#endif

