#include "Process.h"
#include <tlhelp32.h>
#include <tchar.h>
#include <psapi.h>
#include "Utils.h"
#include <shlwapi.h>

struct ModSearchStruct {
    LPTSTR lpszModName;
    LPFNMODSEARCH pfnModSearch;
    LPVOID lpParam;
    PSYSTEM_PROCESSES prc;
};

BOOL WINAPI KillProcessTreeNtHelper(PSYSTEM_PROCESSES, DWORD);
BOOL WINAPI KillProcessTree9xHelper(DWORD);

// In NtDll.dll
NTSTATUS (WINAPI *_ZwQuerySystemInformation)(UINT, PVOID, ULONG, PULONG);

// ToolHelp32 - In Kernel32.dll
HANDLE (WINAPI *_CreateToolhelp32Snapshot)(DWORD, DWORD);
BOOL (WINAPI *_Process32First)(HANDLE, PROCESSENTRY32 *);
BOOL (WINAPI *_Process32Next)(HANDLE, PROCESSENTRY32 *);
BOOL (WINAPI *_Module32First)(HANDLE, MODULEENTRY32 *);
BOOL (WINAPI *_Module32Next)(HANDLE, MODULEENTRY32 *);

// PSAPI - In Psapi.dll
BOOL (WINAPI *_EnumProcessModules)(HANDLE, HMODULE *, DWORD, LPDWORD);
BOOL (WINAPI *_GetModuleInformation)(HANDLE, HMODULE, LPMODULEINFO, DWORD);
BOOL (WINAPI *_GetModuleFileNameEx)(HANDLE, HMODULE, LPTSTR, DWORD);

// In AdvAPI32.dll
BOOL (WINAPI *_OpenProcessToken)(HANDLE, DWORD, PHANDLE);
BOOL (WINAPI *_GetTokenInformation)(HANDLE, TOKEN_INFORMATION_CLASS,
    LPVOID, DWORD, PDWORD);
BOOL (WINAPI *_LookupAccountSid)(LPCTSTR, PSID, LPTSTR, LPDWORD,
    LPTSTR, LPDWORD, PSID_NAME_USE);
BOOL (WINAPI *_LookupPrivilegeName)(LPCTSTR, PLUID, LPTSTR, LPDWORD);
BOOL (WINAPI *_LookupPrivilegeValue)(LPCTSTR, LPCTSTR, PLUID);
BOOL (WINAPI *_AdjustTokenPrivileges)(HANDLE, BOOL, PTOKEN_PRIVILEGES,
    DWORD, PTOKEN_PRIVILEGES, PDWORD);

BOOL WINAPI LoadProcessDLLs()
{
	OSVERSIONINFO osvi;
    HINSTANCE hNtDll, hKernel, hPsapi, hAdvApi;
    osvi.dwOSVersionInfoSize = sizeof(osvi);
    GetVersionEx(&osvi);
    if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
        hNtDll = GetModuleHandle(_T("ntdll.dll"));
        hPsapi = LoadLibrary(_T("psapi.dll"));
        hAdvApi = LoadLibrary(_T("advapi32.dll"));

        if (!hNtDll || !hPsapi || !hAdvApi)
            return false;

        *(FARPROC *)&_ZwQuerySystemInformation =
            GetProcAddress(hNtDll, "ZwQuerySystemInformation");
        if (!_ZwQuerySystemInformation) return FALSE;

        *(FARPROC *)&_EnumProcessModules =
            GetProcAddress(hPsapi, "EnumProcessModules");
        *(FARPROC *)&_GetModuleInformation =
            GetProcAddress(hPsapi, "GetModuleInformation");
#ifdef UNICODE
        *(FARPROC *)&_GetModuleFileNameEx =
            GetProcAddress(hPsapi, "GetModuleFileNameExW");
#else
        *(FARPROC *)&_GetModuleFileNameEx =
            GetProcAddress(hPsapi, "GetModuleFileNameExA");
#endif

        if (!_EnumProcessModules ||
            !_GetModuleInformation ||
            !_GetModuleFileNameEx)
            return false;

        *(FARPROC *)&_OpenProcessToken =
            GetProcAddress(hAdvApi, "OpenProcessToken");
        *(FARPROC *)&_GetTokenInformation =
            GetProcAddress(hAdvApi, "GetTokenInformation");
#ifdef UNICODE
        *(FARPROC *)&_LookupPrivilegeName =
            GetProcAddress(hAdvApi, "LookupPrivilegeNameW");
        *(FARPROC *)&_LookupPrivilegeValue =
            GetProcAddress(hAdvApi, "LookupPrivilegeValueW");
        *(FARPROC *)&_LookupAccountSid =
            GetProcAddress(hAdvApi, "LookupAccountSidW");
#else
        *(FARPROC *)&_LookupPrivilegeName =
            GetProcAddress(hAdvApi, "LookupPrivilegeNameA");
        *(FARPROC *)&_LookupPrivilegeValue =
            GetProcAddress(hAdvApi, "LookupPrivilegeValueA");
        *(FARPROC *)&_LookupAccountSid =
            GetProcAddress(hAdvApi, "LookupAccountSidA");
#endif
        *(FARPROC *)&_AdjustTokenPrivileges =
            GetProcAddress(hAdvApi, "AdjustTokenPrivileges");

        if (!_OpenProcessToken ||
            !_GetTokenInformation ||
            !_LookupPrivilegeName ||
            !_LookupPrivilegeValue ||
            !_LookupAccountSid ||
            !_AdjustTokenPrivileges)
            return FALSE;
    }
    else
    {
        hKernel = GetModuleHandle(_T("kernel32.dll"));
        if (!hKernel) return FALSE;

        *(FARPROC *)&_CreateToolhelp32Snapshot =
            GetProcAddress(hKernel, "CreateToolhelp32Snapshot");
#ifdef UNICODE
        *(FARPROC *)&_Process32First =
            GetProcAddress(hKernel, "Process32FirstW");
        *(FARPROC *)&_Process32Next =
            GetProcAddress(hKernel, "Process32NextW");
        *(FARPROC *)&_Module32First =
            GetProcAddress(hKernel, "Module32FirstW");
        *(FARPROC *)&_Module32Next =
            GetProcAddress(hKernel, "Module32NextW");
#else
        *(FARPROC *)&_Process32First =
            GetProcAddress(hKernel, "Process32First");
        *(FARPROC *)&_Process32Next =
            GetProcAddress(hKernel, "Process32Next");
        *(FARPROC *)&_Module32First =
            GetProcAddress(hKernel, "Module32First");
        *(FARPROC *)&_Module32Next =
            GetProcAddress(hKernel, "Module32Next");
#endif

        if (!_CreateToolhelp32Snapshot ||
            !_Process32First ||
            !_Process32Next ||
            !_Module32First ||
            !_Module32Next)
            return FALSE;
    }

    return TRUE;
}

BOOL WINAPI EnumProcessesNt(LPFNENUMPROC pfnEnumProc, LPVOID lpParam)
{
    HANDLE hHeap;
    NTSTATUS Status;
    ULONG cbBuffer = 0x8000;
    LPVOID pBuffer = NULL;
    PSYSTEM_PROCESSES pProcesses;

    if (!pfnEnumProc) return FALSE;

    hHeap = GetProcessHeap();

    do
    {
        pBuffer = HeapAlloc(hHeap, 0, cbBuffer);
        if (!pBuffer) return FALSE;

        Status = _ZwQuerySystemInformation(
            SystemProcessesAndThreadsInformation,
            pBuffer, cbBuffer, NULL);

        if (Status == STATUS_INFO_LENGTH_MISMATCH)
        {
            HeapFree(hHeap, 0, pBuffer);
            cbBuffer <<= 1;
        }
        else if (!NT_SUCCESS(Status))
        {
            HeapFree(hHeap, 0, pBuffer);
            return FALSE;
        }
    } while (Status == STATUS_INFO_LENGTH_MISMATCH);

    pProcesses = (PSYSTEM_PROCESSES)pBuffer;

    for (;;)
    {
        pfnEnumProc(pProcesses, lpParam);
        if (!pProcesses->NextEntryDelta) break;
        pProcesses = (PSYSTEM_PROCESSES)(((LPBYTE)pProcesses)
            + pProcesses->NextEntryDelta);
    }

    HeapFree(hHeap, 0, pBuffer);

    return TRUE;
}

BOOL WINAPI EnumProcesses9x(LPFNENUMPROC pfnEnumProc, LPVOID lpParam)
{
    HANDLE hSnapshot;
    SYSTEM_PROCESSES Process;
    PROCESSENTRY32 Entry;
    LPSTR p;
    WCHAR ProcessName[MAX_PATH];

    if (!pfnEnumProc) return FALSE;

    hSnapshot = _CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    if (hSnapshot == INVALID_HANDLE_VALUE)
        return FALSE;

    Entry.dwSize = sizeof(Entry);
    if (!_Process32First(hSnapshot, &Entry))
        return FALSE;

    ZeroMemory(&Process, sizeof(Process));

    do
    {
        Process.ProcessId = Entry.th32ProcessID;
        Process.ThreadCount = Entry.cntThreads;
        Process.InheritedFromProcessId = Entry.th32ParentProcessID;
        Process.BasePriority = Entry.pcPriClassBase;
#ifndef UNICODE
        MultiByteToWideChar(CP_ACP, 0, Entry.szExeFile, -1,
            ProcessName, sizeof(ProcessName) >> 1);
#else
        lstrcpyW(ProcessName, Entry.szExeFile);
#endif
        Process.ProcessName.Buffer = ProcessName;
        Process.ProcessName.Length = lstrlenW(ProcessName);
        pfnEnumProc(&Process, lpParam);
    } while (_Process32Next(hSnapshot, &Entry));

    CloseHandle(hSnapshot);

    return TRUE;
}

BOOL WINAPI EnumModulesNt(DWORD ProcessId,
    LPFNENUMMOD pfnEnumProc, LPVOID lpParam)
{
    HMODULE *hMods;
    HANDLE hHeap, hProcess;
    DWORD Size, dwNeeded = 0, i;
    MODULEINFO mi;
    TCHAR szModPath[MAX_PATH];

    if ((hProcess = OpenProcess(PROCESS_VM_READ | PROCESS_QUERY_INFORMATION,
        FALSE, ProcessId)) == NULL)
        return FALSE;

    hHeap = GetProcessHeap();
    Size = 1024;

    for (;;)
    {
        hMods = (HMODULE *)HeapAlloc(hHeap, 0, Size);
        if (!hMods)
        {
            CloseHandle(hProcess);
            return FALSE;
        }

        if (!_EnumProcessModules(hProcess, hMods, Size, &dwNeeded))
        {
            if (dwNeeded > Size) Size = dwNeeded;
            else
            {
                HeapFree(hHeap, 0, hMods);
                CloseHandle(hProcess);
                return FALSE;
            }
        } else break;

        HeapFree(hHeap, 0, hMods);
    }

    dwNeeded >>= 2;
    for (i = 0; i < dwNeeded; i++)
    {
        if (_GetModuleInformation(hProcess, hMods[i], &mi, sizeof(mi)) &&
            _GetModuleFileNameEx(hProcess, hMods[i],
                szModPath, sizeof(szModPath)))
                pfnEnumProc(szModPath, mi.lpBaseOfDll, mi.SizeOfImage, lpParam);
    }

    HeapFree(hHeap, 0, hMods);
    CloseHandle(hProcess);

    return TRUE;
}

BOOL WINAPI EnumModules9x(DWORD ProcessId, LPFNENUMMOD pfnEnumProc, LPVOID lpParam)
{
    HANDLE hSnapshot;
    MODULEENTRY32 Entry;
    TCHAR szModPath[MAX_PATH];

    if (!pfnEnumProc) return FALSE;

    hSnapshot = _CreateToolhelp32Snapshot(TH32CS_SNAPMODULE, ProcessId);
    if (hSnapshot == INVALID_HANDLE_VALUE)
        return FALSE;

    Entry.dwSize = sizeof(Entry);
    if (!_Module32First(hSnapshot, &Entry))
        return FALSE;

    do
    {
        if (GetModuleFileName(Entry.hModule, szModPath, sizeof(szModPath)))
            pfnEnumProc(szModPath, Entry.modBaseAddr,
            Entry.modBaseSize, lpParam);
    } while (_Module32Next(hSnapshot, &Entry));

    CloseHandle(hSnapshot);

    return TRUE;
}

BOOL WINAPI EnumMemRegions(DWORD ProcessId,
    LPFNENUMMEM pfnEnumProc, LPVOID lpParam)
{
    HANDLE hProcess;
    MEMORY_BASIC_INFORMATION mbi;
    LPBYTE lpAddr = NULL;

    if ((hProcess = OpenProcess(PROCESS_VM_READ | PROCESS_QUERY_INFORMATION,
        FALSE, ProcessId)) == INVALID_HANDLE_VALUE) return FALSE;

    while (VirtualQueryEx(hProcess, lpAddr, &mbi, sizeof(mbi)))
    {
        pfnEnumProc(&mbi, lpParam);
        lpAddr += mbi.RegionSize;
    }

    CloseHandle(hProcess);

    return TRUE;
}

BOOL WINAPI GetProcessModulePath(DWORD dwProcessId, LPTSTR ModulePath)
{
    HMODULE *hMods;
    HANDLE hHeap, hPList, hProcess;
    MODULEENTRY32 Entry;
    DWORD Size, dwNeeded = 0;
    MODULEINFO mi;
	OSVERSIONINFO osvi;
    osvi.dwOSVersionInfoSize = sizeof(osvi);

    if ((hProcess = OpenProcess(PROCESS_VM_READ | PROCESS_QUERY_INFORMATION,
        FALSE, dwProcessId)) == NULL)
        return FALSE;

    GetVersionEx(&osvi);

    if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
    {
        hHeap = GetProcessHeap();
        Size = 1024;

        for (;;)
        {
            hMods = (HMODULE *)HeapAlloc(hHeap, 0, Size);
            if (!hMods)
            {
                CloseHandle(hProcess);
                return FALSE;
            }

            if (!_EnumProcessModules(hProcess, hMods, Size, &dwNeeded))
            {
                if (dwNeeded > Size) Size = dwNeeded;
                else
                {
                    HeapFree(hHeap, 0, hMods);
                    CloseHandle(hProcess);
                    return FALSE;
                }
            } else break;

            HeapFree(hHeap, 0, hMods);
        }

        if (!_GetModuleFileNameEx(hProcess, hMods[0],
            ModulePath, MAX_PATH)) return FALSE;

        HeapFree(hHeap, 0, hMods);
        CloseHandle(hProcess);
    }
    else
    {
        if (dwProcessId && (hPList =
            _CreateToolhelp32Snapshot(TH32CS_SNAPMODULE, dwProcessId)) !=
            INVALID_HANDLE_VALUE)
        {
            Entry.dwSize = sizeof(Entry);
            if (Module32First(hPList, &Entry))
                lstrcpy(ModulePath, Entry.szExePath);
            CloseHandle(hPList);
        }

        CloseHandle(hProcess);
    }

    return TRUE;
}

BOOL WINAPI GetProcessUserName(DWORD dwProcessId, LPTSTR UserName)
{
    HANDLE hToken, hProcess;
    TCHAR t[256];
    TCHAR DomainName[256];
    SID_NAME_USE snu;
    DWORD Size;
    PTOKEN_USER to;

    if ((hProcess = OpenProcess(PROCESS_QUERY_INFORMATION,
        FALSE, dwProcessId)) == NULL) return FALSE;

    if (!_OpenProcessToken(hProcess, TOKEN_READ, &hToken))
        return FALSE;

    to = (PTOKEN_USER)t;
    if (!_GetTokenInformation(hToken, TokenUser, t, sizeof(t), &Size))
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    Size = sizeof(DomainName) / sizeof(TCHAR);

    if (!_LookupAccountSid(NULL, to->User.Sid, t, &Size,
        DomainName, &Size, &snu))
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    if (DomainName[0])
    {
        lstrcpy(UserName, DomainName);
        lstrcat(UserName, "\\");
    }

    lstrcat(UserName, t);
    CloseHandle(hToken);
    CloseHandle(hProcess);

    return TRUE;
}

BOOL WINAPI GetProcessPrivilege(DWORD dwProcessId,
    LPTSTR PrivilegeName, BOOL *Enabled)
{
    HANDLE hToken, hProcess;
    TOKEN_PRIVILEGES *tkp;
    TCHAR Name[128];
    DWORD Size;
    int i, k;

    if ((hProcess = OpenProcess(PROCESS_ALL_ACCESS,
        FALSE, dwProcessId)) == NULL) return FALSE;

    if (!_OpenProcessToken(hProcess, TOKEN_QUERY, &hToken))
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    if (!_GetTokenInformation(hToken, TokenPrivileges, tkp, 0, &Size))
    {
        CloseHandle(hToken);
        CloseHandle(hProcess);
        return FALSE;
    }

    tkp = (TOKEN_PRIVILEGES *)new TCHAR[Size];
    if (!_GetTokenInformation(hToken, TokenPrivileges, tkp, Size, &Size))
    {
        CloseHandle(hToken);
        CloseHandle(hProcess);
        return FALSE;
    }

    k = tkp->PrivilegeCount;
    for (i = 0; i < k; i++)
    {
        Size = sizeof(Name);
        _LookupPrivilegeName("", &tkp->Privileges[i].Luid, Name, &Size);
        if (!lstrcmpi(Name, PrivilegeName))
        {
            *Enabled = tkp->Privileges[i].Attributes == SE_PRIVILEGE_ENABLED;
            CloseHandle(hToken);
            CloseHandle(hProcess);
            return TRUE;
        }
    }

    CloseHandle(hToken);
    CloseHandle(hProcess);

    return FALSE;
}

BOOL WINAPI GetProcessPrivileges(DWORD dwProcessId,
    LPTSTR *PrivilegeNames, BOOL **Enabled, LPDWORD Count)
{
    HANDLE hToken, hProcess, hHeap;
    TOKEN_PRIVILEGES *tkp;
    TCHAR Name[128];
    LPTSTR p;
    DWORD Size;
    int i, k;

    if ((hProcess = OpenProcess(PROCESS_ALL_ACCESS,
        FALSE, dwProcessId)) == NULL) return FALSE;

    if (!_OpenProcessToken(hProcess, TOKEN_QUERY, &hToken))
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    _GetTokenInformation(hToken, TokenPrivileges, tkp, 0, &Size);

    hHeap = GetProcessHeap();
    tkp = (TOKEN_PRIVILEGES *)HeapAlloc(hHeap,
        HEAP_ZERO_MEMORY, Size * sizeof(TCHAR));
    if (!_GetTokenInformation(hToken, TokenPrivileges, tkp, Size, &Size))
    {
        HeapFree(hHeap, 0, tkp);
        CloseHandle(hToken);
        CloseHandle(hProcess);
        return FALSE;
    }

    k = tkp->PrivilegeCount;
    *Count = k;
    *Enabled = (BOOL *)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, k * sizeof(BOOL));
    *PrivilegeNames = (TCHAR *)HeapAlloc(hHeap,
        HEAP_ZERO_MEMORY, k * 128 * sizeof(TCHAR));

    if (!*Enabled || !*PrivilegeNames)
    {
        HeapFree(hHeap, 0, tkp);
        CloseHandle(hToken);
        CloseHandle(hProcess);
        return FALSE;
    }

    p = *PrivilegeNames;

    for (i = 0; i < k; i++)
    {
        Size = sizeof(Name);
        _LookupPrivilegeName("", &tkp->Privileges[i].Luid, Name, &Size);
        (*Enabled)[i] = tkp->Privileges[i].Attributes == SE_PRIVILEGE_ENABLED;
        lstrcpy(p, Name);
        p += (lstrlen(Name) + 1) * sizeof(TCHAR);
    }

    *p = 0;

    HeapFree(hHeap, 0, tkp);
    CloseHandle(hToken);
    CloseHandle(hProcess);

    return TRUE;
}

BOOL WINAPI SetProcessPrivilege(DWORD dwProcessId,
    LPTSTR PrivilegeName, BOOL Enable)
{
    HANDLE hToken, hProcess;
    TOKEN_PRIVILEGES tkp;

    if ((hProcess = OpenProcess(PROCESS_ALL_ACCESS,
        FALSE, dwProcessId)) == NULL) return FALSE;

    if (!_OpenProcessToken(hProcess,
        TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken))
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    _LookupPrivilegeValue(NULL, PrivilegeName, &tkp.Privileges[0].Luid);

    tkp.PrivilegeCount = 1;
    tkp.Privileges[0].Attributes = Enable ? SE_PRIVILEGE_ENABLED : 0;

    if (!_AdjustTokenPrivileges(hToken, FALSE, &tkp, 0,
        (PTOKEN_PRIVILEGES)NULL, 0))
    {
        CloseHandle(hToken);
        CloseHandle(hProcess);
        return FALSE;
    }

    CloseHandle(hToken);
    CloseHandle(hProcess);

    return TRUE;
}

BOOL WINAPI SetProcessPrivileges(DWORD dwProcessId,
    LPTSTR PrivilegeNames, BOOL *Enabled, DWORD Count)
{
    HANDLE hToken, hProcess;
    PTOKEN_PRIVILEGES tkp;
    DWORD i;

    if ((hProcess = OpenProcess(PROCESS_ALL_ACCESS,
        FALSE, dwProcessId)) == NULL) return FALSE;

    if (!_OpenProcessToken(hProcess,
        TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken))
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    tkp = (PTOKEN_PRIVILEGES)new BYTE[sizeof(TOKEN_PRIVILEGES) +
        sizeof(LUID_AND_ATTRIBUTES) * (Count - 1)];
    tkp->PrivilegeCount = Count;

    for (i = 0; i < Count; i++)
    {
        _LookupPrivilegeValue(NULL, PrivilegeNames, &tkp->Privileges[i].Luid);
        tkp->Privileges[i].Attributes = Enabled[i] ? SE_PRIVILEGE_ENABLED : 0;
        while (*PrivilegeNames++);
    }

    if (!_AdjustTokenPrivileges(hToken, FALSE, tkp, 0,
        (PTOKEN_PRIVILEGES)NULL, 0))
    {
        delete tkp;
        CloseHandle(hToken);
        CloseHandle(hProcess);
        return FALSE;
    }

    delete (LPBYTE)tkp;
    CloseHandle(hToken);
    CloseHandle(hProcess);

    return TRUE;
}

BOOL WINAPI KillProcess(DWORD dwProcessId)
{
	HANDLE hProcess = OpenProcess(PROCESS_TERMINATE, FALSE, dwProcessId);

	if (!hProcess || !TerminateProcess(hProcess, (UINT)-1))
    {
    	CloseHandle(hProcess);
        return FALSE;
    }

	CloseHandle(hProcess);

	return TRUE;
}

BOOL WINAPI KillProcessTree(DWORD dwProcessId)
{
	OSVERSIONINFO osvi;
	BOOL dwError;
    NTSTATUS (WINAPI *_ZwQuerySystemInformation)(UINT, PVOID, ULONG, PULONG);
    HANDLE hHeap;
    NTSTATUS Status;
    ULONG cbBuffer = 0x8000;
    LPVOID pBuffer = NULL;
    DWORD Error;

	osvi.dwOSVersionInfoSize = sizeof(osvi);
	GetVersionEx(&osvi);

	if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		hHeap = GetProcessHeap();

		do
		{
			pBuffer = HeapAlloc(hHeap, 0, cbBuffer);
			if (!pBuffer) return FALSE;

			Status = _ZwQuerySystemInformation(
                SystemProcessesAndThreadsInformation,
                pBuffer, cbBuffer, NULL);

			if (Status == STATUS_INFO_LENGTH_MISMATCH)
			{
				HeapFree(hHeap, 0, pBuffer);
				cbBuffer <<= 1;
			}
			else if (!NT_SUCCESS(Status))
			{
				HeapFree(hHeap, 0, pBuffer);
				return SetLastError(Status), NULL;
			}
		}
		while (Status == STATUS_INFO_LENGTH_MISMATCH);

		Error = KillProcessTreeNtHelper((PSYSTEM_PROCESSES)pBuffer,
            dwProcessId);

		HeapFree(hHeap, 0, pBuffer);
	}
	else
	{
		Error = KillProcessTree9xHelper(dwProcessId);
	}

	return Error;
}

BOOL WINAPI KillProcessTreeNtHelper(
    PSYSTEM_PROCESSES pInfo, DWORD dwProcessId)
{
    PSYSTEM_PROCESSES p = pInfo;

	if (!p) return FALSE;

    for (;;)
    {
		if (p->InheritedFromProcessId == dwProcessId)
			KillProcessTreeNtHelper(pInfo, p->ProcessId);
        if (!p->NextEntryDelta) break;
		p = (PSYSTEM_PROCESSES)(((LPBYTE)p) + p->NextEntryDelta);
    }

    return KillProcess(dwProcessId);
}

BOOL WINAPI KillProcessTree9xHelper(DWORD dwProcessId)
{
	HANDLE (WINAPI *_CreateToolhelp32Snapshot)(DWORD, DWORD);
	BOOL (WINAPI *_Process32First)(HANDLE, PROCESSENTRY32 *);
	BOOL (WINAPI *_Process32Next)(HANDLE, PROCESSENTRY32 *);
	HANDLE hSnapshot;
	PROCESSENTRY32 Entry;

	hSnapshot = _CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
	if (hSnapshot == INVALID_HANDLE_VALUE) return FALSE;

	Entry.dwSize = sizeof(Entry);
	if (!_Process32First(hSnapshot, &Entry))
	{
		CloseHandle(hSnapshot);
		return FALSE;
	}

	do
	{
		if (Entry.th32ParentProcessID == dwProcessId)
            KillProcessTree9xHelper(Entry.th32ProcessID);
		Entry.dwSize = sizeof(Entry);
	} while (_Process32Next(hSnapshot, &Entry));

	CloseHandle(hSnapshot);

    if (!KillProcess(dwProcessId)) return FALSE;

	return TRUE;
}

BOOL WINAPI GetProcessStartupInfo(DWORD dwProcessId, LPTSTR *lplpStartupDir,
    LPTSTR *lplpCmdLine, LPTSTR *lplpEnvVars)
{
    MEMORY_BASIC_INFORMATION mbi;
    LPWSTR p;
    LPTSTR q;
    LPWSTR pCmdLine = GetCommandLineW(), pwBuf;
    LPWSTR pStartupDir = pCmdLine;
    LPWSTR pEnvVars = GetEnvironmentStringsW();
    DWORD pid = GetCurrentProcessId(), dwSize, dwRead;
    HANDLE hHeap = GetProcessHeap();
    *lplpStartupDir = *lplpCmdLine = *lplpEnvVars = NULL;
    CHAR DefChar = 0;
    BOOL UseDefChar = TRUE;

    HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION |
        PROCESS_VM_READ, FALSE, pid);
    if (!hProcess) return FALSE;

    VirtualQueryEx(hProcess, pCmdLine, &mbi, sizeof(mbi));
    if (mbi.State != MEM_COMMIT)
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    while (*pStartupDir) pStartupDir--;
    while (!*pStartupDir) pStartupDir--;
    while (*pStartupDir) pStartupDir--;
    while (!*pStartupDir) pStartupDir--;
    while (*pStartupDir) pStartupDir--;
    pStartupDir -= MAX_PATH - 1;

    CloseHandle(hProcess);

    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION |
        PROCESS_VM_READ, FALSE, dwProcessId);
    if (!hProcess) return FALSE;

    VirtualQueryEx(hProcess, pStartupDir, &mbi, sizeof(mbi));
    if (mbi.State != MEM_COMMIT)
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    dwSize = (LPBYTE)mbi.BaseAddress - (LPBYTE)pStartupDir + mbi.RegionSize;
    pwBuf = (LPWSTR)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwSize);
    ReadProcessMemory(hProcess, pStartupDir, pwBuf, dwSize, &dwRead);

    pCmdLine = pwBuf;
    while (*pCmdLine) pCmdLine++;
    if (pCmdLine - pwBuf >= MAX_PATH)
    {
        HeapFree(hHeap, 0, pwBuf);
        return TRUE;
    }
    while (!*pCmdLine) pCmdLine++;
    if (pCmdLine - pwBuf > MAX_PATH)
    {
        HeapFree(hHeap, 0, pwBuf);
        return TRUE;
    }
    while (*pCmdLine) pCmdLine++;
    while (!*pCmdLine) pCmdLine++;
    while (*pCmdLine) pCmdLine++;
    while (!*pCmdLine) pCmdLine++;

#ifdef UNICODE
    *lplpStartupDir = (LPWSTR)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwRead);
    lstrcpy(*lplpStartupDir, pwBuf);
#else
    *lplpStartupDir = (LPSTR)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwRead);
    WideCharToMultiByte(CP_ACP, 0, pwBuf, -1, *lplpStartupDir,
        dwRead / sizeof(WCHAR), &DefChar, &UseDefChar);
#endif

    lstrcpyW(pwBuf, pCmdLine);

#ifdef UNICODE
    *lplpCmdLine = (LPWSTR)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwRead);
    lstrcpy(*lplpCmdLine, pwBuf);
#else
    *lplpCmdLine = (LPSTR)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwRead);
    WideCharToMultiByte(CP_ACP, 0, pwBuf, -1, *lplpCmdLine,
        dwRead / sizeof(WCHAR), &DefChar, &UseDefChar);
#endif

    VirtualQueryEx(hProcess, pEnvVars, &mbi, sizeof(mbi));
    if (mbi.State != MEM_COMMIT)
    {
        CloseHandle(hProcess);
        return FALSE;
    }

    dwSize = (LPBYTE)mbi.BaseAddress - (LPBYTE)pEnvVars + mbi.RegionSize;
    HeapFree(hHeap, 0, pwBuf);
    pwBuf = (LPWSTR)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwSize);
    ReadProcessMemory(hProcess, pEnvVars, pwBuf, dwSize, &dwRead);
#ifdef UNICODE
    *lplpEnvVars = (LPWSTR)HeapAlloc(hHeap, 0, dwRead);
    memcpy(*lplpEnvVars, pwBuf, dwRead);
#else
    *lplpEnvVars = (LPSTR)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwRead);
    p = pwBuf;
    q = *lplpEnvVars;
    do
    {
        WideCharToMultiByte(CP_ACP, 0, p, -1, q,
            q - *lplpEnvVars + dwRead / sizeof(WCHAR), &DefChar, &UseDefChar);
        while (*p++);
        q += lstrlen(q) + 1;
    } while (*p);
#endif

    HeapFree(hHeap, 0, pwBuf);

    return TRUE;
}

VOID WINAPI GetProtectStr(LPTSTR buf, DWORD Protect)
{
    Protect &= ~PAGE_NOCACHE & ~PAGE_GUARD;
    switch (Protect)
    {
        case PAGE_READONLY: lstrcpy(buf, "Read Only"); break;
        case PAGE_READWRITE: lstrcpy(buf, "Read/Write"); break;
        case PAGE_WRITECOPY: lstrcpy(buf, "Write Copy"); break;
        case PAGE_EXECUTE: lstrcpy(buf, "Execute"); break;
        case PAGE_EXECUTE_READ: lstrcpy(buf, "ExecuteRead"); break;
        case PAGE_EXECUTE_READWRITE: lstrcpy(buf, "ExecuteRdWr"); break;
        case PAGE_EXECUTE_WRITECOPY: lstrcpy(buf, "ExecuteWrCopy"); break;
        case PAGE_NOACCESS: lstrcpy(buf, "No Access"); break;
        default: lstrcpy(buf, "N/A");
    }
}

BOOL WINAPI ProcessSearchData(HANDLE hProcess, LPBYTE lpStartAddr,
    LPBYTE Data, DWORD cbSize, LPBYTE *lplpAddr, BOOL IgnoreCase,
    LPFNSEARCH pfnSearch, LPVOID lpParam, LPBOOL Terminate)
{
    MEMORY_BASIC_INFORMATION mbi;
    DWORD dwCurPos = 0, dwSize, i;
    LPBYTE lpEndAddr, t = lpStartAddr;
    BYTE Buf[0x1000], *p;

    if (IgnoreCase)
    {
        p = Data;
        for (i = 0; i < cbSize; i++, p++)
            *p = (BYTE)CharUpper((LPTSTR)*p);
    }

    while (VirtualQueryEx(hProcess, lpStartAddr, &mbi, sizeof(mbi)))
    {
        if (*Terminate) return FALSE;
        if (mbi.State != MEM_COMMIT)
        {
            dwCurPos = 0;
            lpStartAddr = (LPBYTE)mbi.BaseAddress + mbi.RegionSize;
        }
        else
        {
            dwSize = sizeof(Buf) - (lpStartAddr - (LPBYTE)mbi.BaseAddress);
            lpEndAddr = (LPBYTE)mbi.BaseAddress + mbi.RegionSize;
            while (lpStartAddr < lpEndAddr)
            {
                if (*Terminate) return FALSE;
                if (!ReadProcessMemory(hProcess, lpStartAddr, Buf, dwSize, NULL))
                {
                    dwCurPos = 0;
                    break;
                }
                if (IgnoreCase)
                {
                    p = Buf;
                    for (i = 0; i < dwSize; i++, p++)
                        *p = (BYTE)CharUpper((LPTSTR)*p);
                }
                p = Buf;
                for (i = 0; i < dwSize; i++)
                    if (*p++ == Data[dwCurPos])
                    {
                        dwCurPos++;
                        if (dwCurPos == cbSize)
                        {
                            *lplpAddr = lpStartAddr + i - cbSize + 1;
                            return TRUE;
                        }
                    }
                    else dwCurPos = 0;
                lpStartAddr += dwSize;
                dwSize = sizeof(Buf);
                if (pfnSearch) pfnSearch(t, lpStartAddr, lpParam);
            }
            if (lpStartAddr != lpEndAddr)
            {
                lpStartAddr = lpEndAddr;
                if (pfnSearch) pfnSearch(t, lpStartAddr, lpParam);
            }
        }
    }

    return FALSE;
}

VOID WINAPI SearchEnumModules(LPTSTR szModName, LPVOID lpBaseOfDll,
    DWORD SizeOfImage, LPVOID lpParam)
{
    ModSearchStruct *mss = (ModSearchStruct *)lpParam;
    TCHAR szPath[MAX_PATH];
    if (!PathMatchSpec(PathFindFileName(szModName), mss->lpszModName)) return;
    if (IsWinNT())
        GetProcessModulePath(mss->prc->ProcessId, szPath);
    else
    {
#ifndef UNICODE
    WideCharToMultiByte(CP_ACP, 0, mss->prc->ProcessName.Buffer,
        -1, szPath, MAX_PATH, NULL, NULL);
#else
    lstrcpy(szPath, mss->prc->ProcessName.Buffer);
#endif
    }
    mss->pfnModSearch(
    	PathFindFileName(szModName),
        PathFindFileName(szPath),
        mss->prc->ProcessId,
        szPath,
        mss->lpParam);
}

VOID WINAPI SearchEnumProcesses(PSYSTEM_PROCESSES prc, LPVOID lpParam)
{
    ModSearchStruct *mss = (ModSearchStruct *)lpParam;
    mss->prc = prc;
    if (IsWinNT())
        EnumModulesNt(prc->ProcessId, SearchEnumModules, mss);
    else
        EnumModules9x(prc->ProcessId, SearchEnumModules, mss);
}

BOOL WINAPI SearchForModule(LPTSTR lpszModName,
    LPFNMODSEARCH pfnModSearch, LPVOID lpParam)
{
    ModSearchStruct mss = {lpszModName, pfnModSearch, lpParam, 0};
    if (IsWinNT())
    {
        if (!EnumProcessesNt(SearchEnumProcesses, &mss)) return FALSE;
    }
    else
    {
        if (!EnumProcesses9x(SearchEnumProcesses, &mss)) return FALSE;
    }
    return TRUE;
}


