#ifndef _PROCESS_H_
#define _PROCESS_H_

#include <windows.h>

typedef LONG NTSTATUS;
typedef LONG KPRIORITY;

#define NT_SUCCESS(Status)                      ((NTSTATUS)(Status) >= 0)
#define STATUS_INFO_LENGTH_MISMATCH             ((NTSTATUS)0xC0000004L)
#define SystemProcessesAndThreadsInformation    5

typedef struct _CLIENT_ID {
    DWORD UniqueProcess;
    DWORD UniqueThread;
} CLIENT_ID;

typedef struct _UNICODE_STRING {
    USHORT Length;
    USHORT MaximumLength;
    PWSTR Buffer;
} UNICODE_STRING;

typedef struct _VM_COUNTERS {
    SIZE_T PeakVirtualSize;
    SIZE_T VirtualSize;
    ULONG PageFaultCount;
    SIZE_T PeakWorkingSetSize;
    SIZE_T WorkingSetSize;
    SIZE_T QuotaPeakPagedPoolUsage;
    SIZE_T QuotaPagedPoolUsage;
    SIZE_T QuotaPeakNonPagedPoolUsage;
    SIZE_T QuotaNonPagedPoolUsage;
    SIZE_T PagefileUsage;
    SIZE_T PeakPagefileUsage;
} VM_COUNTERS;

typedef struct _SYSTEM_THREADS {
    LARGE_INTEGER KernelTime;
    LARGE_INTEGER UserTime;
    LARGE_INTEGER CreateTime;
    ULONG WaitTime;
    PVOID StartAddress;
    CLIENT_ID ClientId;
    KPRIORITY Priority;
    KPRIORITY BasePriority;
    ULONG ContextSwitchCount;
    LONG State;
    LONG WaitReason;
} SYSTEM_THREADS, *PSYSTEM_THREADS;

typedef struct _SYSTEM_PROCESSES {
    ULONG NextEntryDelta;
    ULONG ThreadCount;
    ULONG Reserved1[6];
    LARGE_INTEGER CreateTime;
    LARGE_INTEGER UserTime;
    LARGE_INTEGER KernelTime;
    UNICODE_STRING ProcessName;
    KPRIORITY BasePriority;
    ULONG ProcessId;
    ULONG InheritedFromProcessId;
    ULONG HandleCount;
    ULONG Reserved2[2];
    VM_COUNTERS VmCounters;
#if _WIN32_WINNT >= 0x500
    IO_COUNTERS IoCounters;
#endif
    SYSTEM_THREADS Threads[1];
} SYSTEM_PROCESSES, *PSYSTEM_PROCESSES;

typedef VOID (WINAPI *PFNENUMPROC)(PSYSTEM_PROCESSES, LPVOID);
typedef PFNENUMPROC LPFNENUMPROC;

typedef VOID (WINAPI *PFNENUMMOD)(LPTSTR, LPVOID, DWORD, LPVOID);
typedef PFNENUMMOD LPFNENUMMOD;

typedef VOID (WINAPI *PFNENUMMEM)(PMEMORY_BASIC_INFORMATION, LPVOID);
typedef PFNENUMMEM LPFNENUMMEM;

typedef VOID (WINAPI *PFNSEARCH)(LPBYTE, LPBYTE, LPVOID lpParam);
typedef PFNSEARCH LPFNSEARCH;

typedef VOID (WINAPI *PFNMODSEARCH)(LPTSTR, LPTSTR, DWORD, LPTSTR, LPVOID);
typedef PFNMODSEARCH LPFNMODSEARCH;

BOOL WINAPI LoadProcessDLLs();
BOOL WINAPI EnumProcessesNt(LPFNENUMPROC pfnEnumProc, LPVOID lpParam);
BOOL WINAPI EnumProcesses9x(LPFNENUMPROC pfnEnumProc, LPVOID lpParam);
BOOL WINAPI EnumModulesNt(DWORD ProcessId, LPFNENUMMOD pfnEnumProc, LPVOID lpParam);
BOOL WINAPI EnumModules9x(DWORD ProcessId, LPFNENUMMOD pfnEnumProc, LPVOID lpParam);
BOOL WINAPI EnumMemRegions(DWORD ProcessId, LPFNENUMMEM pfnEnumProc, LPVOID lpParam);
BOOL WINAPI GetProcessModulePath(DWORD dwProcessId, LPTSTR ModulePath);
BOOL WINAPI GetProcessUserName(DWORD dwProcessId, LPTSTR UserName);
BOOL WINAPI GetProcessPrivilege(DWORD dwProcessId, LPTSTR PrivilegeName, BOOL *Enabled);
BOOL WINAPI GetProcessPrivileges(DWORD dwProcessId, LPTSTR *PrivilegeNames, BOOL **Enabled, LPDWORD Count);
BOOL WINAPI SetProcessPrivilege(DWORD dwProcessId, LPTSTR PrivilegeName, BOOL Enable);
BOOL WINAPI SetProcessPrivileges(DWORD dwProcessId, LPTSTR PrivilegeNames, BOOL *Enabled, DWORD Count);
BOOL WINAPI KillProcess(DWORD dwProcessId);
BOOL WINAPI KillProcessTree(DWORD dwProcessId);
BOOL WINAPI GetProcessStartupInfo(DWORD dwProcessId, LPTSTR *lplpStartupDir, LPTSTR *lplpCmdLine, LPTSTR *lplpEnvVars);
VOID WINAPI GetProtectStr(LPTSTR buf, DWORD Protect);
BOOL WINAPI ProcessSearchData(HANDLE hProcess, LPBYTE lpStartAddr, LPBYTE Data, DWORD cbSize, LPBYTE *lplpAddr, BOOL IgnoreCase, LPFNSEARCH pfnSearch, LPVOID lpParam, LPBOOL Terminate);
BOOL WINAPI SearchForModule(LPTSTR lpszModName, LPFNMODSEARCH pfnModSearch, LPVOID lpParam);

#endif  // _PROCESS_H_

