//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "TMemForm.h"
#include "Main.h"
#include "TSaveTableDlg.h"
#include "Utils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "Placemnt"
#pragma resource "*.dfm"
TMemForm *MemForm;
//---------------------------------------------------------------------------
__fastcall TMemForm::TMemForm(TComponent* Owner)
    : TForm(Owner)
{
    LastCol = 0;
    OrderBy = 1;
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::SetProcess(TMyProcess *prc)
{
    Process = prc;
    if (!prc)
    {
        MemList->Items->Clear();
        Caption = "";
        StatusBar1->Panels->Items[0]->Text = "";
        StatusBar1->Panels->Items[1]->Text = "";
        StatusBar1->Panels->Items[2]->Text = "";
        StatusBar1->Panels->Items[3]->Text = "";
        StatusBar2->Panels->Items[1]->Text = "";
        StatusBar2->Panels->Items[2]->Text = "";
        StatusBar2->Panels->Items[3]->Text = "";
        return;
    }
    RefreshList();
}
//---------------------------------------------------------------------------
VOID WINAPI EnumProc(PMEMORY_BASIC_INFORMATION mem, LPVOID lpParam)
{
    TListView *list = (TListView *)lpParam;
    TListItem *li = list->Items->Add();
    char buf[64];

    li->Data = mem->BaseAddress;

    wsprintf(buf, "%08X", mem->BaseAddress);
    li->Caption = buf;
    wsprintf(buf, "%08X", mem->AllocationBase);
    li->SubItems->Add(buf);
    GetProtectStr(buf, mem->AllocationProtect);
    li->SubItems->Add(buf);
    FormatSize(buf, mem->RegionSize);
    li->SubItems->Add(buf);

    switch (mem->State)
    {
        case MEM_COMMIT:
            lstrcpy(buf, "Committed");
            MemForm->Committed += mem->RegionSize;
            break;
        case MEM_FREE:
            lstrcpy(buf, "Free");
            MemForm->Free += mem->RegionSize;
            break;
        case MEM_RESERVE:
            lstrcpy(buf, "Reserved");
            MemForm->Reserved += mem->RegionSize;
            break;
        default: lstrcpy(buf, "N/A");
    }
    li->SubItems->Add(buf);

    GetProtectStr(buf, mem->Protect);
    li->SubItems->Add(buf);

    switch (mem->Type)
    {
        case MEM_IMAGE:
            lstrcpy(buf, "Image");
            MemForm->Image += mem->RegionSize;
            break;
        case MEM_MAPPED: lstrcpy(buf, "Mapped");
            MemForm->Mapped += mem->RegionSize;
            break;
        case MEM_PRIVATE: lstrcpy(buf, "Private");
            MemForm->Private += mem->RegionSize;
            break;
        default: lstrcpy(buf, "N/A");
    }
    li->SubItems->Add(buf);
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::RefreshList()
{
    char buf[MAX_PATH + 64];

    MemList->Items->Clear();
    if (!Process) return;
    MemList->Enabled = Process->pid;
    Private = Mapped = Image = 0;

    Private = Mapped = Image = 0;
    Committed = Free = Reserved = 0;

    EnumMemRegions(Process->pid, EnumProc, MemList);

    wsprintf(buf, "   \"%s\" (PID=%lu)",
        Process->ProcessName, Process->pid);
    Caption = buf;

    SortColumn(LastCol, OrderBy);

    StatusBar1->Panels->Items[0]->Text =
        AnsiString(" : ") + MemList->Items->Count;
    FormatSize(buf, Private);
    StatusBar1->Panels->Items[1]->Text = AnsiString("Private: ") + buf;
    FormatSize(buf, Mapped);
    StatusBar1->Panels->Items[2]->Text = AnsiString("Mapped: ") + buf;
    FormatSize(buf, Image);
    StatusBar1->Panels->Items[3]->Text = AnsiString("Image: ") + buf;

    FormatSize(buf, Committed);
    StatusBar2->Panels->Items[1]->Text = AnsiString("Committed: ") + buf;
    FormatSize(buf, Reserved);
    StatusBar2->Panels->Items[2]->Text = AnsiString("Reserved: ") + buf;
    FormatSize(buf, Free);
    StatusBar2->Panels->Items[3]->Text = AnsiString("Free: ") + buf;
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::RefreshBtnClick(TObject *Sender)
{
    RefreshList();
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::FormResize(TObject *Sender)
{
    ToolBar->Width = ControlBar->ClientWidth;
}
//---------------------------------------------------------------------------
int __stdcall CustomSortProc3(long Item1, long Item2, long ParamSort)
{
    int res, n = (ParamSort & 0xFFFF), OrderBy = ParamSort >> 16;
    TListItem *li1 = (TListItem *)Item1, *li2 = (TListItem *)Item2;

    if (!n)
        res = MyCompareStr(li1->Caption.c_str(),
            li2->Caption.c_str(), true);
    else
        res = MyCompareStr(li1->SubItems->Strings[n - 1].c_str(),
            li2->SubItems->Strings[n - 1].c_str(), n == 3);

    return res * OrderBy;
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::SortColumn(int Col, int OrderBy)
{
    int i = 0, k = MemList->Columns->Count;
    LVCOLUMN col;
    for (; i < k; i++)
    {
        col.mask = LVCF_FMT;
        col.fmt = 0;
        if (i == Col)
        {
            col.fmt |=
                LVCFMT_BITMAP_ON_RIGHT |
                LVCFMT_COL_HAS_IMAGES |
                LVCFMT_IMAGE;
            col.mask |= LVCF_IMAGE;
            col.iImage = OrderBy > 0 ? 3 : 4;
        }
        ListView_SetColumn(MemList->Handle, i, &col);
    }
    MemList->CustomSort(CustomSortProc3, Col | (OrderBy << 16));
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::MemListColumnClick(TObject *Sender,
      TListColumn *Column)
{
    int index = Column->Index;
    OrderBy = index == LastCol ? OrderBy * -1 : 1;
    SortColumn(index, OrderBy);
    LastCol = index;
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::StayOnTopBtnClick(TObject *Sender)
{
    SetStayOnTop(Handle, StayOnTopBtn->Down);
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    MainForm->iMemInfo->Checked = false;
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::SaveBtnClick(TObject *Sender)
{
    SaveTableDlg->Save(MemList);
}
//---------------------------------------------------------------------------
void __fastcall TMemForm::MemListKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if (Key == VK_F5) RefreshList();
    else if (Key == 'S' && Shift.Contains(ssCtrl))
        SaveBtnClick(NULL);
}
//---------------------------------------------------------------------------

