//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "TThreadsForm.h"
#include "Main.h"
#include "TSaveTableDlg.h"
#include "Columns.h"
#include "EditSecurity.h"
#include "Utils.h"
#include "TCallStackDlg.h"
#include <stdio.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "Placemnt"
#pragma resource "*.dfm"
TThreadsForm *ThreadsForm;
//---------------------------------------------------------------------------
__fastcall TThreadsForm::TThreadsForm(TComponent* Owner)
    : TForm(Owner)
{
    LastCol = 0;
    OrderBy = 1;
    if (!IsWinNT())
    {
        ThreadPopup->Items->Delete(ThreadPopup->Items->IndexOf(iSecurity));
        ThreadPopup->Items->Delete(ThreadPopup->Items->IndexOf(N3));
    }
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::SetProcess(TMyProcess *prc)
{
    char buf[MAX_PATH + 64];
    Process = prc;
    if (!prc)
    {
        ThreadsList->Items->Clear();
        Caption = "";
        StatusBar->SimpleText = "";
        return;
    }
    RefreshList();
	wsprintf(buf, "  \"%s\" PID=%lu (0x%p)",
		Process->ProcessName, Process->pid, Process->pid);
    Caption = buf;
    SortColumn(LastCol, OrderBy);
    StatusBar->SimpleText = AnsiString(" : ") +
        Process->ThreadCount;
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::RefreshList()
{
    if (!Process) return;

	HANDLE hProcess;
	if (!(hProcess = OpenProcess(
		PROCESS_QUERY_INFORMATION,
		FALSE,
		Process->pid)))
	{
		MainForm->iThreadsList->Checked = false;
		Close();
		return;
	}
	CloseHandle(hProcess);

    TListItems *lis = ThreadsList->Items;
    TListItem *li;
    PSYSTEM_THREADS pThread;
    int i, j, k = ThreadsList->Items->Count;

    bool *Used = (bool *)LocalAlloc(LMEM_FIXED | LMEM_ZEROINIT,
        k * sizeof(bool));

    for (j = 0; j < (int)Process->ThreadCount; j++)
    {
        pThread = Process->Threads + j;

        for (i = 0; i < k; i++)
        {
            li = lis->Item[i];
            if (!Used[i] &&
                (DWORD)li->Data == pThread->ClientId.UniqueThread)
            {
                Used[i] = true;
                AddThreadToList(li, pThread);
                break;
            }
        }

        if (i == k)
        {
            li = lis->Add();
            li->Data = (LPVOID)pThread->ClientId.UniqueThread;
            AddThreadToList(li, pThread);
        }
    }

    for (i = 0, j = 0; i < k; i++)
        if (!Used[i])
            ThreadsList->Items->Delete(j);
        else j++;

    LocalFree(Used);
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
    MainForm->iThreadsList->Checked = false;
}
//---------------------------------------------------------------------------
int __stdcall CustomSortProc4(long Item1, long Item2, long ParamSort)
{
    int res, n = (ParamSort & 0xFFFF), OrderBy = ParamSort >> 16;
    TListItem *li1 = (TListItem *)Item1, *li2 = (TListItem *)Item2;

    if (!n)
        res = MyCompareStr(li1->Caption.c_str(),
            li2->Caption.c_str(), true);
    else
        res = MyCompareStr(li1->SubItems->Strings[n - 1].c_str(),
            li2->SubItems->Strings[n - 1].c_str(), n == 6 || n == 7 || n == 8);

    return res * OrderBy;
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::SortColumn(int Col, int OrderBy)
{
    int i = 0, k = ThreadsList->Columns->Count;
    LVCOLUMN col;
    for (; i < k; i++)
    {
        col.mask = LVCF_FMT;
        col.fmt = 0;
        if (i == Col)
        {
            col.fmt |=
                LVCFMT_BITMAP_ON_RIGHT |
                LVCFMT_COL_HAS_IMAGES |
                LVCFMT_IMAGE;
            col.mask |= LVCF_IMAGE;
            col.iImage = OrderBy > 0 ? 4 : 5;
        }
        ListView_SetColumn(ThreadsList->Handle, i, &col);
    }
    ThreadsList->CustomSort(CustomSortProc4, Col | (OrderBy << 16));
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::ThreadsListColumnClick(TObject *Sender,
      TListColumn *Column)
{
    int index = Column->Index;
    OrderBy = index == LastCol ? OrderBy * -1 : 1;
    SortColumn(index, OrderBy);
    LastCol = index;
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::AddThreadToList(TListItem *li,
    PSYSTEM_THREADS pThread)
{
    TStrings *ts = li->SubItems;
    int k = ThreadsList->Columns->Count, n = 0, m = ts->Count;
    char buf[64];

    FormatThreadData(pThread, 0, buf);
    if (li->Caption != buf)
        li->Caption = buf;

    for (int i = 1; i < k; i++)
    {
        FormatThreadData(pThread, i, buf);
        if (n < m)
        {
            if (lstrcmpi(ts->Strings[n].c_str(), buf))
                ts->Strings[n] = buf;
        } else
        {
            ts->Add(buf);
            m++;
        }
        n++;
    }

    while (k < m)
    {
        li->SubItems->Delete(k);
        m--;
    }
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::FormResize(TObject *Sender)
{
    ToolBar->Width = ControlBar->ClientWidth;
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::StayOnTopBtnClick(TObject *Sender)
{
    SetStayOnTop(Handle, StayOnTopBtn->Down);
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::RefreshBtnClick(TObject *Sender)
{
    RefreshList();    
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::SaveBtnClick(TObject *Sender)
{
    SaveTableDlg->Save(ThreadsList);
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::ThreadsListSelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
    TerminateBtn->Enabled = ThreadsList->SelCount;
	iTerminate->Enabled = ThreadsList->SelCount;
	iCallStack->Enabled = ThreadsList->Selected;
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::TerminateBtnClick(TObject *Sender)
{
    int i = 0, k = ThreadsList->Items->Count;
    if (k == 1)
    {
        Application->MessageBox(
            "     ",
            "", MB_OK | MB_ICONEXCLAMATION);
        return;
    }
    if (k == ThreadsList->SelCount)
    {
        Application->MessageBox(
            "     ",
            "", MB_OK | MB_ICONEXCLAMATION);
        return;
    }
    if (Application->MessageBox("  ?",
        "", MB_YESNO | MB_ICONWARNING) != IDYES) return;
    HANDLE hThread;
    DWORD id;
    for (; i < k; i++)
    {
        if (!ThreadsList->Items->Item[i]->Selected) continue;
        id = (DWORD)ThreadsList->Items->Item[i]->Data;
        if ((hThread = OpenThread(THREAD_TERMINATE, FALSE, id)) == NULL ||
            !TerminateThread(hThread, 0))
        {
            char buf[64];
            wsprintf(buf, "   #%lu", id);
            ShowGUIError(Handle, buf);
            break;
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::ThreadsListKeyDown(TObject *Sender,
      WORD &Key, TShiftState Shift)
{
    switch (Key)
    {
        case VK_F5:
            RefreshList();
            break;
        case 'S':
            if (Shift.Contains(ssCtrl))
                SaveBtnClick(NULL);
            break;
        case 'A':
            if (Shift.Contains(ssCtrl))
                ThreadsList->SelectAll();
            break;
        case VK_DELETE:
            if (TerminateBtn->Enabled)
                TerminateBtnClick(NULL);
            break;
    }
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::iTerminateClick(TObject *Sender)
{
    TerminateBtnClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::SuspendResume(bool Suspend)
{
    HANDLE hThread;
    DWORD id;
    char buf[64];
    int i = 0, k = ThreadsList->Items->Count;
    for (; i < k; i++)
    {
        if (!ThreadsList->Items->Item[i]->Selected) continue;
        id = (DWORD)ThreadsList->Items->Item[i]->Data;
        if ((hThread = OpenThread(THREAD_SUSPEND_RESUME, FALSE, id)) == NULL)
        {
            wsprintf(buf, "   #%lu", id);
            ShowGUIError(Handle, buf);
            break;
        }
        if (Suspend)
        {
            if (SuspendThread(hThread) == 0xFFFFFFFF)
            {
                wsprintf(buf, "    #%lu", id);
                ShowGUIError(Handle, buf);
                break;
            }
        }
        else
        {
            if (ResumeThread(hThread) == 0xFFFFFFFF)
            {
                wsprintf(buf, "    #%lu", id);
                ShowGUIError(Handle, buf);
                break;
            }
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::ResumeBtnClick(TObject *Sender)
{
    SuspendResume(false);
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::SuspendBtnClick(TObject *Sender)
{
	SuspendResume(true);
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::SetPriority(TObject *Sender)
{
    int Priority = 0;
	int i = 0, k = ThreadsList->Items->Count, id;
	HANDLE hThread;

    if (Sender == iTimeCritical)
        Priority = THREAD_PRIORITY_TIME_CRITICAL;
    else if (Sender == iHighest)
        Priority = THREAD_PRIORITY_HIGHEST;
    else if (Sender == iAboveNormal)
        Priority = THREAD_PRIORITY_ABOVE_NORMAL;
    else if (Sender == iNormal)
        Priority = THREAD_PRIORITY_NORMAL;
    else if (Sender == iBelowNormal)
        Priority = THREAD_PRIORITY_BELOW_NORMAL;
    else if (Sender == iLowest)
        Priority = THREAD_PRIORITY_LOWEST;
    else if (Sender == iIdle)
        Priority = THREAD_PRIORITY_IDLE;

    for (; i < k; i++)
    {
        if (!ThreadsList->Items->Item[i]->Selected) continue;
		id = (DWORD)ThreadsList->Items->Item[i]->Data;
		if ((hThread = OpenThread(THREAD_SET_INFORMATION, FALSE, id)) == NULL ||
			!SetThreadPriority(hThread, Priority))
		{
			char buf[64];
            wsprintf(buf, "    #%lu", id);
            ShowGUIError(Handle, buf);
            break;
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::iSecurityClick(TObject *Sender)
{
    if (!EditThreadSecurity(Handle, (DWORD)ThreadsList->Selected->Data))
        ShowGUIError(Handle,
        " /   ");
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::ThreadsListMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if (Button != mbRight) return;
    TListItem *li = ThreadsList->GetItemAt(X, Y);
    if (!li) return;
    TPoint p = ThreadsList->ClientToScreen(TPoint(X, Y));
    ThreadPopup->Popup(p.x, p.y);
}
//---------------------------------------------------------------------------
bool __fastcall TThreadsForm::ReadProcessMem(
	HANDLE hProcess,
	PVOID pRemoteAddr,
	PDWORD pLocalAddr)
{
	DWORD dwRead = 0;
	return ReadProcessMemory(
		hProcess,
		pRemoteAddr,
		pLocalAddr,
		sizeof(DWORD),
		&dwRead);
}
//---------------------------------------------------------------------------
void __fastcall TThreadsForm::iCallStackClick(TObject *Sender)
{
	if (!ThreadsList->Selected)
	{
		return;
	}
	DWORD id = (DWORD)ThreadsList->Selected->Data;
	HANDLE hThread;
	HANDLE hProcess = OpenProcess(PROCESS_VM_READ, FALSE, Process->pid);

	if (!hProcess ||
		!(hThread = OpenThread(
			THREAD_GET_CONTEXT | THREAD_SUSPEND_RESUME,
			FALSE, id)))
	{
		return;
	}
	CONTEXT	ctx;
	ZeroMemory(&ctx,sizeof(ctx));
	ctx.ContextFlags = CONTEXT_FULL;

	if (GetCurrentThreadId() != id)
	{
		SuspendThread(hThread);
	}
	GetThreadContext(hThread,&ctx);

	PDWORD EBP = (PDWORD)ctx.Ebp;
	DWORD dwT, dwT2;
	TStringList* list = new TStringList();
	char szBuf[32];

	while (true)
	{
		if (!ReadProcessMem(hProcess, EBP + 1, &dwT) ||
			!ReadProcessMem(hProcess, (PVOID)dwT, &dwT2))
		{
			break;
		}

		sprintf(szBuf, "0x%p", dwT);
		list->Add(szBuf);

		if (!ReadProcessMem(hProcess, EBP, &dwT))
		{
			break;
		}
		EBP = (PDWORD)dwT;
	}

	ResumeThread(hThread);

	CallStackDlg->Execute(list, ctx.Eip);
	delete list;
}
//---------------------------------------------------------------------------

