//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#define CACHE_SIZE      4096

#include "TViewMemForm.h"
#include "TEditMemDlg.h"
#include "TSearchDlg.h"
#include "SearchThread.h"
#include "TProgressDlg.h"
#include "Utils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "Placemnt"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
__fastcall TViewMemForm::TViewMemForm(TComponent* Owner, TMyProcess *prc)
    : TForm(Owner)
{
    if ((hProcess = OpenProcess(PROCESS_VM_READ | PROCESS_QUERY_INFORMATION,
        FALSE, prc->pid)) == INVALID_HANDLE_VALUE)
    {
        Application->MessageBox("     ",
            "", MB_OK | MB_ICONSTOP);
        throw 0;
    }

    Process = prc;
    Cache = (LPBYTE)malloc(CACHE_SIZE);
    CacheAddr = 0;
    CurRegion = NULL;
    CurRegionSize = 0;
    ReadCache();

    RefreshList();

    Caption = AnsiString("    \"") +
        prc->ProcessName + "\"";

    MemGrid->ColWidths[0] = 96;
    MemGrid->ColWidths[17] = 160;
    MemGrid->Col = 1;
    MemGrid->Visible = true;
}
//---------------------------------------------------------------------------
__fastcall TViewMemForm::~TViewMemForm()
{
    free(Cache);
    if (hProcess != INVALID_HANDLE_VALUE) CloseHandle(hProcess);
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
    delete this;
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::MemGridDrawCell(TObject *Sender, int ACol,
      int ARow, TRect &Rect, TGridDrawState State)
{
    int x, y, i;
    char buf[64], *p;

    if (!ACol)
    {
        wsprintf(buf, "%07X0:", ARow);
    }
    else if (ACol != 17)
    {
        DWORD RealAddr = ARow;
        RealAddr = (RealAddr << 4) + ACol - 1;
        BYTE b = GetFromCache((LPBYTE)RealAddr);
        if (WasRead)
            wsprintf(buf, "%02X", b);
        else
            lstrcpy(buf, "??");
    }
    else
    {
        DWORD RealAddr = DWORD(ARow) << 4;
        p = buf;
        if (!WasRead)
            lstrcpy(buf, "................");
        else
        for (i = 0; i < 16; i++, p++)
        {
            *p = GetFromCache((LPBYTE)RealAddr++);
            if ((BYTE)*p < 0x20 || *p == -1) *p = '.';
        }
        buf[16] = 0;
    }

//    TSize size = Canvas->TextExtent((char *)buf);
    x = Rect.Left + 4;// + ((Rect.Width() - size.cx) >> 1);
    y = Rect.Top;// + ((Rect.Height() - size.cy) >> 1);
    TextOut(MemGrid->Canvas->Handle, x, y, buf, lstrlen(buf));
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::ReadCache()
{
    WasRead = ReadProcessMemory(hProcess, CacheAddr, Cache, CACHE_SIZE, NULL);
}
//---------------------------------------------------------------------------
inline BYTE TViewMemForm::GetFromCache(LPBYTE Addr)
{
    if (Addr < CacheAddr || Addr >= CacheAddr + CACHE_SIZE)
    {
        CacheAddr = (LPBYTE)((DWORD)Addr & 0xFFFFF000);
        ReadCache();
    }
    return Cache[Addr - CacheAddr];
}
//---------------------------------------------------------------------------
VOID WINAPI EnumProc2(PMEMORY_BASIC_INFORMATION mem, LPVOID lpParam)
{
    TViewMemForm *frm = (TViewMemForm *)lpParam;
    TListItem *li = frm->RegsList->Items->Add();
    char buf[64];
    wsprintf(buf, "%08X - %08X", mem->BaseAddress,
        (DWORD)mem->BaseAddress + mem->RegionSize - 1);
    li->Caption = buf;
    li->Data = mem->BaseAddress;
    frm->VMSize += mem->RegionSize;
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::RefreshList()
{
    RegsList->Items->Clear();
    VMSize = 0;
    EnumMemRegions(Process->pid, EnumProc2, this);
    MemGrid->RowCount = VMSize >> 4;
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::RegsListSelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
    MemGrid->TopRow = (DWORD)Item->Data >> 4;
    MemGrid->Row = MemGrid->TopRow;
    MemGrid->Col = 1;
    MemGrid->Refresh();
    UpdateRegionInfo((LPBYTE)Item->Data);
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::MemGridSelectCell(TObject *Sender, int ACol,
      int ARow, bool &CanSelect)
{
    char buf[256];
    LPBYTE RealAddr = GetAddr(ARow, ACol);

    if (ACol >= 1 && ACol <= 16)
    {
        BYTE b = GetFromCache(RealAddr);
        wsprintf(buf, " : %08X", RealAddr);
        StatusBar2->Panels->Items[0]->Text = buf;
        if (WasRead)
        {
            char BinBuf[9], OctBuf[4];
            ByteToBinStr(b, BinBuf);
            ByteToOctStr(b, OctBuf);
            wsprintf(buf, "Dec: %d, Hex: %02X, Bin: %s, Oct: %s",
                b, b, BinBuf, OctBuf);
            StatusBar2->Panels->Items[1]->Text = buf;
        }
        else
            StatusBar2->Panels->Items[1]->Text = " ";
        UpdateRegionInfo(RealAddr);
    }
    else
    {
        UpdateRegionInfo(RealAddr);
        StatusBar2->Panels->Items[0]->Text = "";
        StatusBar2->Panels->Items[1]->Text = "";
    }
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::UpdateRegionInfo(LPBYTE Addr)
{
    MEMORY_BASIC_INFORMATION mbi;
    char buf[256];

    if (DWORD(Addr - CurRegion) < CurRegionSize)
        return;

    int k = RegsList->Items->Count, i = k - 1;
    while (Addr < RegsList->Items->Item[i]->Data) i--;
    if (!VirtualQueryEx(hProcess, RegsList->Items->Item[i]->Data,
        &mbi, sizeof(mbi))) return;

    CurRegion = (LPBYTE)mbi.BaseAddress;
    CurRegionSize = mbi.RegionSize;

    wsprintf(buf, " : %08X", CurRegion);
    StatusBar1->Panels->Items[0]->Text = buf;
    lstrcpy(buf, ": ");
    FormatSize(buf + lstrlen(buf), CurRegionSize);
    StatusBar1->Panels->Items[1]->Text = buf;
    lstrcpy(buf, ": ");
    GetProtectStr(buf + lstrlen(buf), mbi.Protect);
    StatusBar1->Panels->Items[2]->Text = buf;
}
//---------------------------------------------------------------------------
inline LPBYTE __fastcall TViewMemForm::GetAddr(DWORD Row, DWORD Col)
{
    if (!Col || Col == 17) return LPBYTE(Row << 4);
    return LPBYTE((Row << 4) + Col - 1);
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::EditMem()
{
    if (EditMemDlg->Execute(hProcess, Process->pid,
        (LPBYTE)GetAddr(MemGrid->Row, MemGrid->Col)))
    {
        ReadCache();
        MemGrid->Repaint();
    }
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::MemGridDblClick(TObject *Sender)
{
    EditMem();
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::MemGridKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    switch (Key)
    {
        case VK_RETURN:
            EditMem();
            break;
        case VK_F5:
            GotoBtnClick(NULL);
            break;
        case VK_F7:
            if (Shift.Contains(ssShift))
            {
                if (SearchAgainBtn->Enabled)
                    SearchAgainBtnClick(NULL);
            }
            else SearchBtnClick(NULL);
            break;
    }
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::FormResize(TObject *Sender)
{
    ToolBar->Width = ControlBar->ClientWidth;
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::EditBtnClick(TObject *Sender)
{
    EditMem();
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::GotoBtnClick(TObject *Sender)
{
    DWORD Addr = (DWORD)GetAddr(MemGrid->Row, MemGrid->Col);
    char buf[32];
    wsprintf(buf, "%08X", Addr);
    AnsiString val = buf;
    if (!InputQuery("  ", " :", val)) return;
    if (!HexStrToDWord(val.c_str(), (LPDWORD)&Addr))
    {
        Application->MessageBox("  ", "",
            MB_OK | MB_ICONEXCLAMATION);
        return;
    }
    MemGrid->Row = Addr >> 4;
    MemGrid->TopRow = MemGrid->Row;
    MemGrid->Col = (Addr & 0x0F) + 1;
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::SearchBtnClick(TObject *Sender)
{
    if (SearchDlg->ShowModal() != mrOk) return;
    Repaint();
    Search(GetAddr(MemGrid->Row, MemGrid->Col));
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::Search(LPBYTE lpStartAddr)
{
    TSearchThread *SearchThread = new TSearchThread(false,
        hProcess, lpStartAddr,
        (LPBYTE)SearchDlg->Buf, SearchDlg->dwSize,
        SearchDlg->IgnoreCase->Checked);
    ProgressDlg->Execute();
    SearchThread->TimeToTerminate = TRUE;
    SearchThread->WaitFor();
    SearchAgainBtn->Enabled = SearchThread->Found;
    if (!SearchThread->Found)
        Application->MessageBox("   ",
            "", MB_OK | MB_ICONEXCLAMATION);
    else
    {
        LastFoundAddr = SearchThread->lpFoundAddr;
        MemGrid->Row = (DWORD)LastFoundAddr >> 4;
        int TopRow = MemGrid->Row - (MemGrid->VisibleRowCount >> 1);
        if (TopRow < 0) TopRow = 0;
        MemGrid->TopRow = TopRow;
        MemGrid->Col = ((DWORD)LastFoundAddr & 0x0F) + 1;
    }
    delete SearchThread;
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::SearchAgainBtnClick(TObject *Sender)
{
    LPBYTE lpAddr = GetAddr(MemGrid->Row, MemGrid->Col);
    Repaint();
    Search(lpAddr + (lpAddr == LastFoundAddr ? 1 : 0));
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::ToolButton2Click(TObject *Sender)
{
    RefreshList();
}
//---------------------------------------------------------------------------
void __fastcall TViewMemForm::RegsListKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if (Key == VK_F5) RefreshList();
}
//---------------------------------------------------------------------------

