#include "Utils.h"
#include <commctrl.h>
#include <fstream.h>
#include <shlobj.h>

BOOL IsWinNT()
{
    static int IsIt = -1;
	OSVERSIONINFO osvi;
    if (IsIt < 0)
    {
    	osvi.dwOSVersionInfoSize = sizeof(osvi);
	    GetVersionEx(&osvi);
    	IsIt = osvi.dwPlatformId == VER_PLATFORM_WIN32_NT;
    }
    return IsIt;
}

void FormatSize2(LPTSTR buf, DWORD Size)
{
    TCHAR t[8];
    DWORD d = 1000000000;
    int i = 0;

    if (!Size)
    {
        buf[0] = '0';
        buf[1] = 0;
        return;
    }
    else buf[0] = 0;

    for (; i < 4; i++, d /= 1000)
    {
        if (!Size) lstrcat(buf, ",000");
        else if (Size >= d)
        {
            if (!buf[0]) wsprintf(buf, "%d", Size / d);
            else
            {
                wsprintf(t, "%03d", Size / d);
                lstrcat(buf, ",");
                lstrcat(buf, t);
            }
        }
        Size %= d;
    }
}

void FormatSize(LPTSTR buf, DWORD Size)
{
    if (Size > 1024)
    {
        FormatSize2(buf, Size >> 10);
        lstrcat(buf, " K");
    }
    else wsprintf(buf, "%d", Size);
}

int MyCompareStr(LPTSTR s1, LPTSTR s2, BOOL IsNumber)
{
    int l1 = lstrlen(s1), l2 = lstrlen(s2);
    return l1 == l2 || !IsNumber ? lstrcmp(s1, s2) : l1 - l2;
}

BOOL GetFileCreateDateTime(LPTSTR FileName, LPTSTR DateTime)
{
    HANDLE hFile;
    FILETIME ft, lt;
    SYSTEMTIME st;
    DateTime[0] = 0;

    if ((hFile = CreateFile(FileName, GENERIC_READ, FILE_SHARE_READ, NULL,
        OPEN_EXISTING, 0, 0)) == NULL) return FALSE;

    if (!GetFileTime(hFile, &ft, NULL, NULL))
    {
        CloseHandle(hFile);
        return FALSE;
    }

    FileTimeToLocalFileTime(&ft, &lt);
    FileTimeToSystemTime(&lt, &st);

    wsprintf(DateTime, "%02d.%02d.%d %d:%02d",
        st.wDay, st.wMonth, st.wYear, st.wHour, st.wMinute);

    CloseHandle(hFile);

    return TRUE;
}

BOOL GetFileWriteDateTime(LPTSTR FileName, LPTSTR DateTime)
{
    HANDLE hFile;
    FILETIME ft, lt;
    SYSTEMTIME st;
    DateTime[0] = 0;

    if ((hFile = CreateFile(FileName, GENERIC_READ, FILE_SHARE_READ, NULL,
        OPEN_EXISTING, 0, 0)) == NULL) return FALSE;

    if (!GetFileTime(hFile, NULL, NULL, &ft))
    {
        CloseHandle(hFile);
        return FALSE;
    }

    FileTimeToLocalFileTime(&ft, &lt);
    FileTimeToSystemTime(&lt, &st);

    wsprintf(DateTime, "%02d.%02d.%d %d:%02d",
        st.wDay, st.wMonth, st.wYear, st.wHour, st.wMinute);

    CloseHandle(hFile);

    return TRUE;
}

BOOL CopyStringToClipboard(LPSTR lpStr)
{
    if (!OpenClipboard(NULL)) return FALSE;

    EmptyClipboard();
    
    HANDLE hData = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE,
        lstrlen(lpStr) + 1);
    LPVOID lpData = GlobalLock(hData);
    lstrcpy((LPSTR)lpData, lpStr);
    GlobalUnlock(lpData);

    if (!SetClipboardData(CF_TEXT, hData)) return FALSE;

    CloseClipboard();

    return TRUE;
}

void SetStayOnTop(HWND hWnd, BOOL StayOnTop)
{
    SetWindowPos(hWnd, StayOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
        0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
}

void SetNumberStyle(HWND hWnd)
{
    SetWindowLong(hWnd, GWL_STYLE, GetWindowLong(hWnd, GWL_STYLE) | ES_NUMBER);
}

BOOL SaveListView(HWND hWnd, LPTSTR FileName, DWORD CodePage, DWORD Decoration)
{
    DWORD ColCount = 0, ItemCount, Max[256], i, j, t;
    LVCOLUMN col;
    LVITEM item;
    CHAR buf[512];
    ofstream out(FileName, ios::binary);

    ZeroMemory(Max, sizeof(Max));
    col.mask = LVCF_WIDTH | LVCF_TEXT;
    col.cx = -1;
    col.pszText = buf;
    col.cchTextMax = sizeof(buf);
    ListView_GetColumn(hWnd, ColCount, &col);
    while (col.cx >= 0)
    {
        t = lstrlen(buf);
        if (t > Max[ColCount]) Max[ColCount] = t;
        ColCount++;
        col.cx = -1;
        ListView_GetColumn(hWnd, ColCount, &col);
    }

    ItemCount = ListView_GetItemCount(hWnd);
    item.mask = LVIF_TEXT;
    item.pszText = buf;
    item.cchTextMax = sizeof(buf);
    for (i = 0; i < ColCount; i++)
    {
        item.iSubItem = i;
        for (j = 0; j < ItemCount; j++)
        {
            item.iItem = j;
            ListView_GetItem(hWnd, &item);
            t = lstrlen(buf);
            if (t > Max[i]) Max[i] = t;
        }
    }

    if (Decoration == 1)
    {
        out << '\xDA';
        for (i = 0; i < ColCount; i++)
        {
            if (i) out << '\xC2';
            for (j = 0; j < Max[i] + 2; j++)
                out << '\xC4';
        }
        out << "\xBF\x0D\x0A";
    }
    else if (Decoration == 2)
    {
        for (i = 0; i < ColCount; i++)
        {
            out << '+';
            for (j = 0; j < Max[i] + 2; j++)
                out << '-';
        }
        out << "+\x0D\x0A";
    }

    col.mask = LVCF_TEXT;
    if (Decoration == 1)
        out << "\xB3 ";
    else if (Decoration == 2)
        out << "| ";
    for (i = 0; i < ColCount; i++)
    {
        memset(buf, ' ', Max[i]);
        buf[Max[i]] = 0;
        col.mask = LVCF_TEXT;
        ListView_GetColumn(hWnd, i, &col);
        t = lstrlen(buf);
        if (t < Max[i]) buf[t] = ' ';
        if (!CodePage) CharToOem(buf, buf);
        out << buf;
        if (i < ColCount - 1)
        {
            if (!Decoration)
                out << "   ";
            else if (Decoration == 1)
                out << " \xB3 ";
            else if (Decoration == 2)
                out << " | ";
        }
        else
        {
            if (!Decoration)
                out << "\x0D\x0A";
            if (Decoration == 1)
                out << " \xB3\x0D\x0A";
            else if (Decoration == 2)
                out << " |\x0D\x0A";
        }
    }

    if (Decoration == 1)
    {
        out << '\xC3';
        for (i = 0; i < ColCount; i++)
        {
            if (i) out << '\xC5';
            for (j = 0; j < Max[i] + 2; j++)
                out << '\xC4';
        }
        out << "\xB4\x0D\x0A";
    }
    else if (Decoration == 2)
    {
        for (i = 0; i < ColCount; i++)
        {
            out << '+';
            for (j = 0; j < Max[i] + 2; j++)
                out << '-';
        }
        out << "+\x0D\x0A";
    }

    for (i = 0; i < ItemCount; i++)
    {
        item.iItem = i;
        if (Decoration == 1)
            out << "\xB3 ";
        else if (Decoration == 2)
            out << "| ";
        for (j = 0; j < ColCount; j++)
        {
            memset(buf, ' ', Max[j]);
            buf[Max[j]] = 0;
            item.iSubItem = j;
            ListView_GetItem(hWnd, &item);
            t = lstrlen(buf);
            if (t < Max[j]) buf[t] = ' ';
            if (!CodePage) CharToOem(buf, buf);
            out << buf;
            if (j < ColCount - 1)
            {
                if (!Decoration)
                    out << "   ";
                else if (Decoration == 1)
                    out << " \xB3 ";
                else if (Decoration == 2)
                    out << " | ";
            }
            else
            {
                if (!Decoration)
                    out << "\x0D\x0A";
                if (Decoration == 1)
                    out << " \xB3\x0D\x0A";
                else if (Decoration == 2)
                    out << " |\x0D\x0A";
            }
        }
    }

    if (Decoration == 1)
    {
        out << '\xC0';
        for (i = 0; i < ColCount; i++)
        {
            if (i) out << '\xC1';
            for (j = 0; j < Max[i] + 2; j++)
                out << '\xC4';
        }
        out << "\xD9\x0D\x0A";
    }
    else if (Decoration == 2)
    {
        for (i = 0; i < ColCount; i++)
        {
            out << '+';
            for (j = 0; j < Max[i] + 2; j++)
                out << '-';
        }
        out << "+\x0D\x0A";
    }

    out.close();

    return TRUE;
}

BOOL GetVersionString(LPTSTR FileName, LPTSTR VersionStr,
    LPTSTR lpBuf, DWORD cbBuf)
{
    HANDLE hHeap = GetProcessHeap();
    DWORD dwZero;
    TCHAR buf[256];
    LPSTR pText;
    UINT uSize;
    DWORD dwSize = GetFileVersionInfoSize(FileName, &dwZero);

    ZeroMemory(lpBuf, cbBuf);
    if (!dwSize) return FALSE;

    LPBYTE pBlock = (LPBYTE)HeapAlloc(hHeap, HEAP_ZERO_MEMORY, dwSize);
    if (!pBlock) return FALSE;
    if (!GetFileVersionInfo(FileName, 0, dwSize, pBlock))
        return FALSE;

    LPWORD pCP;
    UINT VarSize;
    if (!VerQueryValue(pBlock, "\\VarFileInfo\\Translation",
        (LPVOID*)&pCP, &VarSize))
    {
        HeapFree(hHeap, 0, pBlock);
        return TRUE;
    }

    if (!lstrcmpi(VersionStr, "VersionLanguage"))
    {
        VerLanguageName(pCP[0], lpBuf, cbBuf);
        HeapFree(hHeap, 0, pBlock);
        return TRUE;
    }

    wsprintf(buf, "\\StringFileInfo\\%04X%04X\\%s", pCP[0], pCP[1],
        VersionStr);

    if (!VerQueryValue(pBlock, buf, (LPVOID*)&pText, &uSize))
    {
        HeapFree(hHeap, 0, pBlock);
        return TRUE;
    }

    memcpy(lpBuf, pText, (cbBuf < uSize ? cbBuf : uSize) + 1);

    return TRUE;
}

BOOL ShowFileProperties(LPTSTR FileName)
{
    LPWSTR lpFileName, t;
    IShellFolder *shRoot = NULL, *shFolder = NULL;
    ITEMIDLIST *pidlFolder, *pidlFile;
    IContextMenu *ContextMenu = NULL;
    CMINVOKECOMMANDINFO ici;
    BOOL Res = FALSE;

    SHGetDesktopFolder(&shRoot);
    if (!shRoot) return FALSE;

#ifdef UNICODE
    lpFileName = FileName;
#else
    lpFileName = (LPWSTR)LocalAlloc(LMEM_FIXED | LMEM_ZEROINIT,
        (lstrlen(FileName) + 1) * sizeof(WCHAR));
    MultiByteToWideChar(CP_ACP, 0, FileName, -1, lpFileName,
        lstrlen(FileName) + 1);
#endif

    t = lpFileName + lstrlen(FileName);
    while (t > lpFileName && *t != '\\') t--;
    if (t > lpFileName) *t++ = 0;

    shRoot->ParseDisplayName(0, NULL, lpFileName, NULL, &pidlFolder, NULL);
    shRoot->BindToObject(pidlFolder, NULL, IID_IShellFolder,
        (LPVOID *)&shFolder);

    if (shFolder)
    {
        shFolder->ParseDisplayName(0, NULL, t, NULL, &pidlFile, NULL);
        shFolder->GetUIObjectOf(0, 1, (LPCITEMIDLIST *)&pidlFile, IID_IContextMenu,
            NULL, (LPVOID *)&ContextMenu);

        if (ContextMenu)
        {
            ZeroMemory(&ici, sizeof(ici));
            ici.cbSize = sizeof(ici);
            ici.fMask = CMIC_MASK_ASYNCOK;
            ici.hwnd = 0;
            ici.lpVerb = "properties";
            ici.nShow = SW_SHOWNORMAL;
            ContextMenu->InvokeCommand(&ici);
            Res = TRUE;
        }
    }

#ifndef UNICODE
    LocalFree(lpFileName);
#endif

    return Res;
}

void ByteToBinStr(BYTE b, LPTSTR lpStr)
{
    int i = 0;
    for (; i < 8; i++, b <<= 1)
        *lpStr++ = TCHAR(b & 0x80 ? '1' : '0');
    *lpStr = 0;
}

void WordToBinStr(WORD w, LPTSTR lpStr)
{
    int i = 0;
    for (; i < 16; i++, w <<= 1)
        *lpStr++ = TCHAR(w & 0x8000 ? '1' : '0');
    *lpStr = 0;
}

void DWordToBinStr(DWORD dw, LPTSTR lpStr)
{
    int i = 0;
    for (; i < 32; i++, dw <<= 1)
        *lpStr++ = TCHAR(dw & 0x80000000 ? '1' : '0');
    *lpStr = 0;
}

void ByteToOctStr(BYTE b, LPTSTR lpStr)
{
    *lpStr++ = (b >> 6) + '0';
    *lpStr++ = ((b >> 3) & 7) + '0';
    *lpStr++ = (b & 7) + '0';
    *lpStr = 0;
}

void WordToOctStr(WORD w, LPTSTR lpStr)
{
    *lpStr++ = (w >> 15) + '0';
    *lpStr++ = ((w >> 12) & 7) + '0';
    *lpStr++ = ((w >> 9) & 7) + '0';
    *lpStr++ = ((w >> 6) & 7) + '0';
    *lpStr++ = (w & 7) + '0';
    *lpStr = 0;
}

void DWordToOctStr(DWORD dw, LPTSTR lpStr)
{
    for (int i = 30; i >= 0; i -= 3)
        *lpStr++ = ((dw >> i) & 7) + '0';
    *lpStr = 0;
}

BOOL BinStrToDWord(LPSTR lpStr, LPDWORD lpDw)
{
    *lpDw = 0;

    if (lstrlen(lpStr) > 32) return FALSE;

    while (*lpStr)
    {
        *lpDw <<= 1;
        if (*lpStr != '0' && *lpStr != '1') return FALSE;
        *lpDw |= *lpStr++ - '0';
    }

    return TRUE;
}

BOOL OctStrToDWord(LPSTR lpStr, LPDWORD lpDw)
{
    *lpDw = 0;

    if (lstrlen(lpStr) > 11) return FALSE;

    while (*lpStr)
    {
        *lpDw <<= 3;
        if (*lpStr < '0' || *lpStr > '7') return FALSE;
        *lpDw |= *lpStr++ - '0';
    }

    return TRUE;
}

BOOL DecStrToDWord(LPSTR lpStr, LPDWORD lpDw)
{
    LPSTR t = lpStr;

    if (lstrlen(lpStr) > 10) return FALSE;

    while (*t)
    {
        if (*t < '0' || *t > '9') return FALSE;
        t++;
    }

    *lpDw = atol(lpStr);

    return TRUE;
}

BOOL HexStrToDWord(LPSTR lpStr, LPDWORD lpDw)
{
    *lpDw = 0;
    CHAR c;

    if (lstrlen(lpStr) > 8) return FALSE;

    while (*lpStr)
    {
        c = (CHAR)CharUpper((LPSTR)*lpStr);
        *lpDw <<= 4;
        if (!(c >= '0' && c <= '9') &&
            !(c >= 'A' && c <= 'F')) return FALSE;
        if (c <= '9')
            *lpDw |= c - '0';
        else
            *lpDw |= c - 'A' + 10;
        lpStr++;
    }

    return TRUE;
}

VOID WINAPI ShowGUIError(HWND hWnd, LPSTR Comment)
{
    char *lpMsgBuf = NULL, *msg;
    DWORD err = GetLastError();

    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER |
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        err,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR) &lpMsgBuf,
        0,
        NULL);

    if (!lpMsgBuf) msg = new char[512];
    else msg = new char[lstrlen(lpMsgBuf) + 512];

    wsprintf(msg, "%s\n : %d (0x%08x)\n: %s",
        Comment, err, err, lpMsgBuf ? lpMsgBuf : "");

    MessageBox(hWnd, msg, "", MB_OK | MB_ICONSTOP);

    LocalFree(lpMsgBuf);
}

