//#include "Types.h"

#include <windows.h>
#include <shlwapi.h>
#include <stdio.h>

#include "Ver.h"

//#pragma comment(lib, "version.lib")


BOOL	CModuleVersion::Init(PCSTR pFile)
{
	tTransl.charset = 1252;	// default = ANSI code page
	ZeroMemory(&vsFileInfo,sizeof(vsFileInfo));

	int		iVerLen;	
	DWORD	dwDummyHandle; // will always be set to zero
   
	if (0 >= (iVerLen=GetFileVersionInfoSize((PSTR)pFile, &dwDummyHandle)))
		return FALSE;
	
	pVersionInfo = malloc(iVerLen);
	
	if (!::GetFileVersionInfo((PSTR)pFile, dwDummyHandle, iVerLen, pVersionInfo))
		return FALSE;

	PVOID	pVI;
	UINT	dwLen;
	
	if (!VerQueryValue(pVersionInfo, "\\", &pVI, &dwLen))
		return FALSE;
	
	// copy fixed info to myself, which am derived from VS_FIXEDFILEINFO
	vsFileInfo = *(VS_FIXEDFILEINFO*)pVI;


	PTRANSLATION	pT;

   // Get translation info
   if (VerQueryValue(pVersionInfo,"\\VarFileInfo\\Translation", (PVOID*)&pT, &dwLen) && dwLen >= 4)
      tTransl = *pT;

   return vsFileInfo.dwSignature == VS_FFI_SIGNATURE;
}

void	CModuleVersion::Destroy()
{
	if (pVersionInfo)
	{
		free(pVersionInfo);
		pVersionInfo = NULL;
	}
}

PCSTR CModuleVersion::GetValue(PCSTR pKeyName)
{
	char	sQuery[1024];

	if (!pVersionInfo)
		return NULL;

	// To get a string value must pass query in the form
	//
	//    "\StringFileInfo\<langID><codepage>\keyname"
	//
	// where <langID><codepage> is the languageID concatenated with the
	// code page, in hex. Wow.
	//
	sprintf(sQuery,"\\StringFileInfo\\%04x%04x\\%s",tTransl.langID,tTransl.charset,pKeyName);

	PCSTR	pVal;
	UINT	dwLen;
	
	if (VerQueryValue(pVersionInfo, sQuery, (PVOID*)&pVal, &dwLen))
		return pVal;
	return NULL;
}


BOOL CDLLVersion::Init(PCSTR	pFile)
{
	this->hDLL = GetModuleHandle(pFile);
	
	if (hDLL)
		return Init(hDLL);

	if ((hDLL = LoadLibrary(pFile))!=NULL)
		bFreeAtExit = TRUE;
	
	return (hDLL) ? Init(hDLL) : FALSE;
}

BOOL CDLLVersion::Init(HMODULE hDLL)
{
	DLLGETVERSIONPROC	pDllGetVersion;
	
	pDllGetVersion = (DLLGETVERSIONPROC)GetProcAddress(hDLL, "DllGetVersion");
	
	if (!pDllGetVersion)
		return FALSE;

	this->hDLL = hDLL;

	ZeroMemory(&dvDVI,sizeof(dvDVI));
	dvDVI.info1.cbSize = sizeof(dvDVI);
	
	return SUCCEEDED((*pDllGetVersion)((DLLVERSIONINFO*)&dvDVI));
}

//void Destroy()
void CDLLVersion::Destroy()
{
	if (bFreeAtExit)
	{
		FreeLibrary(hDLL);
		hDLL = 0;
	}
}





// from winbase.h
#ifndef VER_PLATFORM_WIN32s
#define VER_PLATFORM_WIN32s             0
#endif
#ifndef VER_PLATFORM_WIN32_WINDOWS
#define VER_PLATFORM_WIN32_WINDOWS      1
#endif
#ifndef VER_PLATFORM_WIN32_NT
#define VER_PLATFORM_WIN32_NT           2
#endif
#ifndef VER_PLATFORM_WIN32_CE
#define VER_PLATFORM_WIN32_CE           3
#endif


/*
This table has been assembled from Usenet postings, personal
observations, and reading other people's code.  Please feel
free to add to it or correct it.

dwPlatFormID  dwMajorVersion  dwMinorVersion  dwBuildNumber
95             1              4               0             950
95 SP1         1              4               0        >950 && <=1080
95 OSR2        1              4             <10           >1080
98             1              4              10            1998
98 SP1         1              4              10       >1998 && <2183
98 SE          1              4              10          >=2183
ME             1              4              90            3000

NT 3.51        2              3              51
NT 4           2              4               0            1381
2000           2              5               0            2195
XP             2              5               1            2600
2003 Server    2              5               2            3790

CE             3
*/

#define WUNKNOWNSTR		_T("unknown Windows version")

#define W95STR			_T("Windows 95")
#define W95SP1STR		_T("Windows 95 SP1")
#define W95OSR2STR		_T("Windows 95 OSR2")
#define W98STR			_T("Windows 98")
#define W98SP1STR		_T("Windows 98 SP1")
#define W98SESTR		_T("Windows 98 SE")
#define WMESTR			_T("Windows ME")

#define WNT351STR		_T("Windows NT 3.51")
#define WNT4STR			_T("Windows NT 4")
#define W2KSTR			_T("Windows 2000")
#define WXPSTR			_T("Windows XP")
#define W2003SERVERSTR	_T("Windows 2003 Server")

#define WCESTR			_T("Windows CE")


//CWinVer()
CWinVer::CWinVer() throw()
{
	OSVERSIONINFO osinfo;
	osinfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	if (!GetVersionEx(&osinfo))
		throw GetLastError();

	m_dwPlatformId   = osinfo.dwPlatformId;
	m_dwMinorVersion = osinfo.dwMinorVersion;
	m_dwMajorVersion = osinfo.dwMajorVersion;
	m_dwBuildNumber  = osinfo.dwBuildNumber & 0xFFFF;	// Win 95 needs this

	sprintf(m_sDotVersion, "%u.%u.%u", m_dwMajorVersion, m_dwMinorVersion, m_dwBuildNumber);


	if ((m_dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) && (m_dwMajorVersion == 4))
	{
		if ((m_dwMinorVersion < 10) && (m_dwBuildNumber == 950))
			m_iVersion = W95;
		else if ((m_dwMinorVersion < 10) && ((m_dwBuildNumber > 950) && (m_dwBuildNumber <= 1080)))
			m_iVersion = W95SP1;
		else if ((m_dwMinorVersion < 10) && (m_dwBuildNumber > 1080))
			m_iVersion = W95OSR2;
		else if ((m_dwMinorVersion == 10) && (m_dwBuildNumber == 1998))
			m_iVersion = W98;
		else if ((m_dwMinorVersion == 10) && ((m_dwBuildNumber > 1998) && (m_dwBuildNumber < 2183)))
			m_iVersion = W98SP1;
		else if ((m_dwMinorVersion == 10) && (m_dwBuildNumber >= 2183))
			m_iVersion = W98SE;
		else if (m_dwMinorVersion == 90)
			m_iVersion = WME;
	}
	else if (m_dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		if ((m_dwMajorVersion == 3) && (m_dwMinorVersion == 51))
			m_iVersion = WNT351;
		else if ((m_dwMajorVersion == 4) && (m_dwMinorVersion == 0))
			m_iVersion = WNT4;
		else if ((m_dwMajorVersion == 5) && (m_dwMinorVersion == 0))
			m_iVersion = W2K;
		else if ((m_dwMajorVersion == 5) && (m_dwMinorVersion == 1))
			m_iVersion = WXP;
		else if ((m_dwMajorVersion == 5) && (m_dwMinorVersion == 2))
			m_iVersion = W2003SERVER;
	}
	else if (m_dwPlatformId == VER_PLATFORM_WIN32_CE)
	{
		m_iVersion = WCE;
	}
}

int		CWinVer::GetVersion()
{
	return m_iVersion;
}

PCTSTR	CWinVer::GetVersionStr()
{
	#define CASE_VER(x)	\
		case x:			\
			return x##STR;
		
	switch (m_iVersion)
	{
	CASE_VER(W95)
	CASE_VER(W95SP1)
	CASE_VER(W95OSR2)
	CASE_VER(W98)
	CASE_VER(W98SP1)
	CASE_VER(W98SE)
	CASE_VER(WME)
	CASE_VER(WNT351)
	CASE_VER(WNT4)
	CASE_VER(W2K)
	CASE_VER(WXP)
	CASE_VER(W2003SERVER)
	default:
		return WUNKNOWNSTR;
	}
	#undef CASE_VER
}

PCTSTR	CWinVer::GetDotVersion()
{
	return m_sDotVersion;
}