#ifndef __MODULEVER__
#define __MODULEVER__

#include <shlwapi.h>
#include <tchar.h>

#define W95			1
#define W95SP1		2
#define W95OSR2		3
#define W98			4
#define W98SP1		5
#define W98SE		6
#define WME			7

#define WNT351		101
#define WNT4		102
#define W2K			103
#define WXP			104
#define W2003SERVER	105

#define WCE			201

#define HIDWORD(x)	(x >> 32)
#define LODWORD(x)	(x & 0xFFFFFFFF)
#define MAKEQWORD(lo,hi)	((QWORD)hi<<32 | lo)

typedef unsigned __int64 QWORD;

//based on code GetWinVer by Hans Dietrich
class CWinVer
{
private:
	//system GetWinVer mirror
	DWORD	m_dwPlatformId;
	DWORD	m_dwMinorVersion;
	DWORD	m_dwMajorVersion;
	DWORD	m_dwBuildNumber;

	//own
	int		m_iVersion;
	char	m_sDotVersion[64];

public:
	CWinVer() throw();
	
	int		GetVersion();
	PCTSTR	GetDotVersion();
	PCTSTR	GetVersionStr();

	__declspec(property(get=GetVersion))			int		Version;
	__declspec(property(get=GetDotVersion))			PCTSTR	DotVersion;
	__declspec(property(get=GetVersionStr))			PCTSTR	VersionStr;
};


//class CModuleVersion
class CModuleVersion
{
	typedef	struct tagTRANSLATION {
		WORD langID;         // language ID
		WORD charset;        // character set (code page)
	} TRANSLATION, *PTRANSLATION;

private:
	PVOID				pVersionInfo;   // all version info
	VS_FIXEDFILEINFO	vsFileInfo;
	TRANSLATION			tTransl;
	BOOL				bOk;

protected:
	BOOL	Init(PCSTR	pFile);
	void	Destroy();

public:
	inline  CModuleVersion(PCSTR pFile);
	inline ~CModuleVersion();

	inline	BOOL	operator!()		const	{ return !bOk;		}
	inline	operator BOOL()			const	{ return bOk;		}


	PCSTR	GetValue(PCSTR pKeyName);

	inline	QWORD	GetFileVersion()	{ return MAKEQWORD(vsFileInfo.dwFileVersionLS,		vsFileInfo.dwFileVersionMS);	}
	inline	QWORD	GetProductVersion()	{ return MAKEQWORD(vsFileInfo.dwProductVersionLS,	vsFileInfo.dwProductVersionMS);	}
	inline	DWORD	GetType()			{ return vsFileInfo.dwFileType;		}
	inline	DWORD	GetOS()				{ return vsFileInfo.dwFileOS;		}
	inline	DWORD	GetFlags()			{ return vsFileInfo.dwFileFlags;	}

	__declspec(property(get=GetFileVersion))	QWORD	FileVersion;
	__declspec(property(get=GetProductVersion))	QWORD	ProductVersion;
	__declspec(property(get=GetType))			DWORD	Type;
	__declspec(property(get=GetOS))				DWORD	OS;
	__declspec(property(get=GetFlags))			DWORD	Flags;

};

//CModuleVersion(PCSTR pFile)
inline  CModuleVersion::CModuleVersion(PCSTR pFile)
	:pVersionInfo(0),bOk(FALSE)
{
	bOk = Init(pFile);
	if (!bOk)
		Destroy();
}

//~CModuleVersion()
inline	CModuleVersion::~CModuleVersion()
{
	Destroy();
}


//class CDLLVersion
class CDLLVersion
{
private:
	HMODULE				hDLL;
	BOOL				bFreeAtExit;
	DLLVERSIONINFO2		dvDVI;
	BOOL				bOk;

protected:
	BOOL	Init(HMODULE hDLL); 
	BOOL	Init(PCSTR	pFile);
	void	Destroy();

public:
	inline  CDLLVersion(HMODULE hDLL);
	inline  CDLLVersion(PCSTR pFile);
	inline ~CDLLVersion();

	inline	BOOL	operator!()		const	{ return !bOk;		}
	inline	operator BOOL()			const	{ return bOk;		}

	inline	DWORD	GetMajor()		{ return dvDVI.info1.dwMajorVersion;		}
	inline	DWORD	GetMinor()		{ return dvDVI.info1.dwMinorVersion;		}
	inline	DWORD	GetBuild()		{ return (DWORD)dvDVI.ullVersion & 0xFFFF;	}
	inline	DWORD	GetRelease()	{ return dvDVI.info1.dwBuildNumber;			}
	
	inline	BOOL	IsNT()			{ return dvDVI.info1.dwPlatformID == DLLVER_PLATFORM_NT;}

	__declspec(property(get=GetMajor))		DWORD	Major;
	__declspec(property(get=GetMinor))		DWORD	Minor;
	__declspec(property(get=GetRelease))	DWORD	Release;
	__declspec(property(get=GetBuild))		DWORD	Build;
};

//CDLLVersion(HMODULE hDLL)
inline  CDLLVersion::CDLLVersion(HMODULE hDLL)
	:hDLL(0), bFreeAtExit(0), bOk(0)
{
	ZeroMemory(&dvDVI,sizeof(dvDVI));
	bOk = Init(hDLL);
}

//CDLLVersion(PCSTR pFile)
inline  CDLLVersion::CDLLVersion(PCSTR pFile)
	:hDLL(0), bFreeAtExit(0), bOk(0)
{
	ZeroMemory(&dvDVI,sizeof(dvDVI));
	bOk = Init(pFile);
}

//~CDLLVersion()
inline	CDLLVersion::~CDLLVersion()
{
	Destroy();
}



#endif
