#include "cpuid.h"
#include <string.h>
#include <stdio.h>

extern char* ProcessorFeaturesDesc[];
extern FEATURES_INDEX Intel_Features[];
extern FEATURES_INDEX Intel_FeaturesEx[];
extern FEATURES_INDEX AMD_Features[];
extern FEATURES_INDEX AMD_FeaturesEx[];

void __stdcall EnumFeatures(bool, LPCPUID_FN_ENUM, void *);

int __stdcall IsIntel()
{
	char szVendorID[CPUID_VENDOR_ID_LENGTH + 1];
	CPUID_GetVendorID(szVendorID);
	return strcmp(szVendorID, CPUID_ID_INTEL) ? 0 : 1;
}

int __stdcall IsAMD()
{
	char szVendorID[CPUID_VENDOR_ID_LENGTH + 1];
	CPUID_GetVendorID(szVendorID);
	return strcmp(szVendorID, CPUID_ID_AMD) ? 0 : 1;
}

void __stdcall GetProcessorType(char *szProcessorType)
{
	PROCESSOR_SIGNATURE psig;
	PROCESSOR_MISC_INFO pmisc;
	if (CPUID_GetMaxFuncNumberEx() >= 0x80000004)
	{
		char szTemp[CPUID_BRAND_STR_LENGTH];
		CPUID_GetBrandString(szTemp);
		char *p = szTemp, *q = szTemp + strlen(szTemp);
		while (*p == ' ' && p < q) p++;
		while (*(q - 1) == ' ' && q > p) q--;
		*q = 0;
		strcpy(szProcessorType, szTemp);
		return;
	}
	if (IsIntel())
	{
		CPUID_GetProcessorMiscInfo(&pmisc);
		CPUID_GetProcessorSignature(&psig);
		unsigned long sig = *(unsigned long *)&psig;
		if (pmisc.BrandID)
		{
			switch (pmisc.BrandID)
			{
				case 0x01:
					strcpy(szProcessorType, "Intel Celeron");
					break;
				case 0x02:
					strcpy(szProcessorType, "Intel Pentium III");
					break;
				case 0x03:
					if (sig == 0x000006B1)
						strcpy(szProcessorType, "Intel Pentium III Xeon");
					else
						strcpy(szProcessorType, "Intel Celeron");
					break;
				case 0x04:
					strcpy(szProcessorType, "Intel Pentium III");
					break;
				case 0x06:
					strcpy(szProcessorType, "Mobile Intel Pentium III Processor-M");
					break;
				case 0x07:
					strcpy(szProcessorType, "Mobile Intel Celeron");
					break;
				case 0x08:
					if (sig >= 0x00000F13)
						strcpy(szProcessorType, "Intel Genuine");
					else
						strcpy(szProcessorType, "Intel Pentium 4");
					break;
				case 0x09:
					strcpy(szProcessorType, "Intel Pentium 4");
					break;
				case 0x0A:
					strcpy(szProcessorType, "Intel Celeron");
					break;
				case 0x0B:
					if (sig < 0x00000F13)
						strcpy(szProcessorType, "Intel Xeon MP");
					else
						strcpy(szProcessorType, "Intel Xeon");
					break;
				case 0x0C:
					strcpy(szProcessorType, "Intel Xeon MP");
					break;
				case 0x0E:
					if (sig < 0x00000F13)
						strcpy(szProcessorType, "Intel Xeon");
					else
						strcpy(szProcessorType, "Mobile Intel Pentium 4 Processor-M");
					break;
				case 0x0F:
					strcpy(szProcessorType, "Mobile Intel Celeron");
			}
		}
		else
		{
			GetProcessorTypeBySignature(sig, szProcessorType);
			if (psig.ProcessorType < 3)
			{
				strcat(szProcessorType, " (");
				switch (psig.ProcessorType)
				{
					case 0:
						strcat(szProcessorType, "Original OEM processor");
						break;
					case 1:
						strcat(szProcessorType, "OverDrive processor");
						break;
					case 2:
						strcat(szProcessorType, "Dual processor");
				}
				strcat(szProcessorType, ")");
			}
		}
	}
	else if (IsAMD())
	{
		CPUID_GetProcessorSignature(&psig);
		unsigned long sig = *(unsigned long *)&psig;
		GetProcessorTypeBySignature(sig, szProcessorType);
	}
	else
	{
		strcpy(szProcessorType, "");
	}
}

void __stdcall GetProcessorTypeBySignature(unsigned long Signature, char *szProcessorType)
{
	int i = 0, k;
	PCPUTYPE CPUs = NULL;
	if (IsIntel())
	{
		CPUs = Intel_CPUs;
		k = KNOWN_INTEL_CPUS;
	}
	else if (IsAMD())
	{
		CPUs = AMD_CPUs;
		k = KNOWN_AMD_CPUS;
	}
	else
	{
		strcpy(szProcessorType, "");
		return;
	}
	for (; i < k; i++)
		if ((Signature & CPUs[i].Mask) == CPUs[i].Signature)
		{
			strcpy(szProcessorType, CPUs[i].CPUName);
			return;
		}
	strcpy(szProcessorType, "");
}

void __stdcall EnumFeatures(bool ex, LPCPUID_FN_ENUM lpFn, void *lpParam)
{
	unsigned long Features = CPUID_GetProcessorFeatures(NULL);
	PFEATURES_INDEX index;
	int i = 0, k;
	if (IsIntel())
	{
		Features &= CPUID_INTEL_MASK;
		if (!ex)
		{
			k = KNOWN_INTEL_FEATURES;
			index = Intel_Features;
		}
		else
		{
			k = KNOWN_INTEL_FEATURES_EX;
			index = Intel_FeaturesEx;
		}
	}
	else if (IsAMD())
	{
		Features &= CPUID_AMD_OPTERON_MASK;
		if (!ex)
		{
			k = KNOWN_AMD_FEATURES;
			index = AMD_Features;
		}
		else
		{
			k = KNOWN_AMD_FEATURES_EX;
			index = AMD_FeaturesEx;
		}
	}
	for (; i < k; i++)
	{
		if (Features & index[i].Mask)
			if (!lpFn(ProcessorFeaturesDesc[index[i].Index], lpParam))
				return;
	}
}

void __stdcall EnumProcessorFeatures(LPCPUID_FN_ENUM lpFn, char *lpParam)
{
	EnumFeatures(false, lpFn, lpParam);
}

void __stdcall EnumProcessorFeaturesEx(LPCPUID_FN_ENUM lpFn, void *lpParam)
{
	EnumFeatures(true, lpFn, lpParam);
}

void __stdcall EnumProcessorFeaturesAll(LPCPUID_FN_ENUM lpFn, void *lpParam)
{
	EnumFeatures(false, lpFn, lpParam);
	EnumFeatures(true, lpFn, lpParam);
}

void __stdcall EnumProcessorCacheInfo(LPCPUID_FN_ENUM lpFn, void *lpParam)
{
	int i, j, k;
	if (IsIntel())
	{
		unsigned char buf[256];
		CPUID_GetIntelCacheInfo(buf, &k);
		for (i = 0; i < k; i++)
		{
			for (j = 0; j < KNOWN_INTEL_TLB_DESC; j++)
			{
				if (Intel_TLB[j].Val == buf[i])
				{
					lpFn(Intel_TLB[j].Description, lpParam);
					break;
				}
			}
		}
	}
	else if (IsAMD())
	{
		char szBuf[256];
		AMD_CACHE_TLB PageTlb1;
		AMD_CACHE_TLB PageTlb2;
		AMD_CACHE_INFO PageInfo1;
		AMD_CACHE_INFO PageInfo2;
		unsigned int n = CPUID_GetMaxFuncNumberEx();
		if (n >= 0x80000005)
		{
			CPUID_GetAMDL1CacheInfo(&PageTlb1, &PageTlb2, &PageInfo1, &PageInfo2);
			sprintf(szBuf, "L1 Cache Info:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "    2-Mbyte/4-Mbyte Pages:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Data TLB:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Entries: %d", (int)PageTlb1.DataTLB.Entries);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Associativity: %d", (int)PageTlb1.DataTLB.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Instruction TLB:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Entries: %d", (int)PageTlb1.InstructionTLB.Entries);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Associativity: %d", (int)PageTlb1.InstructionTLB.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "    4-Kbyte Pages:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Data TLB:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Entries: %d", (int)PageTlb2.DataTLB.Entries);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Associativity: %d", (int)PageTlb2.DataTLB.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Instruction TLB:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Entries: %d", (int)PageTlb2.InstructionTLB.Entries);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Associativity: %d", (int)PageTlb2.InstructionTLB.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "    L1 Data Cache:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Line Size: %d", (int)PageInfo1.LineSize);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Lines per Tag: %d", (int)PageInfo1.LinesPerTag);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Associativity: %d", (int)PageInfo1.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Size: %d Kbytes", (int)PageInfo1.Size);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "    L1 Instruction Cache:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Line Size: %d", (int)PageInfo2.LineSize);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Lines per Tag: %d", (int)PageInfo2.LinesPerTag);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Associativity: %d", (int)PageInfo2.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Size: %d Kbytes", (int)PageInfo2.Size);
			lpFn(szBuf, lpParam);
		}
		if (n >= 0x80000006)
		{
			CPUID_GetAMDL2CacheInfo(&PageTlb1, &PageTlb2, &PageInfo1, &PageInfo2);
			sprintf(szBuf, "L2 Cache Info:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "    2-Mbyte/4-Mbyte Pages:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Data TLB:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Entries: %d", (int)PageTlb1.DataTLB.Entries);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Associativity: %d", (int)PageTlb1.DataTLB.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Instruction TLB:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Entries: %d", (int)PageTlb1.InstructionTLB.Entries);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Associativity: %d", (int)PageTlb1.InstructionTLB.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "    4-Kbyte Pages:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Data TLB:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Entries: %d", (int)PageTlb2.DataTLB.Entries);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Associativity: %d", (int)PageTlb2.DataTLB.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Instruction TLB:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Entries: %d", (int)PageTlb2.InstructionTLB.Entries);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "            Associativity: %d", (int)PageTlb2.InstructionTLB.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "    L2 Data Cache:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Line Size: %d", (int)PageInfo1.LineSize);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Lines per Tag: %d", (int)PageInfo1.LinesPerTag);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Associativity: %d", (int)PageInfo1.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Size: %d Kbytes", (int)PageInfo1.Size);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "    L2 Instruction Cache:");
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Line Size: %d", (int)PageInfo2.LineSize);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Lines per Tag: %d", (int)PageInfo2.LinesPerTag);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Associativity: %d", (int)PageInfo2.Associativity);
			lpFn(szBuf, lpParam);
			sprintf(szBuf, "        Size: %d Kbytes", (int)PageInfo2.Size);
			lpFn(szBuf, lpParam);
		}
	}
}

void __stdcall EnumAMDPowerFeatures(LPCPUID_FN_ENUM lpFn, void *lpParam)
{
	unsigned int n = CPUID_GetMaxFuncNumberEx();
	if (n >= 0x80000007)
	{
		AMD_APM_FEATURES pfeatures;
		CPUID_GetAMDAPMFeatures(&pfeatures);
		printf("AMD APM features:\n");
		if (pfeatures.TS) lpFn("Temperature Sensor", lpParam);
		if (pfeatures.FID) lpFn("Frequency ID Control", lpParam);
		if (pfeatures.VID) lpFn("Voltage ID Control", lpParam);
		if (pfeatures.TTP) lpFn("Thermal Trip", lpParam);
		if (pfeatures.TM) lpFn("Thermal Monitoring", lpParam);
		if (pfeatures.STC) lpFn("Software Thermal Control", lpParam);
	}
}
