#ifndef _CPUID_H_
#define _CPUID_H_

#define CPUID_VENDOR_ID_LENGTH	12
#define CPUID_BRAND_STR_LENGTH	48
#define CPUID_ID_INTEL			"GenuineIntel"
#define CPUID_ID_AMD			"AuthenticAMD"
#define CPUID_MAX_CPUTYPE_LENGTH	256

#define CPUID_STD_FPU			0x00000001
#define CPUID_STD_VME			0x00000002
#define CPUID_STD_DEBUGEXT		0x00000004
#define CPUID_STD_4MPAGE		0x00000008
#define CPUID_STD_TSC			0x00000010
#define CPUID_STD_MSR			0x00000020
#define CPUID_STD_PAE			0x00000040
#define CPUID_STD_MCHKXCP		0x00000080
#define CPUID_STD_CMPXCHG8B		0x00000100
#define CPUID_STD_APIC			0x00000200
#define CPUID_STD_SYSENTER		0x00000800
#define CPUID_STD_MTRR			0x00001000
#define CPUID_STD_GPE			0x00002000
#define CPUID_STD_MCHKARCH		0x00004000
#define CPUID_STD_CMOV			0x00008000
#define CPUID_STD_PAT			0x00010000
#define CPUID_STD_PSE36			0x00020000
#define CPUID_STD_CLFSH			0x00080000
#define CPUID_STD_MMX			0x00800000
#define CPUID_STD_FXSAVE		0x01000000
#define CPUID_STD_SSE			0x02000000
#define CPUID_STD_SSE2			0x04000000

#define CPUID_STD_INTEL_PSN		0x00040000
#define CPUID_STD_INTEL_DS		0x00200000
#define CPUID_STD_INTEL_ACPI	0x00400000
#define CPUID_STD_INTEL_SS		0x08000000
#define CPUID_STD_INTEL_HTT		0x10000000
#define CPUID_STD_INTEL_TM		0x20000000
#define CPUID_STD_INTEL_SBF		0x80000000

#define CPUID_EXT_AMD_CPUID		0x00000001
#define CPUID_EXT_AMD_TSC		0x00000010
#define CPUID_EXT_AMD_MMX		0x00000020
#define CPUID_EXT_AMD_CMOV		0x00000040
#define CPUID_EXT_AMD_3DNOW		0x00000080
#define CPUID_EXT_AMD_3DNOWEXT	0x00000100
#define CPUID_EXT_AMD_MMXEXT	0x00000200
#define CPUID_EXT_AMD_SSEFP		0x00000400
#define CPUID_EXT_AMD_K6_MTRR	0x00000800
#define CPUID_EXT_AMD_P6_MTRR	0x00001000

#define CPUID_EXT_INTEL_TM2		0x00000080
#define CPUID_EXT_INTEL_EST		0x00000100
#define CPUID_EXT_INTEL_CID		0x00000400

#define CPUID_INTEL_MASK		0xBFEFFBFF
#define CPUID_AMD_OPTERON_MASK	0x078BFBFF
#define CPUID_AMD_ATHLON_MASK	0x0383FBFF
#define CPUID_AMD_DURON_MASK	0x0383FBFF
#define CPUID_AMD_K6_MASK		0x0183FBFF

#define KNOWN_INTEL_CPUS		29
#define KNOWN_INTEL_FEATURES	29
#define KNOWN_INTEL_FEATURES_EX	3
#define KNOWN_INTEL_TLB_DESC	47
#define KNOWN_AMD_CPUS			28
#define KNOWN_AMD_FEATURES		22
#define KNOWN_AMD_FEATURES_EX	10

typedef struct tagPROCESSORSIGNATURE {
	unsigned SteppingID:4;
	unsigned ModelNumber:4;
	unsigned FamilyCode:4;
	unsigned ProcessorType:2;
	unsigned Reserved1:2;
	unsigned ExtendedModel:4;
	unsigned ExtendedFamily:8;
	unsigned Reserved2:4;
} PROCESSOR_SIGNATURE, *PPROCESSOR_SIGNATURE;

typedef struct tagPROCESSORMISCINFO {
	unsigned BrandID:8;
	unsigned CacheLineSize:8;
	unsigned Reserved:8;
	unsigned APIC_ID:8;
} PROCESSOR_MISC_INFO, *PPROCESSOR_MISC_INFO;

typedef struct tagINTELFEATURES {
	bool FPU:1;
	bool VME:1;
	bool DE:1;
	bool PSE:1;
	bool TSC:1;
	bool MSR:1;
	bool PAE:1;
	bool MCE:1;
	bool CX8:1;
	bool APIC:1;
	bool Reserved1:1;
	bool SEP:1;
	bool MTRR:1;
	bool PGE:1;
	bool MCA:1;
	bool CMOV:1;
	bool PAT:1;
	bool PSE_36:1;
	bool PSN:1;
	bool CLFSH:1;
	bool Reserved2:1;
	bool DS:1;
	bool ACPI:1;
	bool MMX:1;
	bool FXSR:1;
	bool SSE:1;
	bool SSE2:1;
	bool SS:1;
	bool HTT:1;
	bool TM:1;
	bool Reserved3:1;
	bool SBF:1;
} INTEL_FEATURES, *PINTEL_FEATURES;

typedef struct tagAMDFEATURES {
	bool FPU:1;
	bool VME:1;
	bool DE:1;
	bool PSE:1;
	bool TSC:1;
	bool MSR:1;
	bool PAE:1;
	bool MCE:1;
	bool CX8:1;
	bool APIC:1;
	bool Reserved1:1;
	bool SEP:1;
	bool MTRR:1;
	bool PGE:1;
	bool MCA:1;
	bool CMOV:1;
	bool PAT:1;
	bool PSE_36:1;
	bool Reserved2:1;
	bool CLFSH:1;
	bool Reserved3:3;
	bool MMX:1;
	bool FXSR:1;
	bool SSE:1;
	bool SSE2:1;
	bool Reserved4:5;
} AMD_FEATURES, *PAMD_FEATURES;

typedef struct tagFEATURESINDEX {
	unsigned long Mask;
	unsigned char Index;
} FEATURES_INDEX, *PFEATURES_INDEX;

typedef struct tagPROCESSORSN {
	unsigned short Part1;
	unsigned short Part2;
	unsigned short Part3;
	unsigned short Part4;
	unsigned short Part5;
	unsigned short Part6;
} PROCESSOR_SN, *PPROCESSOR_SN;

typedef struct tagCPUTYPE {
	unsigned long Signature;
	unsigned long Mask;
	char *CPUName;
} CPUTYPE, *PCPUTYPE;

typedef struct tagINTELTLBDESCRIPTOR {
	unsigned char Val;
	char *Description;
} INTEL_TLB_DESCRIPTOR, *PINTEL_TLB_DESCRIPTOR;

typedef struct tagAMDTLB {
	unsigned Entries:8;
	unsigned Associativity:8;
} AMD_TLB, *PAMD_TLB;

typedef struct tagAMDCACHETLB {
	AMD_TLB InstructionTLB;
	AMD_TLB DataTLB;
} AMD_CACHE_TLB, *PAMD_CACHE_TLB;

typedef struct tagAMDCACHEINFO {
	unsigned LineSize:8;
	unsigned LinesPerTag:8;
	unsigned Associativity:8;
	unsigned Size:8;
} AMD_CACHE_INFO, *PAMD_CACHE_INFO;

typedef struct tagAMDAPMFEATURES {
	bool TS:1;
	bool FID:1;
	bool VID:1;
	bool TTP:1;
	bool TM:1;
	bool STC:1;
	long Reserved:26;
} AMD_APM_FEATURES, *PAMD_APM_FEATURES;

typedef struct tagAMDADDRESSRANGEINFO {
	unsigned MaxPhysAddrWidth:8;
	unsigned MaxVirtAddrWidth:8;
	unsigned Reserved:16;
} AMD_ADDRESS_RANGE_INFO, *PAMD_ADDRESS_RANGE_INFO;

extern CPUTYPE AMD_CPUs[];
extern CPUTYPE Intel_CPUs[];
extern INTEL_TLB_DESCRIPTOR Intel_TLB[];

typedef int (__stdcall *PCPUIDFNENUM)(char *lpszFeatureName, void *lpParam);
typedef PCPUIDFNENUM LPCPUID_FN_ENUM;

extern "C" {
int __stdcall CPUID_IsAvailable();
unsigned long __stdcall CPUID_GetMaxFuncNumber();
unsigned long __stdcall CPUID_GetMaxFuncNumberEx();
void __stdcall CPUID_GetVendorID(char *lpszVendorID);
void __stdcall CPUID_GetProcessorMiscInfo(PPROCESSOR_MISC_INFO pMiscInfo);
void __stdcall CPUID_GetBrandString(char *lpszBrandString);
void __stdcall CPUID_GetProcessorSignature(PPROCESSOR_SIGNATURE pSignature);
unsigned long __stdcall CPUID_GetProcessorFeatures(void *lpFeatures);
unsigned long __stdcall CPUID_GetProcessorFeaturesEx(void *lpFeaturesEx);
void __stdcall CPUID_GetSerialNumber(PPROCESSOR_SN pSerial);
void __stdcall CPUID_GetIntelCacheInfo(unsigned char *, int *);
void __stdcall CPUID_GetAMDL1CacheInfo(PAMD_CACHE_TLB, PAMD_CACHE_TLB,
							 PAMD_CACHE_INFO, PAMD_CACHE_INFO);
void __stdcall CPUID_GetAMDL2CacheInfo(PAMD_CACHE_TLB, PAMD_CACHE_TLB,
							 PAMD_CACHE_INFO, PAMD_CACHE_INFO);
void __stdcall CPUID_GetAMDAPMFeatures(PAMD_APM_FEATURES pFeatures);
void __stdcall CPUID_GetAMDAddressRangeInfo(PAMD_ADDRESS_RANGE_INFO pAddrRangeInfo);
};

int __stdcall IsIntel();
int __stdcall IsAMD();
void __stdcall GetProcessorType(char*);
void __stdcall GetProcessorTypeBySignature(unsigned long, char*);
void __stdcall EnumProcessorFeatures(LPCPUID_FN_ENUM lpFn, void *lpParam);
void __stdcall EnumProcessorFeaturesEx(LPCPUID_FN_ENUM lpFn, void *lpParam);
void __stdcall EnumProcessorFeaturesAll(LPCPUID_FN_ENUM lpFn, void *lpParam);
void __stdcall EnumProcessorCacheInfo(LPCPUID_FN_ENUM lpFn, void *lpParam);
void __stdcall EnumAMDPowerFeatures(LPCPUID_FN_ENUM lpFn, void *lpParam);

#endif // _CPUID_H_
